/*
 * This file is part of staq.
 *
 * Copyright (c) 2019 - 2024 softwareQ Inc. All rights reserved.
 *
 * MIT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef GRID_SYNTH_S3_TABLE_HPP_
#define GRID_SYNTH_S3_TABLE_HPP_

#include <string>

#include "staq/grid_synth/matrix.hpp"
#include "staq/grid_synth/rings.hpp"

namespace staq {
namespace grid_synth {

static const int S3_TABLE_SIZE = 3712;

// clang-format off
static const int S3_TABLE_KEYS[S3_TABLE_SIZE][10] = 
   {{0,0,-1,0,-1,1,1,0,2,4},
    {0,0,0,-1,0,-1,1,1,2,2},
    {0,0,0,1,0,1,-1,-1,2,2},
    {0,0,0,-1,0,-1,1,1,2,1},
    {1,0,0,0,-1,0,-1,1,2,7},
    {0,1,0,0,-1,-1,0,-1,2,5},
    {0,0,1,0,1,-1,-1,0,2,3},
    {0,-1,0,0,1,1,0,1,2,4},
    {0,0,-1,0,-1,1,1,0,2,2},
    {0,0,0,-1,0,-1,1,1,2,0},
    {0,1,0,0,-1,-1,0,-1,2,4},
    {0,0,1,0,1,-1,-1,0,2,2},
    {0,0,1,0,1,-1,-1,0,2,1},
    {0,0,0,1,0,1,-1,-1,2,7},
    {-1,0,0,0,1,0,1,-1,2,5},
    {0,-1,0,0,1,1,0,1,2,2},
    {0,0,-1,0,-1,1,1,0,2,0},
    {1,0,0,0,-1,0,-1,1,2,4},
    {0,1,0,0,-1,-1,0,-1,2,2},
    {-1,0,0,0,1,0,1,-1,2,4},
    {0,-1,0,0,1,1,0,1,2,1},
    {0,0,-1,0,-1,1,1,0,2,7},
    {0,0,1,0,1,-1,-1,0,2,7},
    {-1,0,0,0,1,0,1,-1,2,3},
    {0,0,-1,0,-1,1,1,0,2,6},
    {0,0,0,-1,0,-1,1,1,2,4},
    {0,0,1,0,1,-1,-1,0,2,6},
    {-1,0,0,0,1,0,1,-1,2,2},
    {0,0,-1,0,-1,1,1,0,2,5},
    {0,0,0,-1,0,-1,1,1,2,3},
    {0,1,0,0,-1,-1,0,-1,2,7},
    {0,0,1,0,1,-1,-1,0,2,5},
    {0,0,0,1,0,1,-1,-1,2,3},
    {0,0,1,0,-1,-1,1,0,2,0},
    {0,0,0,1,0,-1,-1,1,2,6},
    {0,1,0,0,-1,1,0,1,2,1},
    {0,0,0,1,0,-1,-1,1,2,5},
    {0,-1,0,0,1,-1,0,-1,2,1},
    {1,0,0,0,1,0,1,1,2,2},
    {0,0,1,0,-1,-1,1,0,2,6},
    {-1,0,0,0,-1,0,-1,-1,2,2},
    {0,-1,0,0,1,-1,0,-1,2,0},
    {0,0,-1,0,1,1,-1,0,2,7},
    {1,0,0,0,1,0,1,1,2,1},
    {0,1,0,0,-1,1,0,1,2,7},
    {-1,0,0,0,-1,0,-1,-1,2,1},
    {0,0,-1,0,1,1,-1,0,2,5},
    {1,0,0,0,1,0,1,1,2,0},
    {0,1,0,0,-1,1,0,1,2,6},
    {0,0,1,0,-1,-1,1,0,2,4},
    {-1,0,0,0,-1,0,-1,-1,2,0},
    {0,0,0,-1,0,1,1,-1,2,2},
    {1,0,0,0,1,0,1,1,2,7},
    {0,1,0,0,-1,1,0,1,2,5},
    {0,0,1,0,-1,-1,1,0,2,3},
    {0,0,0,1,0,-1,-1,1,2,1},
    {0,0,-1,0,1,1,-1,0,2,3},
    {1,0,0,0,1,0,1,1,2,6},
    {0,0,0,-1,0,1,1,-1,2,0},
    {1,0,0,0,1,0,1,1,2,5},
    {0,0,0,1,0,-1,-1,1,2,7},
    {0,-1,0,0,1,-1,0,-1,2,3},
    {0,0,-1,0,1,1,-1,0,2,1},
    {1,0,1,1,0,0,1,0,2,2},
    {-1,-1,1,0,-1,0,0,0,2,6},
    {-1,0,-1,-1,0,0,-1,0,2,2},
    {1,-1,0,-1,0,0,0,-1,2,0},
    {1,0,1,1,0,0,1,0,2,1},
    {-1,-1,1,0,-1,0,0,0,2,5},
    {0,-1,-1,1,0,-1,0,0,2,3},
    {1,-1,0,-1,0,0,0,-1,2,7},
    {0,1,1,-1,0,1,0,0,2,3},
    {1,0,1,1,0,0,1,0,2,0},
    {-1,1,0,1,0,0,0,1,2,6},
    {-1,-1,1,0,-1,0,0,0,2,4},
    {1,-1,0,-1,0,0,0,-1,2,6},
    {0,1,1,-1,0,1,0,0,2,2},
    {1,0,1,1,0,0,1,0,2,7},
    {-1,0,-1,-1,0,0,-1,0,2,7},
    {1,1,-1,0,1,0,0,0,2,3},
    {0,1,1,-1,0,1,0,0,2,1},
    {1,0,1,1,0,0,1,0,2,6},
    {-1,-1,1,0,-1,0,0,0,2,2},
    {-1,0,-1,-1,0,0,-1,0,2,6},
    {-1,1,0,1,0,0,0,1,2,3},
    {0,-1,-1,1,0,-1,0,0,2,7},
    {1,0,1,1,0,0,1,0,2,4},
    {-1,1,0,1,0,0,0,1,2,2},
    {-1,0,0,0,1,0,1,-1,2,6},
    {-1,0,-1,-1,0,0,-1,0,2,4},
    {-1,1,0,1,0,0,0,1,2,1},
    {-1,-1,1,0,-1,0,0,0,2,7},
    {0,-1,-1,1,0,-1,0,0,2,5},
    {-1,0,-1,-1,0,0,-1,0,2,3},
    {1,1,-1,0,1,0,0,0,2,7},
    {0,-1,1,1,0,1,0,0,2,0},
    {0,1,-1,-1,0,-1,0,0,2,0},
    {-1,0,-1,1,0,0,1,0,2,5},
    {1,0,1,-1,0,0,-1,0,2,5},
    {-1,1,1,0,1,0,0,0,2,1},
    {0,-1,1,1,0,1,0,0,2,6},
    {-1,-1,0,-1,0,0,0,1,2,2},
    {0,1,-1,-1,0,-1,0,0,2,6},
    {-1,1,1,0,1,0,0,0,2,0},
    {-1,0,-1,1,0,0,1,0,2,3},
    {-1,-1,0,-1,0,0,0,1,2,1},
    {0,1,-1,-1,0,-1,0,0,2,5},
    {-1,1,1,0,1,0,0,0,2,7},
    {1,-1,-1,0,-1,0,0,0,2,6},
    {-1,0,-1,1,0,0,1,0,2,1},
    {0,1,-1,-1,0,-1,0,0,2,3},
    {1,1,0,1,0,0,0,-1,2,7},
    {0,-1,1,1,0,1,0,0,2,2},
    {0,1,-1,-1,0,-1,0,0,2,2},
    {1,0,1,-1,0,0,-1,0,2,0},
    {-1,0,-1,1,0,0,1,0,2,7},
    {0,1,-1,-1,0,-1,0,0,2,1},
    {1,0,1,-1,0,0,-1,0,2,7},
    {-1,0,0,0,1,0,1,1,2,4},
    {0,-1,0,0,-1,1,0,1,2,2},
    {0,0,-1,0,-1,-1,1,0,2,0},
    {1,0,0,0,-1,0,-1,-1,2,4},
    {0,0,1,0,1,1,-1,0,2,0},
    {-1,0,0,0,1,0,1,1,2,3},
    {1,0,0,0,-1,0,-1,-1,2,3},
    {0,1,0,0,1,-1,0,-1,2,1},
    {0,-1,0,0,-1,1,0,1,2,0},
    {1,0,0,0,-1,0,-1,-1,2,2},
    {0,1,0,0,1,-1,0,-1,2,0},
    {0,0,1,0,1,1,-1,0,2,6},
    {0,0,0,-1,0,-1,-1,1,2,3},
    {1,0,0,0,-1,0,-1,-1,2,1},
    {0,1,0,0,1,-1,0,-1,2,7},
    {0,-1,0,0,-1,1,0,1,2,6},
    {0,0,-1,0,-1,-1,1,0,2,4},
    {0,0,0,-1,0,-1,-1,1,2,2},
    {1,0,0,0,-1,0,-1,-1,2,0},
    {0,-1,0,0,-1,1,0,1,2,5},
    {0,0,0,-1,0,-1,-1,1,2,1},
    {0,0,1,0,1,1,-1,0,2,3},
    {0,0,0,1,0,1,1,-1,2,1},
    {-1,0,0,0,1,0,1,1,2,6},
    {0,-1,0,0,-1,1,0,1,2,4},
    {0,0,-1,0,-1,-1,1,0,2,2},
    {0,1,0,0,1,-1,0,-1,2,4},
    {0,0,1,0,1,1,-1,0,2,2},
    {0,0,0,1,0,1,1,-1,2,0},
    {0,-1,0,0,-1,1,0,1,2,3},
    {0,0,-1,0,-1,-1,1,0,2,1},
    {0,0,0,-1,0,-1,-1,1,2,7},
    {1,0,0,0,-1,0,-1,-1,2,5},
    {0,1,0,0,1,-1,0,-1,2,3},
    {0,0,0,1,0,1,1,-1,2,7},
    {0,-1,0,0,-1,-1,0,-1,2,6},
    {0,0,1,0,-1,1,1,0,2,4},
    {-1,0,0,0,-1,0,-1,1,2,7},
    {0,-1,0,0,-1,-1,0,-1,2,5},
    {0,0,-1,0,1,-1,-1,0,2,3},
    {1,0,0,0,1,0,1,-1,2,7},
    {0,1,0,0,1,1,0,1,2,5},
    {0,0,1,0,-1,1,1,0,2,3},
    {0,0,-1,0,1,-1,-1,0,2,2},
    {0,0,0,-1,0,1,-1,-1,2,0},
    {1,0,0,0,1,0,1,-1,2,6},
    {0,1,0,0,1,1,0,1,2,4},
    {0,0,1,0,-1,1,1,0,2,2},
    {-1,0,0,0,-1,0,-1,1,2,5},
    {0,-1,0,0,-1,-1,0,-1,2,3},
    {0,1,0,0,1,1,0,1,2,3},
    {0,0,1,0,-1,1,1,0,2,1},
    {-1,0,0,0,-1,0,-1,1,2,4},
    {0,-1,0,0,-1,-1,0,-1,2,2},
    {0,0,0,-1,0,1,-1,-1,2,6},
    {1,0,0,0,1,0,1,-1,2,4},
    {0,-1,0,0,1,1,0,1,2,7},
    {0,1,0,0,1,1,0,1,2,2},
    {0,0,0,1,0,-1,1,1,2,5},
    {-1,0,0,0,-1,0,-1,1,2,3},
    {0,-1,0,0,-1,-1,0,-1,2,1},
    {0,0,-1,0,1,-1,-1,0,2,7},
    {1,0,0,0,1,0,1,-1,2,3},
    {0,0,0,-1,0,1,-1,-1,2,4},
    {1,0,0,0,1,0,1,-1,2,2},
    {0,0,-1,0,1,1,-1,0,2,0},
    {0,0,1,0,-1,1,1,0,2,6},
    {0,0,0,1,0,-1,1,1,2,3},
    {0,0,-1,0,1,-1,-1,0,2,5},
    {1,0,0,0,1,0,1,-1,2,1},
    {0,1,0,0,1,1,0,1,2,7},
    {0,0,1,0,-1,1,1,0,2,5},
    {-1,0,-1,1,0,0,-1,0,2,6},
    {1,-1,-1,0,1,0,0,0,2,2},
    {0,1,-1,-1,0,1,0,0,2,0},
    {1,0,1,-1,0,0,1,0,2,6},
    {1,1,0,1,0,0,0,1,2,4},
    {0,-1,1,1,0,-1,0,0,2,7},
    {-1,-1,0,-1,0,0,0,-1,2,3},
    {1,-1,-1,0,1,0,0,0,2,1},
    {1,1,0,1,0,0,0,1,2,3},
    {-1,-1,0,-1,0,0,0,-1,2,2},
    {1,0,1,-1,0,0,1,0,2,4},
    {-1,1,1,0,-1,0,0,0,2,0},
    {0,1,-1,-1,0,1,0,0,2,5},
    {1,0,1,-1,0,0,1,0,2,3},
    {0,-1,1,1,0,-1,0,0,2,4},
    {0,0,0,-1,0,1,1,-1,2,6},
    {1,-1,-1,0,1,0,0,0,2,6},
    {1,0,0,0,1,0,1,1,2,4},
    {1,0,1,-1,0,0,1,0,2,2},
    {-1,1,1,0,-1,0,0,0,2,6},
    {0,-1,1,1,0,-1,0,0,2,3},
    {0,1,-1,-1,0,1,0,0,2,3},
    {1,0,1,-1,0,0,1,0,2,1},
    {-1,1,1,0,-1,0,0,0,2,5},
    {-1,0,-1,1,0,0,-1,0,2,0},
    {-1,0,0,0,1,0,1,1,2,5},
    {-1,-1,0,-1,0,0,0,-1,2,6},
    {1,0,1,-1,0,0,1,0,2,0},
    {0,-1,1,1,0,-1,0,0,2,1},
    {0,1,-1,-1,0,1,0,0,2,1},
    {1,0,1,-1,0,0,1,0,2,7},
    {1,1,0,1,0,0,0,1,2,5},
    {1,-1,-1,0,1,0,0,0,2,5},
    {1,-1,0,-1,0,0,0,1,2,0},
    {0,1,1,-1,0,-1,0,0,2,4},
    {-1,1,0,1,0,0,0,-1,2,0},
    {0,-1,-1,1,0,1,0,0,2,3},
    {-1,0,-1,-1,0,0,1,0,2,1},
    {1,-1,0,-1,0,0,0,1,2,7},
    {0,1,1,-1,0,-1,0,0,2,3},
    {0,0,-1,0,-1,-1,1,0,2,7},
    {1,0,1,1,0,0,-1,0,2,1},
    {-1,-1,1,0,1,0,0,0,2,4},
    {1,1,-1,0,-1,0,0,0,2,4},
    {-1,-1,1,0,1,0,0,0,2,3},
    {0,-1,-1,1,0,1,0,0,2,1},
    {-1,0,-1,-1,0,0,1,0,2,7},
    {1,-1,0,-1,0,0,0,1,2,5},
    {0,1,1,-1,0,-1,0,0,2,1},
    {-1,-1,1,0,1,0,0,0,2,2},
    {0,-1,-1,1,0,1,0,0,2,0},
    {1,-1,0,-1,0,0,0,1,2,4},
    {1,0,1,1,0,0,-1,0,2,6},
    {-1,-1,1,0,1,0,0,0,2,1},
    {-1,0,-1,-1,0,0,1,0,2,5},
    {1,-1,0,-1,0,0,0,1,2,3},
    {1,1,-1,0,-1,0,0,0,2,1},
    {-1,1,0,1,0,0,0,-1,2,3},
    {-1,-1,1,0,1,0,0,0,2,0},
    {0,-1,-1,1,0,1,0,0,2,6},
    {1,-1,0,-1,0,0,0,1,2,2},
    {1,1,-1,0,-1,0,0,0,2,0},
    {-1,1,0,1,0,0,0,-1,2,2},
    {-1,-1,1,0,1,0,0,0,2,7},
    {1,1,-1,0,-1,0,0,0,2,7},
    {1,0,1,1,0,0,-1,0,2,3},
    {0,-1,0,0,-1,1,1,0,2,5},
    {0,1,0,0,1,-1,-1,0,2,5},
    {-1,0,-1,1,0,0,1,0,2,2},
    {0,0,1,0,0,1,-1,-1,2,3},
    {0,0,0,1,1,0,1,-1,2,1},
    {0,-1,0,0,-1,1,1,0,2,4},
    {0,0,-1,0,0,-1,1,1,2,2},
    {0,1,0,0,1,-1,-1,0,2,4},
    {-1,0,0,0,1,1,0,1,2,6},
    {1,-1,-1,0,-1,0,0,0,2,1},
    {0,-1,0,0,-1,1,1,0,2,3},
    {0,0,-1,0,0,-1,1,1,2,1},
    {0,0,0,-1,-1,0,-1,1,2,7},
    {0,1,0,0,-1,1,0,1,2,4},
    {0,0,0,1,1,0,1,-1,2,7},
    {-1,0,0,0,1,1,0,1,2,5},
    {0,1,0,0,-1,-1,0,-1,2,1},
    {0,0,-1,0,0,-1,1,1,2,0},
    {0,0,0,-1,-1,0,-1,1,2,6},
    {0,1,0,0,1,-1,-1,0,2,2},
    {-1,0,0,0,-1,0,-1,-1,2,3},
    {0,-1,0,0,-1,1,1,0,2,1},
    {1,0,0,0,-1,-1,0,-1,2,3},
    {0,1,0,0,1,-1,-1,0,2,1},
    {0,0,-1,0,0,-1,1,1,2,6},
    {1,0,0,0,-1,-1,0,-1,2,2},
    {0,0,1,0,0,1,-1,-1,2,6},
    {0,0,0,-1,-1,0,-1,1,2,3},
    {0,0,1,0,0,1,-1,-1,2,5},
    {0,0,0,1,1,0,1,-1,2,0},
    {-1,0,0,0,1,1,0,1,2,1},
    {0,-1,0,0,-1,1,1,0,2,6},
    {0,0,-1,0,0,-1,1,1,2,4},
    {0,0,0,-1,-1,0,-1,1,2,2},
    {1,0,0,0,-1,-1,0,-1,2,0},
    {0,0,1,0,0,1,-1,-1,2,4},
    {0,0,1,0,0,-1,-1,1,2,7},
    {0,0,0,1,-1,0,-1,-1,2,5},
    {-1,0,0,0,1,-1,0,-1,2,3},
    {0,0,0,-1,1,0,1,1,2,5},
    {0,1,0,0,-1,-1,1,0,2,0},
    {0,0,0,1,-1,0,-1,-1,2,4},
    {-1,0,0,0,1,-1,0,-1,2,2},
    {0,-1,0,0,1,1,-1,0,2,0},
    {0,0,0,-1,1,0,1,1,2,4},
    {0,1,0,0,-1,-1,1,0,2,7},
    {0,0,1,0,0,-1,-1,1,2,5},
    {-1,1,1,0,-1,0,0,0,2,4},
    {0,0,0,1,-1,0,-1,-1,2,3},
    {0,0,-1,0,0,1,1,-1,2,5},
    {0,0,1,0,0,-1,-1,1,2,4},
    {-1,0,0,0,1,-1,0,-1,2,0},
    {0,-1,0,0,1,1,-1,0,2,6},
    {0,0,-1,0,0,1,1,-1,2,4},
    {0,0,0,-1,1,0,1,1,2,2},
    {0,-1,0,0,1,-1,0,-1,2,2},
    {1,0,0,0,-1,1,0,1,2,7},
    {0,0,1,0,0,-1,-1,1,2,3},
    {0,0,0,1,-1,0,-1,-1,2,1},
    {-1,0,0,0,1,-1,0,-1,2,7},
    {0,-1,0,0,1,1,-1,0,2,5},
    {0,1,0,0,-1,-1,1,0,2,4},
    {0,0,1,0,0,-1,-1,1,2,2},
    {0,-1,0,0,1,1,-1,0,2,4},
    {0,0,0,-1,0,-1,-1,1,2,4},
    {0,0,-1,0,0,1,1,-1,2,2},
    {0,0,0,-1,1,0,1,1,2,0},
    {1,0,0,0,-1,1,0,1,2,2},
    {0,0,0,1,-1,0,-1,-1,2,7},
    {-1,0,0,0,1,-1,0,-1,2,5},
    {0,-1,0,0,1,1,-1,0,2,3},
    {0,0,-1,0,0,1,1,-1,2,1},
    {0,0,0,-1,1,0,1,1,2,7},
    {0,1,0,0,-1,-1,1,0,2,2},
    {0,0,1,0,0,-1,-1,1,2,0},
    {0,0,0,1,-1,0,-1,-1,2,6},
    {0,0,-1,0,1,1,-1,0,2,2},
    {-1,0,0,0,1,-1,0,-1,2,4},
    {0,0,0,-1,1,0,1,1,2,6},
    {-1,1,0,1,-1,0,0,0,2,7},
    {-1,-1,1,0,0,-1,0,0,2,5},
    {0,-1,-1,1,0,0,-1,0,2,3},
    {1,-1,0,-1,1,0,0,0,2,7},
    {0,1,1,-1,0,0,1,0,2,3},
    {1,0,1,1,0,0,0,1,2,0},
    {-1,-1,1,0,0,-1,0,0,2,4},
    {0,-1,-1,1,0,0,-1,0,2,2},
    {0,0,0,-1,0,-1,1,1,2,6},
    {-1,0,-1,-1,0,0,0,-1,2,0},
    {-1,-1,0,-1,0,0,0,1,2,5},
    {1,-1,0,-1,1,0,0,0,2,6},
    {0,1,1,-1,0,0,1,0,2,2},
    {1,0,0,0,-1,0,-1,1,2,5},
    {1,0,1,1,0,0,0,1,2,7},
    {-1,-1,1,0,0,-1,0,0,2,3},
    {0,-1,-1,1,0,0,-1,0,2,1},
    {-1,0,-1,-1,0,0,0,-1,2,7},
    {1,-1,0,-1,1,0,0,0,2,5},
    {0,1,1,-1,0,0,1,0,2,1},
    {1,0,1,1,0,0,0,1,2,6},
    {0,-1,-1,1,0,0,-1,0,2,0},
    {1,-1,0,-1,1,0,0,0,2,4},
    {1,1,-1,0,0,1,0,0,2,2},
    {0,1,1,-1,0,0,1,0,2,0},
    {1,0,1,1,0,0,0,1,2,5},
    {-1,0,-1,-1,0,0,0,-1,2,5},
    {-1,1,1,0,-1,0,0,0,2,2},
    {1,-1,0,-1,1,0,0,0,2,3},
    {1,1,-1,0,0,1,0,0,2,1},
    {0,-1,1,1,0,1,0,0,2,1},
    {0,1,1,-1,0,0,1,0,2,7},
    {-1,1,0,1,-1,0,0,0,2,2},
    {-1,-1,1,0,0,-1,0,0,2,0},
    {-1,0,-1,-1,0,0,0,-1,2,4},
    {0,0,0,-1,-1,0,-1,1,2,0},
    {1,-1,0,-1,1,0,0,0,2,2},
    {0,1,1,-1,0,0,1,0,2,6},
    {1,-1,-1,0,-1,0,0,0,2,0},
    {1,0,1,1,0,0,0,1,2,3},
    {1,-1,0,-1,0,0,0,1,2,6},
    {-1,0,-1,-1,0,0,0,-1,2,3},
    {1,0,1,1,0,0,0,1,2,2},
    {0,-1,-1,1,0,0,-1,0,2,4},
    {0,-1,1,1,0,0,1,0,2,7},
    {-1,-1,0,-1,-1,0,0,0,2,3},
    {1,-1,-1,0,0,-1,0,0,2,1},
    {-1,0,-1,1,0,0,0,1,2,4},
    {-1,-1,0,-1,-1,0,0,0,2,2},
    {1,0,1,-1,0,0,0,-1,2,4},
    {0,-1,0,0,-1,1,0,1,2,7},
    {1,1,0,1,1,0,0,0,2,2},
    {1,-1,-1,0,0,-1,0,0,2,7},
    {0,1,-1,-1,0,0,-1,0,2,5},
    {1,0,1,-1,0,0,0,-1,2,3},
    {-1,1,1,0,0,1,0,0,2,7},
    {-1,0,-1,1,0,0,0,1,2,2},
    {-1,-1,0,-1,-1,0,0,0,2,0},
    {1,-1,-1,0,0,-1,0,0,2,6},
    {0,1,-1,-1,0,0,-1,0,2,4},
    {1,0,0,0,1,0,1,-1,2,5},
    {-1,1,1,0,0,1,0,0,2,6},
    {0,1,-1,-1,0,0,-1,0,2,3},
    {0,-1,0,0,-1,1,1,0,2,7},
    {1,0,1,-1,0,0,0,-1,2,1},
    {1,-1,-1,0,1,0,0,0,2,4},
    {1,1,0,1,1,0,0,0,2,7},
    {-1,1,1,0,0,1,0,0,2,5},
    {0,-1,1,1,0,0,1,0,2,2},
    {-1,0,-1,1,0,0,0,1,2,0},
    {-1,-1,0,-1,-1,0,0,0,2,6},
    {1,-1,-1,0,0,-1,0,0,2,4},
    {0,1,-1,-1,0,0,-1,0,2,2},
    {1,1,0,1,1,0,0,0,2,6},
    {-1,1,1,0,0,1,0,0,2,4},
    {-1,0,-1,1,0,0,0,1,2,7},
    {-1,-1,0,-1,-1,0,0,0,2,5},
    {1,-1,-1,0,0,-1,0,0,2,3},
    {1,0,1,-1,0,0,0,-1,2,7},
    {-1,1,1,0,0,1,0,0,2,3},
    {-1,-1,0,-1,0,0,0,-1,2,4},
    {-1,0,-1,1,0,0,0,1,2,6},
    {1,-1,-1,0,0,-1,0,0,2,2},
    {0,-1,0,0,-1,-1,1,0,2,1},
    {0,0,-1,0,0,-1,-1,1,2,7},
    {1,0,0,0,1,-1,0,-1,2,3},
    {0,0,0,1,1,0,1,1,2,5},
    {0,-1,0,0,-1,-1,1,0,2,0},
    {0,0,0,-1,-1,0,-1,-1,2,4},
    {1,0,0,0,1,-1,0,-1,2,2},
    {0,0,1,0,0,1,1,-1,2,6},
    {0,0,0,1,1,0,1,1,2,4},
    {-1,0,0,0,-1,1,0,1,2,1},
    {0,0,0,-1,-1,0,-1,-1,2,3},
    {1,0,0,0,1,-1,0,-1,2,1},
    {0,1,0,0,1,1,-1,0,2,7},
    {0,0,1,0,0,1,1,-1,2,5},
    {0,-1,0,0,-1,-1,1,0,2,6},
    {0,1,0,0,1,1,-1,0,2,6},
    {0,0,1,0,0,1,1,-1,2,4},
    {0,0,0,1,-1,0,-1,-1,2,0},
    {-1,0,0,0,-1,1,0,1,2,7},
    {0,-1,0,0,-1,-1,1,0,2,5},
    {0,0,-1,0,0,-1,-1,1,2,3},
    {0,0,0,-1,-1,0,-1,-1,2,1},
    {1,0,0,0,-1,-1,0,-1,2,6},
    {1,0,0,0,1,-1,0,-1,2,7},
    {0,0,0,1,1,0,1,1,2,1},
    {-1,0,0,0,-1,1,0,1,2,6},
    {0,0,-1,0,0,-1,-1,1,2,2},
    {0,0,0,-1,-1,0,-1,-1,2,0},
    {1,0,0,0,1,-1,0,-1,2,6},
    {0,0,1,0,0,1,1,-1,2,2},
    {-1,0,0,0,-1,1,0,1,2,5},
    {0,-1,0,0,-1,-1,1,0,2,3},
    {0,0,0,-1,-1,0,-1,-1,2,7},
    {1,0,0,0,1,-1,0,-1,2,5},
    {0,0,1,0,0,1,1,-1,2,1},
    {0,0,0,1,1,0,1,1,2,7},
    {-1,0,0,0,-1,1,0,1,2,4},
    {1,0,0,0,1,-1,0,-1,2,4},
    {0,1,0,0,1,1,-1,0,2,2},
    {-1,0,0,0,1,1,0,1,2,3},
    {0,0,1,0,0,1,1,-1,2,0},
    {0,0,0,1,1,0,1,1,2,6},
    {0,0,-1,0,0,1,-1,-1,2,3},
    {0,0,0,-1,1,0,1,-1,2,1},
    {0,0,1,0,0,-1,1,1,2,3},
    {0,0,0,1,-1,0,-1,1,2,0},
    {-1,0,0,0,-1,-1,0,-1,2,6},
    {0,-1,0,0,1,-1,-1,0,2,4},
    {0,0,0,-1,1,0,1,-1,2,0},
    {1,0,0,0,1,1,0,1,2,6},
    {0,1,0,0,-1,1,1,0,2,4},
    {-1,0,0,0,-1,-1,0,-1,2,5},
    {0,0,0,-1,1,0,1,-1,2,7},
    {1,0,0,0,1,1,0,1,2,5},
    {0,0,1,0,0,-1,-1,1,2,1},
    {0,0,0,1,-1,0,-1,1,2,6},
    {-1,0,0,0,-1,-1,0,-1,2,4},
    {0,-1,0,0,1,-1,-1,0,2,2},
    {0,0,-1,0,0,1,-1,-1,2,0},
    {0,0,0,-1,1,0,1,-1,2,6},
    {0,1,0,0,-1,1,1,0,2,2},
    {-1,0,0,0,-1,-1,0,-1,2,3},
    {0,-1,0,0,1,-1,-1,0,2,1},
    {0,0,-1,0,0,1,-1,-1,2,7},
    {0,-1,-1,1,0,1,0,0,2,4},
    {0,0,0,-1,1,0,1,-1,2,5},
    {1,0,0,0,1,1,0,1,2,3},
    {0,1,0,0,-1,1,1,0,2,1},
    {0,0,1,0,0,-1,1,1,2,7},
    {0,0,0,1,-1,0,-1,1,2,4},
    {-1,0,0,0,-1,-1,0,-1,2,2},
    {0,-1,0,0,1,-1,-1,0,2,0},
    {0,0,-1,0,0,1,-1,-1,2,6},
    {0,0,0,-1,1,0,1,-1,2,4},
    {1,0,0,0,1,1,0,1,2,2},
    {0,1,0,0,-1,1,1,0,2,0},
    {0,0,1,0,0,-1,1,1,2,6},
    {0,-1,0,0,1,-1,-1,0,2,7},
    {0,0,-1,0,0,1,-1,-1,2,5},
    {0,1,0,0,-1,1,1,0,2,7},
    {0,0,1,0,0,-1,1,1,2,5},
    {0,0,0,1,-1,0,-1,1,2,2},
    {-1,0,0,0,-1,-1,0,-1,2,0},
    {0,-1,0,0,1,-1,-1,0,2,6},
    {0,0,-1,0,0,1,-1,-1,2,4},
    {0,0,0,-1,1,0,1,-1,2,2},
    {0,0,0,1,1,0,1,1,2,2},
    {1,0,0,0,1,1,0,1,2,0},
    {0,1,0,0,-1,1,1,0,2,6},
    {0,0,1,0,0,-1,1,1,2,4},
    {0,-1,1,1,0,0,-1,0,2,7},
    {0,0,0,-1,-1,0,-1,1,2,5},
    {-1,-1,0,-1,1,0,0,0,2,3},
    {1,-1,-1,0,0,1,0,0,2,1},
    {-1,1,1,0,0,-1,0,0,2,1},
    {-1,0,-1,1,0,0,0,-1,2,4},
    {-1,-1,0,-1,1,0,0,0,2,2},
    {0,1,-1,-1,0,0,1,0,2,6},
    {-1,-1,1,0,-1,0,0,0,2,1},
    {0,-1,1,1,0,0,-1,0,2,5},
    {0,-1,-1,1,0,0,-1,0,2,7},
    {-1,-1,0,-1,1,0,0,0,2,1},
    {1,-1,-1,0,0,1,0,0,2,7},
    {1,1,0,1,-1,0,0,0,2,1},
    {-1,1,1,0,0,-1,0,0,2,7},
    {-1,-1,0,-1,1,0,0,0,2,0},
    {1,-1,-1,0,0,1,0,0,2,6},
    {0,1,-1,-1,0,0,1,0,2,4},
    {0,-1,1,1,0,0,-1,0,2,3},
    {-1,-1,0,-1,1,0,0,0,2,7},
    {1,-1,-1,0,0,1,0,0,2,5},
    {-1,-1,1,0,0,-1,0,0,2,6},
    {1,1,0,1,-1,0,0,0,2,7},
    {-1,0,-1,1,0,0,0,-1,2,0},
    {-1,-1,0,-1,1,0,0,0,2,6},
    {1,1,0,1,-1,0,0,0,2,6},
    {0,-1,1,1,0,0,-1,0,2,1},
    {-1,-1,0,-1,1,0,0,0,2,5},
    {1,-1,-1,0,0,1,0,0,2,3},
    {0,1,0,0,1,-1,-1,0,2,0},
    {1,0,1,-1,0,0,0,1,2,7},
    {-1,1,1,0,0,-1,0,0,2,3},
    {-1,-1,0,-1,1,0,0,0,2,4},
    {1,-1,-1,0,0,1,0,0,2,2},
    {0,1,-1,-1,0,0,1,0,2,0},
    {1,1,0,1,0,0,0,-1,2,2},
    {1,0,1,-1,0,0,0,1,2,6},
    {1,1,0,1,-1,0,0,0,2,4},
    {-1,-1,1,0,0,1,0,0,2,5},
    {1,-1,0,-1,-1,0,0,0,2,7},
    {0,0,0,1,1,0,1,1,2,3},
    {1,1,-1,0,0,-1,0,0,2,5},
    {0,1,1,-1,0,0,-1,0,2,3},
    {1,0,1,1,0,0,0,-1,2,1},
    {-1,1,0,1,1,0,0,0,2,7},
    {0,-1,-1,1,0,0,1,0,2,2},
    {-1,0,-1,-1,0,0,0,1,2,0},
    {0,1,0,0,-1,1,1,0,2,3},
    {1,-1,0,-1,-1,0,0,0,2,6},
    {0,0,-1,0,1,-1,-1,0,2,1},
    {0,1,1,-1,0,0,-1,0,2,2},
    {1,0,1,1,0,0,0,-1,2,0},
    {-1,1,0,1,1,0,0,0,2,6},
    {0,-1,-1,1,0,0,1,0,2,1},
    {1,-1,0,-1,-1,0,0,0,2,5},
    {0,0,-1,0,-1,-1,1,0,2,3},
    {0,1,1,-1,0,0,-1,0,2,1},
    {1,0,1,1,0,0,0,-1,2,7},
    {-1,1,0,1,1,0,0,0,2,5},
    {-1,0,-1,-1,0,0,0,1,2,6},
    {0,1,1,-1,0,0,-1,0,2,0},
    {0,0,-1,0,0,-1,-1,1,2,5},
    {-1,1,1,0,0,-1,0,0,2,6},
    {-1,1,0,1,1,0,0,0,2,4},
    {-1,0,-1,-1,0,0,0,1,2,5},
    {1,-1,0,-1,-1,0,0,0,2,3},
    {0,1,1,-1,0,0,-1,0,2,7},
    {1,0,1,1,0,0,0,-1,2,5},
    {1,-1,0,-1,-1,0,0,0,2,2},
    {-1,1,1,0,1,0,0,0,2,6},
    {1,1,-1,0,0,-1,0,0,2,0},
    {0,1,1,-1,0,0,-1,0,2,6},
    {1,0,1,1,0,0,0,-1,2,4},
    {-1,-1,1,0,0,1,0,0,2,7},
    {-1,0,-1,-1,0,0,0,1,2,3},
    {1,-1,0,-1,-1,0,0,0,2,1},
    {1,0,1,1,0,0,0,-1,2,3},
    {-1,-1,1,0,0,1,0,0,2,6},
    {0,-1,-1,1,0,0,1,0,2,4},
    {1,-1,0,-1,-1,0,0,0,2,0},
    {1,1,-1,0,1,0,0,0,2,1},
    {0,1,1,-1,0,0,-1,0,2,4},
    {1,0,1,1,0,0,0,-1,2,2},
    {0,0,-1,0,-1,0,-1,1,2,2},
    {0,0,0,-1,-1,-1,0,-1,2,0},
    {0,0,0,1,1,1,0,1,2,0},
    {-1,0,0,0,-1,1,1,0,2,6},
    {0,0,-1,0,-1,0,-1,1,2,1},
    {0,0,0,-1,-1,-1,0,-1,2,7},
    {0,0,1,0,1,0,1,-1,2,1},
    {-1,-1,1,0,-1,0,0,0,2,3},
    {0,-1,0,0,0,-1,1,1,2,2},
    {0,0,-1,0,-1,0,-1,1,2,0},
    {1,0,1,1,0,0,-1,0,2,5},
    {0,1,0,0,0,1,-1,-1,2,2},
    {0,0,1,0,1,0,1,-1,2,0},
    {-1,0,-1,-1,0,0,-1,0,2,5},
    {-1,0,0,0,-1,1,1,0,2,4},
    {1,0,0,0,-1,1,0,1,2,3},
    {-1,1,0,1,1,0,0,0,2,3},
    {1,0,0,0,1,-1,-1,0,2,3},
    {0,0,-1,0,-1,0,-1,1,2,6},
    {0,0,0,-1,-1,-1,0,-1,2,4},
    {0,0,1,0,1,0,1,-1,2,6},
    {-1,0,0,0,-1,1,1,0,2,2},
    {0,-1,0,0,0,-1,1,1,2,7},
    {-1,0,0,0,-1,1,1,0,2,1},
    {1,0,0,0,1,-1,-1,0,2,0},
    {0,1,0,0,0,1,-1,-1,2,6},
    {0,0,0,1,0,-1,-1,1,2,3},
    {0,0,1,0,1,0,1,-1,2,4},
    {-1,0,0,0,-1,0,-1,-1,2,6},
    {0,0,0,1,1,1,0,1,2,2},
    {-1,0,0,0,-1,1,1,0,2,0},
    {0,-1,0,0,0,-1,1,1,2,5},
    {0,0,-1,0,-1,0,-1,1,2,3},
    {0,0,0,-1,0,-1,1,1,2,5},
    {0,1,0,0,0,1,-1,-1,2,7},
    {1,0,0,0,1,-1,-1,0,2,7},
    {0,0,1,0,1,0,1,-1,2,3},
    {0,0,0,1,1,1,0,1,2,1},
    {1,-1,0,0,-1,0,0,1,2,3},
    {0,0,1,0,-1,-1,1,0,2,2},
    {1,0,0,1,0,0,-1,-1,2,5},
    {-1,1,0,0,1,0,0,-1,2,3},
    {0,0,0,1,1,1,0,1,2,5},
    {0,-1,1,0,1,1,0,0,2,1},
    {0,0,-1,1,0,1,1,0,2,7},
    {1,1,-1,0,0,-1,0,0,2,2},
    {1,-1,0,0,-1,0,0,1,2,2},
    {0,1,-1,0,-1,-1,0,0,2,0},
    {0,0,1,-1,0,-1,-1,0,2,6},
    {1,0,0,1,0,0,-1,-1,2,4},
    {-1,1,0,0,1,0,0,-1,2,2},
    {0,-1,1,0,1,1,0,0,2,0},
    {0,0,-1,1,0,1,1,0,2,6},
    {0,1,1,-1,0,-1,0,0,2,7},
    {-1,0,0,-1,0,0,1,1,2,4},
    {1,-1,0,0,-1,0,0,1,2,1},
    {0,1,-1,0,-1,-1,0,0,2,7},
    {-1,0,0,0,-1,1,1,0,2,3},
    {0,0,1,-1,0,-1,-1,0,2,5},
    {1,0,0,1,0,0,-1,-1,2,3},
    {0,-1,1,1,0,-1,0,0,2,0},
    {-1,1,0,0,1,0,0,-1,2,1},
    {0,0,-1,1,0,1,1,0,2,5},
    {0,1,-1,0,-1,-1,0,0,2,6},
    {0,-1,1,0,1,1,0,0,2,6},
    {-1,1,0,1,0,0,0,1,2,0},
    {0,0,0,1,1,0,1,-1,2,3},
    {-1,0,0,-1,0,0,1,1,2,2},
    {1,-1,0,0,-1,0,0,1,2,7},
    {1,0,1,-1,0,0,0,1,2,5},
    {0,1,-1,0,-1,-1,0,0,2,5},
    {-1,0,0,0,-1,0,-1,1,2,2},
    {0,0,1,-1,0,-1,-1,0,2,3},
    {1,0,0,1,0,0,-1,-1,2,1},
    {-1,1,0,0,1,0,0,-1,2,7},
    {0,0,-1,1,0,1,1,0,2,3},
    {-1,0,0,-1,0,0,1,1,2,1},
    {0,1,-1,0,-1,-1,0,0,2,4},
    {1,0,0,1,0,0,-1,-1,2,0},
    {0,-1,1,0,1,1,0,0,2,4},
    {1,-1,0,0,-1,0,0,1,2,5},
    {0,-1,1,1,0,0,1,0,2,1},
    {0,1,-1,0,-1,-1,0,0,2,3},
    {1,0,0,1,0,0,-1,-1,2,7},
    {0,-1,1,0,1,1,0,0,2,3},
    {1,-1,-1,0,0,-1,0,0,2,5},
    {1,-1,0,0,-1,0,0,1,2,4},
    {-1,1,0,1,-1,0,0,0,2,5},
    {0,1,-1,0,-1,-1,0,0,2,2},
    {0,0,1,-1,0,-1,-1,0,2,0},
    {0,-1,1,0,1,1,0,0,2,2},
    {0,0,-1,1,0,1,1,0,2,0},
    {-1,0,0,-1,0,0,1,1,2,6},
    {1,0,0,0,-1,-1,1,0,2,2},
    {0,1,0,0,0,-1,-1,1,2,0},
    {0,0,1,0,-1,0,-1,-1,2,6},
    {0,0,0,1,1,-1,0,-1,2,4},
    {-1,0,0,0,1,1,-1,0,2,2},
    {0,-1,0,0,0,1,1,-1,2,0},
    {0,0,0,-1,-1,1,0,1,2,4},
    {0,-1,0,0,1,-1,0,-1,2,5},
    {-1,1,0,1,1,0,0,0,2,1},
    {1,0,0,0,-1,-1,1,0,2,1},
    {0,0,1,0,-1,0,-1,-1,2,5},
    {0,0,0,1,1,-1,0,-1,2,3},
    {0,0,-1,0,1,0,1,1,2,5},
    {0,0,0,-1,-1,1,0,1,2,3},
    {-1,-1,1,0,0,1,0,0,2,0},
    {0,0,0,1,1,-1,0,-1,2,2},
    {0,0,0,-1,0,1,1,-1,2,3},
    {0,-1,0,0,0,1,1,-1,2,6},
    {0,0,-1,0,1,0,1,1,2,4},
    {0,-1,-1,1,0,0,-1,0,2,6},
    {0,0,0,-1,-1,1,0,1,2,2},
    {1,0,0,0,-1,-1,1,0,2,7},
    {1,-1,-1,0,1,0,0,0,2,3},
    {0,1,0,0,0,-1,-1,1,2,5},
    {0,1,1,-1,0,-1,0,0,2,6},
    {0,-1,0,0,0,1,1,-1,2,5},
    {0,0,0,-1,-1,1,0,1,2,1},
    {1,0,0,0,-1,-1,1,0,2,6},
    {0,1,0,0,0,-1,-1,1,2,4},
    {0,0,-1,0,1,1,-1,0,2,4},
    {0,0,1,0,1,1,-1,0,2,1},
    {0,0,1,0,-1,0,-1,-1,2,2},
    {-1,0,0,0,1,1,-1,0,2,6},
    {0,0,-1,0,-1,-1,1,0,2,6},
    {-1,1,0,1,-1,0,0,0,2,3},
    {0,-1,0,0,0,1,1,-1,2,4},
    {0,0,-1,0,1,0,1,1,2,2},
    {0,0,0,-1,-1,1,0,1,2,0},
    {0,0,1,0,-1,0,-1,-1,2,1},
    {0,-1,0,0,0,1,1,-1,2,3},
    {0,0,-1,0,1,0,1,1,2,1},
    {0,-1,1,1,0,1,0,0,2,3},
    {1,1,0,1,-1,0,0,0,2,2},
    {0,0,0,-1,-1,1,0,1,2,7},
    {1,0,0,0,-1,-1,1,0,2,4},
    {0,1,0,0,0,-1,-1,1,2,2},
    {0,0,0,1,1,-1,0,-1,2,6},
    {-1,0,0,0,1,1,-1,0,2,4},
    {0,-1,0,0,0,1,1,-1,2,2},
    {0,0,0,-1,-1,1,0,1,2,6},
    {1,-1,-1,0,1,0,0,0,2,0},
    {0,0,1,0,-1,0,-1,-1,2,7},
    {0,-1,0,0,0,1,1,-1,2,1},
    {0,0,0,-1,-1,1,0,1,2,5},
    {1,0,1,1,-1,0,0,0,2,0},
    {-1,1,0,1,0,-1,0,0,2,6},
    {0,0,1,0,1,0,1,-1,2,2},
    {0,-1,-1,1,0,0,0,-1,2,2},
    {1,-1,0,-1,0,1,0,0,2,6},
    {1,1,-1,0,0,0,1,0,2,4},
    {-1,1,0,1,0,-1,0,0,2,5},
    {-1,-1,1,0,0,0,-1,0,2,3},
    {0,-1,-1,1,0,0,0,-1,2,1},
    {1,-1,0,-1,0,1,0,0,2,5},
    {1,1,-1,0,0,0,1,0,2,3},
    {0,1,1,-1,0,0,0,1,2,1},
    {-1,-1,1,0,0,0,-1,0,2,2},
    {1,-1,0,-1,0,1,0,0,2,4},
    {1,1,-1,0,0,0,1,0,2,2},
    {0,1,1,-1,0,0,0,1,2,0},
    {1,0,1,1,-1,0,0,0,2,5},
    {-1,0,0,-1,0,0,1,1,2,0},
    {-1,-1,1,0,0,0,-1,0,2,1},
    {0,-1,-1,1,0,0,0,-1,2,7},
    {-1,0,-1,-1,0,0,1,0,2,4},
    {1,-1,0,-1,0,1,0,0,2,3},
    {1,1,-1,0,0,0,1,0,2,1},
    {0,1,1,-1,0,0,0,1,2,7},
    {0,-1,-1,1,0,-1,0,0,2,0},
    {-1,1,0,1,0,-1,0,0,2,2},
    {-1,0,-1,-1,1,0,0,0,2,4},
    {1,-1,0,-1,0,1,0,0,2,2},
    {-1,1,0,1,0,-1,0,0,2,1},
    {-1,-1,1,0,0,0,-1,0,2,7},
    {1,1,-1,0,0,-1,0,0,2,1},
    {0,-1,-1,1,0,0,0,-1,2,5},
    {-1,0,-1,-1,1,0,0,0,2,3},
    {1,-1,0,-1,0,1,0,0,2,1},
    {0,-1,-1,1,0,0,0,-1,2,4},
    {-1,0,-1,-1,1,0,0,0,2,2},
    {1,1,-1,0,0,0,1,0,2,6},
    {0,1,1,-1,0,0,0,1,2,4},
    {0,1,0,0,1,1,-1,0,2,3},
    {-1,1,0,1,0,-1,0,0,2,7},
    {-1,-1,1,0,0,0,-1,0,2,5},
    {0,-1,-1,1,0,0,0,-1,2,3},
    {1,-1,0,-1,0,1,0,0,2,7},
    {0,1,1,-1,0,0,0,1,2,3},
    {0,0,0,-1,-1,-1,0,-1,2,2},
    {0,-1,1,1,0,0,0,1,2,6},
    {-1,0,-1,1,-1,0,0,0,2,4},
    {1,0,1,1,0,0,-1,0,2,2},
    {0,1,-1,-1,0,0,0,-1,2,6},
    {1,0,1,-1,1,0,0,0,2,4},
    {1,1,0,1,0,1,0,0,2,2},
    {-1,0,-1,1,-1,0,0,0,2,3},
    {0,1,1,-1,0,-1,0,0,2,5},
    {-1,-1,0,-1,0,-1,0,0,2,1},
    {1,0,1,-1,1,0,0,0,2,3},
    {-1,1,1,0,0,0,1,0,2,7},
    {-1,0,-1,1,-1,0,0,0,2,2},
    {1,0,1,-1,1,0,0,0,2,2},
    {1,1,0,1,0,1,0,0,2,0},
    {-1,1,1,0,0,0,1,0,2,6},
    {0,-1,1,1,0,0,0,1,2,3},
    {-1,0,-1,1,-1,0,0,0,2,1},
    {0,0,0,1,-1,0,-1,1,2,7},
    {-1,-1,0,-1,0,-1,0,0,2,7},
    {0,1,-1,-1,0,0,0,-1,2,3},
    {1,0,1,-1,1,0,0,0,2,1},
    {1,1,0,1,0,1,0,0,2,7},
    {1,0,0,0,-1,1,0,1,2,6},
    {0,-1,1,1,0,0,0,1,2,2},
    {1,0,1,-1,0,0,-1,0,2,2},
    {-1,0,-1,1,-1,0,0,0,2,0},
    {1,-1,-1,0,0,0,-1,0,2,4},
    {1,0,1,-1,1,0,0,0,2,0},
    {1,1,0,1,0,1,0,0,2,6},
    {0,-1,1,1,0,0,0,1,2,1},
    {-1,-1,0,-1,0,-1,0,0,2,5},
    {1,-1,-1,0,0,0,-1,0,2,3},
    {0,1,-1,-1,0,0,0,-1,2,1},
    {1,0,1,-1,1,0,0,0,2,7},
    {-1,1,1,0,0,0,1,0,2,3},
    {-1,0,-1,1,-1,0,0,0,2,6},
    {-1,-1,0,-1,0,-1,0,0,2,4},
    {1,-1,-1,0,0,0,-1,0,2,2},
    {0,1,0,0,-1,1,1,0,2,5},
    {-1,0,0,-1,0,0,1,1,2,7},
    {0,1,-1,-1,0,0,0,-1,2,0},
    {1,1,0,1,0,1,0,0,2,4},
    {0,-1,1,1,0,0,0,1,2,7},
    {-1,0,-1,1,-1,0,0,0,2,5},
    {-1,-1,0,-1,0,-1,0,0,2,3},
    {0,1,-1,-1,0,0,0,-1,2,7},
    {1,0,1,-1,1,0,0,0,2,5},
    {1,1,0,1,0,1,0,0,2,3},
    {-1,1,1,0,0,0,1,0,2,1},
    {0,0,-1,1,-1,0,0,1,2,3},
    {0,1,-1,0,0,0,-1,-1,2,5},
    {0,0,1,-1,1,0,0,-1,2,3},
    {0,1,0,0,1,1,-1,0,2,0},
    {1,0,0,1,1,1,0,0,2,1},
    {0,-1,-1,1,0,-1,0,0,2,4},
    {1,-1,0,-1,0,0,0,-1,2,2},
    {0,-1,1,0,0,0,1,1,2,5},
    {-1,0,-1,1,0,0,0,-1,2,6},
    {-1,0,0,-1,-1,-1,0,0,2,0},
    {1,-1,0,0,0,-1,-1,0,2,6},
    {0,-1,-1,0,1,0,0,1,2,3},
    {-1,1,1,0,0,-1,0,0,2,4},
    {-1,-1,0,0,1,0,0,1,2,1},
    {0,-1,-1,0,0,0,-1,1,2,7},
    {0,1,-1,-1,0,0,-1,0,2,1},
    {0,0,-1,-1,-1,1,0,0,2,1},
    {0,0,0,0,0,0,0,1,0,3},
    {1,0,0,-1,0,-1,1,0,2,7},
    {0,1,1,0,-1,0,0,-1,2,1},
    {-1,-1,1,0,0,0,0,-1,2,3},
    {-1,-1,0,0,0,0,-1,1,2,2},
    {0,0,-1,-1,1,-1,0,0,2,6},
    {0,-1,-1,1,0,-1,0,0,2,6},
    {1,0,0,-1,0,1,-1,0,2,4},
    {0,-1,0,0,0,0,0,1,1,4},
    {1,0,0,0,0,0,1,0,1,2},
    {0,0,1,0,0,1,1,-1,2,7},
    {0,0,-1,0,-1,1,0,1,2,4},
    {1,1,0,0,0,0,1,-1,2,2},
    {0,1,-1,-1,-1,0,0,0,2,5},
    {0,1,1,0,1,0,0,1,2,0},
    {0,-1,-1,0,0,0,1,-1,2,4},
    {-1,1,0,1,0,1,0,0,2,1},
    {0,1,-1,-1,0,-1,0,0,2,7},
    {0,0,-1,-1,1,-1,0,0,2,5},
    {-1,0,0,0,1,-1,0,-1,2,6},
    {0,-1,0,0,0,0,0,-1,1,4},
    {-1,0,0,1,0,-1,1,0,2,2},
    {0,0,-1,0,1,-1,0,-1,2,2},
    {0,1,0,0,0,0,1,0,1,1},
    {0,0,-1,1,1,1,0,0,2,2},
    {0,1,1,0,1,0,0,1,2,6},
    {0,0,1,1,0,0,-1,1,2,2},
    {-1,0,-1,1,0,-1,0,0,2,6},
    {1,-1,0,0,0,1,1,0,2,3},
    {0,0,-1,-1,1,-1,0,0,2,3},
    {-1,0,0,-1,-1,-1,0,0,2,1},
    {0,0,-1,1,0,-1,-1,0,2,0},
    {1,1,0,0,0,0,1,-1,2,7},
    {0,0,-1,0,0,0,0,1,1,4},
    {0,-1,0,0,1,0,1,1,2,1},
    {-1,-1,0,0,0,0,-1,1,2,6},
    {0,-1,-1,1,0,0,1,0,2,7},
    {0,0,-1,0,-1,1,0,1,2,7},
    {1,0,0,-1,0,1,-1,0,2,0},
    {0,0,-1,1,0,0,1,1,2,7},
    {0,1,1,0,1,0,0,1,2,4},
    {-1,1,1,0,1,0,0,0,2,3},
    {0,-1,1,0,1,0,0,-1,2,3},
    {0,0,-1,0,1,0,1,-1,2,3},
    {0,1,-1,-1,0,1,0,0,2,6},
    {0,-1,-1,0,-1,0,0,-1,2,3},
    {0,0,-1,-1,-1,1,0,0,2,7},
    {0,0,-1,0,0,-1,0,0,1,2},
    {-1,-1,0,0,0,0,-1,1,2,4},
    {0,0,-1,0,0,0,0,1,1,5},
    {-1,1,1,0,0,1,0,0,2,2},
    {0,-1,-1,0,-1,0,0,-1,2,2},
    {1,0,0,0,0,0,0,1,1,4},
    {0,-1,-1,0,-1,0,0,-1,2,7},
    {1,0,0,0,-1,-1,0,-1,2,1},
    {0,1,1,0,1,0,0,1,2,2},
    {-1,0,-1,1,0,0,-1,0,2,3},
    {-1,-1,0,0,0,0,-1,1,2,3},
    {0,0,-1,0,1,-1,0,-1,2,6},
    {1,1,0,0,0,0,1,-1,2,3},
    {0,0,1,0,-1,-1,1,0,2,5},
    {0,0,1,1,-1,1,0,0,2,7},
    {1,1,-1,0,0,0,0,-1,2,4},
    {0,0,1,0,-1,0,0,0,1,0},
    {0,0,0,-1,1,1,-1,0,2,3},
    {1,0,0,0,0,1,1,-1,2,1},
    {0,0,-1,0,0,0,-1,0,1,4},
    {0,1,0,0,1,1,0,1,2,1},
    {-1,0,0,0,0,-1,-1,1,2,0},
    {-1,0,0,0,0,1,1,-1,2,4},
    {0,1,-1,-1,0,0,1,0,2,2},
    {0,1,1,0,-1,1,0,0,2,0},
    {0,1,0,0,1,0,1,1,2,6},
    {0,-1,-1,0,0,1,-1,0,2,5},
    {0,0,0,1,-1,-1,1,0,2,2},
    {-1,1,0,1,0,0,0,-1,2,5},
    {-1,0,0,0,0,0,0,1,1,7},
    {0,1,1,0,-1,0,0,-1,2,2},
    {0,0,1,1,-1,1,0,0,2,1},
    {1,0,0,1,1,0,0,-1,2,3},
    {0,0,0,-1,1,1,-1,0,2,1},
    {0,0,1,0,-1,1,0,1,2,3},
    {0,1,0,0,1,0,1,1,2,5},
    {-1,1,0,0,1,1,0,0,2,1},
    {0,1,1,-1,0,0,0,1,2,2},
    {0,-1,0,0,0,-1,-1,1,2,2},
    {0,-1,0,0,-1,-1,1,0,2,2},
    {1,1,0,0,0,0,1,-1,2,0},
    {1,1,-1,0,0,1,0,0,2,7},
    {0,0,0,1,0,-1,0,0,1,2},
    {0,1,0,0,-1,-1,1,0,2,3},
    {0,0,1,1,-1,1,0,0,2,4},
    {0,1,0,0,1,0,1,1,2,4},
    {-1,0,0,0,1,-1,-1,0,2,0},
    {0,1,0,0,1,0,1,1,2,3},
    {0,0,0,1,0,0,0,-1,1,5},
    {0,0,1,0,-1,1,0,1,2,1},
    {0,1,1,0,1,0,0,1,2,1},
    {-1,0,0,0,0,-1,-1,1,2,4},
    {0,0,-1,1,1,1,0,0,2,7},
    {0,1,-1,-1,0,0,0,1,2,3},
    {0,0,-1,0,-1,1,0,1,2,0},
    {-1,0,0,1,1,-1,0,0,2,6},
    {0,0,0,1,-1,-1,1,0,2,6},
    {0,-1,1,1,0,0,1,0,2,5},
    {0,0,1,0,-1,-1,0,-1,2,7},
    {1,0,0,-1,0,-1,1,0,2,3},
    {-1,0,0,1,0,0,1,-1,2,3},
    {-1,0,-1,1,0,0,0,-1,2,2},
    {0,0,0,-1,1,1,0,1,2,7},
    {1,1,-1,0,0,0,0,-1,2,0},
    {0,0,0,1,-1,-1,1,0,2,4},
    {0,0,-1,1,1,1,0,0,2,0},
    {0,-1,1,0,0,0,-1,-1,2,1},
    {0,0,-1,-1,0,0,1,-1,2,0},
    {1,1,0,0,-1,1,0,0,2,4},
    {-1,0,-1,-1,0,0,0,1,2,2},
    {1,1,-1,0,0,-1,0,0,2,6},
    {0,1,1,0,0,-1,1,0,2,2},
    {0,-1,0,0,1,-1,0,-1,2,6},
    {1,0,1,-1,-1,0,0,0,2,2},
    {0,0,1,1,0,0,-1,1,2,0},
    {0,-1,-1,1,0,0,0,-1,2,0},
    {-1,0,-1,-1,1,0,0,0,2,1},
    {1,1,0,0,1,-1,0,0,2,7},
    {1,-1,-1,0,0,0,0,1,2,6},
    {0,-1,-1,1,0,0,-1,0,2,5},
    {-1,0,0,-1,1,0,0,-1,2,4},
    {0,0,-1,0,0,1,0,0,1,3},
    {0,-1,0,0,-1,0,-1,1,2,4},
    {1,-1,0,0,1,1,0,0,2,2},
    {0,0,-1,-1,1,-1,0,0,2,0},
    {0,0,1,0,-1,-1,0,-1,2,4},
    {0,1,-1,0,0,1,1,0,2,0},
    {1,0,0,0,1,1,0,1,2,1},
    {0,0,1,-1,0,0,1,1,2,0},
    {1,1,0,0,-1,1,0,0,2,2},
    {1,0,0,1,-1,0,0,1,2,4},
    {1,0,0,-1,0,-1,1,0,2,5},
    {0,-1,-1,1,-1,0,0,0,2,3},
    {-1,1,0,0,-1,-1,0,0,2,2},
    {0,-1,1,0,0,-1,-1,0,2,7},
    {1,1,0,0,0,1,-1,0,2,1},
    {-1,0,0,-1,1,0,0,-1,2,3},
    {0,1,1,-1,0,0,1,0,2,4},
    {0,-1,1,1,0,0,0,-1,2,7},
    {0,1,-1,0,0,1,1,0,2,7},
    {-1,0,-1,-1,0,0,0,1,2,1},
    {-1,1,0,0,-1,-1,0,0,2,1},
    {1,0,0,0,0,1,1,-1,2,2},
    {-1,1,0,0,-1,-1,0,0,2,0},
    {0,0,1,1,-1,1,0,0,2,0},
    {-1,1,1,0,-1,0,0,0,2,3},
    {-1,0,0,-1,1,0,0,-1,2,1},
    {0,1,-1,0,0,1,1,0,2,5},
    {1,0,0,1,-1,0,0,1,2,1},
    {-1,1,0,1,0,0,0,-1,2,1},
    {0,-1,1,0,0,-1,-1,0,2,4},
    {-1,1,0,0,0,-1,-1,0,2,1},
    {0,0,1,0,-1,1,0,1,2,6},
    {0,1,-1,0,0,1,1,0,2,3},
    {0,0,-1,0,-1,0,-1,-1,2,4},
    {-1,1,0,0,-1,-1,0,0,2,5},
    {0,1,1,-1,-1,0,0,0,2,0},
    {1,0,0,1,-1,0,0,1,2,6},
    {-1,1,0,0,-1,-1,0,0,2,4},
    {0,0,-1,1,-1,-1,0,0,2,4},
    {0,0,1,1,-1,1,0,0,2,5},
    {-1,1,0,1,-1,0,0,0,2,6},
    {-1,-1,1,0,0,-1,0,0,2,1},
    {-1,0,0,-1,1,0,0,-1,2,5},
    {1,-1,0,0,0,1,1,0,2,7},
    {0,0,1,-1,0,0,1,1,2,7},
    {1,1,0,0,-1,1,0,0,2,3},
    {0,0,1,-1,1,1,0,0,2,3},
    {-1,0,0,0,0,1,-1,-1,2,5},
    {-1,0,0,0,0,1,0,0,1,2},
    {-1,1,0,0,1,0,0,-1,2,5},
    {1,1,-1,0,0,0,1,0,2,7},
    {1,0,0,0,0,0,0,1,1,0},
    {1,0,1,1,0,-1,0,0,2,5},
    {0,0,0,-1,1,1,0,1,2,4},
    {1,0,0,-1,1,0,0,1,2,6},
    {0,-1,0,0,0,1,0,0,1,7},
    {1,0,0,0,0,-1,1,1,2,5},
    {0,0,1,0,-1,-1,0,-1,2,1},
    {-1,-1,1,0,0,0,-1,0,2,0},
    {0,0,-1,0,1,1,0,1,2,0},
    {0,0,0,-1,0,0,0,0,0,1},
    {-1,0,0,0,0,1,-1,-1,2,3},
    {1,-1,-1,0,0,0,-1,0,2,1},
    {0,-1,0,0,1,0,1,-1,2,1},
    {0,1,-1,-1,-1,0,0,0,2,0},
    {1,0,1,-1,0,0,0,-1,2,0},
    {0,0,0,1,0,0,0,0,0,1},
    {0,1,-1,0,0,0,1,1,2,7},
    {-1,1,0,1,0,0,0,1,2,7},
    {0,0,0,-1,-1,1,1,0,2,5},
    {1,1,0,1,0,0,0,1,2,6},
    {0,-1,1,0,0,-1,-1,0,2,1},
    {0,1,0,0,-1,0,-1,1,2,1},
    {-1,0,0,0,0,1,-1,-1,2,2},
    {0,-1,-1,0,0,1,-1,0,2,7},
    {1,1,-1,0,0,0,-1,0,2,3},
    {1,1,-1,0,1,0,0,0,2,0},
    {0,0,-1,0,1,1,0,1,2,6},
    {0,0,1,1,-1,0,0,-1,2,3},
    {0,0,0,-1,-1,1,1,0,2,4},
    {0,0,1,-1,0,0,1,1,2,2},
    {-1,0,0,0,0,-1,-1,1,2,6},
    {-1,0,0,0,-1,0,-1,-1,2,5},
    {1,0,0,0,0,-1,1,1,2,2},
    {0,1,0,0,-1,0,-1,1,2,0},
    {0,0,-1,0,1,1,0,1,2,5},
    {1,1,0,1,-1,0,0,0,2,5},
    {1,0,0,-1,-1,1,0,0,2,5},
    {1,0,1,1,0,0,-1,0,2,4},
    {0,0,1,0,-1,-1,0,-1,2,5},
    {1,-1,-1,0,0,0,-1,0,2,0},
    {1,-1,0,-1,0,0,-1,0,2,1},
    {-1,0,0,-1,1,0,0,-1,2,7},
    {0,0,-1,-1,1,-1,0,0,2,7},
    {0,-1,0,0,1,0,1,-1,2,6},
    {0,0,-1,0,0,0,1,0,1,1},
    {0,-1,0,0,0,-1,0,0,1,2},
    {0,-1,0,0,1,1,0,1,2,0},
    {0,0,1,1,0,1,-1,0,2,7},
    {0,0,-1,0,1,1,0,1,2,4},
    {1,0,0,0,0,-1,1,1,2,0},
    {0,0,0,1,-1,1,0,1,2,2},
    {1,0,0,-1,0,-1,1,0,2,6},
    {1,0,1,1,-1,0,0,0,2,1},
    {0,0,0,-1,-1,0,0,0,1,4},
    {1,0,0,0,-1,-1,1,0,2,5},
    {-1,0,-1,-1,0,-1,0,0,2,0},
    {0,0,0,-1,-1,1,1,0,2,1},
    {-1,0,0,1,-1,0,0,-1,2,5},
    {0,1,-1,0,-1,0,0,1,2,0},
    {1,0,0,0,0,-1,1,1,2,7},
    {1,0,0,1,1,0,0,-1,2,0},
    {1,1,0,0,1,0,0,1,2,0},
    {0,0,1,0,-1,-1,0,-1,2,3},
    {1,0,0,0,1,1,0,1,2,4},
    {1,-1,-1,0,0,1,0,0,2,4},
    {-1,0,0,0,0,1,-1,-1,2,6},
    {0,0,0,-1,0,-1,1,1,2,7},
    {0,-1,0,0,1,0,1,-1,2,4},
    {0,0,-1,0,1,1,0,1,2,2},
    {1,0,0,0,0,-1,1,1,2,6},
    {-1,0,0,0,0,0,0,0,0,0},
    {1,-1,0,0,-1,-1,0,0,2,2},
    {0,0,-1,1,0,1,1,0,2,2},
    {0,0,0,1,-1,-1,1,0,2,5},
    {-1,0,0,1,-1,0,0,-1,2,4},
    {0,-1,-1,0,0,1,-1,0,2,0},
    {0,0,-1,-1,0,0,1,-1,2,6},
    {1,0,0,0,-1,1,0,1,2,5},
    {0,0,-1,0,0,1,-1,-1,2,1},
    {0,0,0,1,-1,-1,1,0,2,3},
    {-1,-1,0,-1,0,0,0,1,2,3},
    {0,1,0,0,1,1,-1,0,2,4},
    {1,1,0,0,0,0,1,-1,2,1},
    {1,0,1,-1,0,1,0,0,2,1},
    {0,0,-1,1,0,0,-1,-1,2,1},
    {1,0,0,-1,1,0,0,1,2,4},
    {-1,0,0,0,1,1,0,1,2,2},
    {1,0,0,0,0,1,-1,-1,2,3},
    {-1,0,-1,-1,-1,0,0,0,2,2},
    {1,1,0,1,0,0,0,-1,2,1},
    {0,0,1,1,0,0,-1,1,2,6},
    {0,0,1,0,1,0,1,1,2,4},
    {1,-1,0,0,0,-1,-1,0,2,0},
    {-1,0,0,1,-1,0,0,-1,2,3},
    {1,-1,0,0,0,1,1,0,2,4},
    {0,0,1,1,1,-1,0,0,2,4},
    {1,-1,-1,0,0,0,1,0,2,7},
    {0,-1,1,1,1,0,0,0,2,5},
    {0,0,1,1,-1,0,0,-1,2,6},
    {0,0,0,1,1,-1,-1,0,2,2},
    {-1,-1,0,-1,0,0,1,0,2,1},
    {1,0,1,-1,1,0,0,0,2,6},
    {0,-1,0,0,1,0,1,1,2,3},
    {1,1,0,0,0,-1,1,0,2,1},
    {0,1,1,0,1,0,0,1,2,7},
    {0,-1,-1,0,0,1,-1,0,2,6},
    {-1,0,0,0,0,-1,-1,1,2,5},
    {0,1,1,0,-1,0,0,-1,2,3},
    {0,1,0,0,0,0,0,1,1,6},
    {0,0,0,1,0,-1,1,1,2,6},
    {-1,0,0,0,-1,1,1,0,2,5},
    {-1,1,0,0,0,-1,-1,0,2,4},
    {0,-1,1,1,1,0,0,0,2,3},
    {-1,-1,0,0,-1,0,0,-1,2,7},
    {0,1,0,0,-1,0,-1,1,2,2},
    {0,-1,1,1,0,0,-1,0,2,0},
    {-1,0,-1,1,0,1,0,0,2,1},
    {1,-1,-1,0,0,0,0,1,2,5},
    {1,0,0,0,0,-1,-1,1,2,3},
    {0,-1,1,1,0,0,1,0,2,3},
    {0,1,-1,-1,-1,0,0,0,2,3},
    {0,0,-1,1,0,0,-1,-1,2,7},
    {1,0,0,0,1,0,0,0,1,3},
    {1,1,0,0,0,1,-1,0,2,5},
    {-1,0,-1,-1,1,0,0,0,2,0},
    {0,-1,1,1,1,0,0,0,2,2},
    {0,0,0,1,1,1,0,1,2,4},
    {-1,0,0,0,0,0,-1,0,1,4},
    {-1,1,1,0,0,0,0,-1,2,4},
    {-1,0,0,1,0,-1,1,0,2,7},
    {-1,0,0,0,1,1,0,1,2,4},
    {-1,0,-1,1,0,1,0,0,2,7},
    {1,0,1,-1,0,-1,0,0,2,7},
    {0,1,0,0,0,1,-1,-1,2,3},
    {0,-1,1,0,-1,0,0,1,2,7},
    {1,1,0,1,0,0,-1,0,2,5},
    {-1,-1,0,-1,0,0,0,-1,2,0},
    {1,1,0,0,0,-1,1,0,2,4},
    {0,-1,1,1,1,0,0,0,2,0},
    {-1,-1,0,-1,0,0,1,0,2,4},
    {-1,0,0,1,-1,0,0,-1,2,0},
    {1,-1,-1,0,0,0,1,0,2,6},
    {0,-1,0,0,-1,-1,1,0,2,4},
    {0,0,0,-1,-1,0,0,0,1,1},
    {-1,1,1,0,1,0,0,0,2,4},
    {-1,0,0,0,0,-1,-1,1,2,7},
    {1,1,0,1,0,0,-1,0,2,7},
    {0,0,-1,0,0,0,1,0,1,0},
    {0,0,0,0,0,1,0,0,0,6},
    {1,1,0,1,0,0,0,-1,2,3},
    {0,-1,1,0,0,0,1,1,2,4},
    {0,0,0,-1,-1,0,-1,-1,2,6},
    {0,-1,1,1,1,0,0,0,2,7},
    {0,1,1,0,0,1,-1,0,2,2},
    {0,0,1,0,1,0,0,0,1,6},
    {0,1,0,0,0,0,0,0,0,2},
    {1,1,0,1,0,0,-1,0,2,3},
    {1,1,0,0,1,-1,0,0,2,3},
    {1,-1,-1,0,0,0,0,1,2,0},
    {-1,0,0,-1,1,0,0,-1,2,6},
    {-1,-1,0,0,0,0,1,-1,2,2},
    {1,0,0,-1,0,1,-1,0,2,2},
    {0,0,1,0,0,-1,0,0,1,5},
    {0,0,-1,1,0,-1,-1,0,2,7},
    {-1,1,1,0,0,1,0,0,2,1},
    {-1,-1,0,0,1,-1,0,0,2,0},
    {0,0,-1,-1,0,0,1,-1,2,4},
    {1,-1,0,-1,0,-1,0,0,2,7},
    {1,0,0,-1,1,0,0,1,2,2},
    {1,1,-1,0,-1,0,0,0,2,6},
    {1,1,0,0,-1,1,0,0,2,0},
    {0,0,1,1,0,0,-1,1,2,4},
    {-1,0,0,1,1,0,0,1,2,4},
    {-1,1,0,1,1,0,0,0,2,0},
    {0,1,0,0,-1,0,0,0,1,0},
    {0,0,1,0,1,0,1,1,2,3},
    {1,0,1,-1,0,0,0,-1,2,6},
    {-1,-1,0,-1,0,1,0,0,2,2},
    {0,-1,0,0,1,0,1,1,2,6},
    {0,-1,-1,0,0,-1,1,0,2,7},
    {1,0,0,-1,-1,0,0,-1,2,3},
    {0,0,0,0,0,0,0,1,0,0},
    {1,-1,-1,0,0,0,1,0,2,3},
    {-1,-1,0,0,0,-1,1,0,2,3},
    {0,0,-1,0,-1,-1,0,-1,2,6},
    {1,0,0,1,-1,-1,0,0,2,6},
    {1,1,0,0,1,-1,0,0,2,1},
    {0,0,1,1,0,0,1,-1,2,5},
    {0,0,0,1,-1,-1,0,-1,2,3},
    {-1,0,0,0,0,-1,-1,1,2,2},
    {-1,0,0,1,1,0,0,1,2,2},
    {0,1,-1,-1,-1,0,0,0,2,1},
    {1,1,0,1,0,0,0,1,2,0},
    {0,0,-1,-1,0,0,-1,1,2,4},
    {-1,0,-1,1,0,0,1,0,2,6},
    {0,0,0,-1,0,1,0,0,1,5},
    {1,1,-1,0,0,1,0,0,2,0},
    {0,0,0,-1,0,0,-1,0,1,7},
    {0,-1,-1,0,0,-1,1,0,2,0},
    {-1,1,0,1,0,0,-1,0,2,3},
    {0,0,0,-1,-1,-1,0,-1,2,6},
    {-1,0,0,1,1,0,0,1,2,1},
    {0,0,0,1,1,0,0,0,1,2},
    {-1,1,0,0,-1,-1,0,0,2,6},
    {0,-1,1,1,0,0,1,0,2,6},
    {1,0,0,0,0,0,0,0,0,6},
    {0,0,1,0,0,1,0,0,1,0},
    {1,0,0,1,-1,0,0,1,2,0},
    {1,1,0,1,0,0,-1,0,2,6},
    {0,0,1,1,0,0,1,-1,2,3},
    {-1,0,0,1,-1,0,0,-1,2,6},
    {-1,0,0,-1,1,1,0,0,2,5},
    {-1,-1,0,0,1,-1,0,0,2,4},
    {-1,0,0,0,-1,-1,1,0,2,1},
    {0,0,-1,-1,0,0,-1,1,2,2},
    {0,0,-1,1,0,0,1,1,2,3},
    {0,1,1,-1,0,1,0,0,2,6},
    {-1,0,0,1,1,0,0,1,2,7},
    {-1,-1,0,0,-1,1,0,0,2,5},
    {0,-1,0,0,0,0,0,-1,1,0},
    {1,1,-1,0,0,0,-1,0,2,7},
    {1,-1,-1,0,-1,0,0,0,2,2},
    {1,0,0,-1,-1,0,0,-1,2,7},
    {1,0,1,-1,0,-1,0,0,2,3},
    {1,1,0,0,1,-1,0,0,2,5},
    {1,0,1,-1,0,0,0,1,2,4},
    {0,1,-1,-1,-1,0,0,0,2,4},
    {0,1,-1,-1,0,0,1,0,2,1},
    {0,1,1,0,0,1,-1,0,2,3},
    {-1,-1,1,0,0,0,0,1,2,7},
    {0,-1,-1,0,0,1,-1,0,2,3},
    {-1,0,0,1,1,0,0,1,2,6},
    {-1,0,-1,-1,0,0,1,0,2,0},
    {0,0,0,-1,-1,0,0,0,1,6},
    {0,0,0,-1,0,0,0,1,1,1},
    {-1,-1,1,0,1,0,0,0,2,6},
    {1,0,1,1,0,1,0,0,2,4},
    {0,1,1,0,-1,0,0,-1,2,7},
    {-1,-1,0,0,-1,1,0,0,2,4},
    {0,1,1,-1,0,1,0,0,2,4},
    {-1,0,-1,1,0,-1,0,0,2,0},
    {0,1,1,0,0,-1,1,0,2,0},
    {0,-1,-1,0,0,-1,1,0,2,2},
    {1,0,0,0,0,-1,1,1,2,3},
    {-1,0,0,0,0,0,0,-1,1,3},
    {0,0,-1,-1,0,0,-1,1,2,0},
    {0,-1,-1,0,0,-1,1,0,2,5},
    {0,1,1,0,0,1,-1,0,2,6},
    {0,0,0,-1,1,-1,-1,0,2,4},
    {-1,0,0,1,0,-1,1,0,2,6},
    {-1,0,0,1,1,0,0,1,2,5},
    {1,0,0,0,-1,0,-1,1,2,2},
    {0,0,1,-1,0,0,1,1,2,6},
    {0,-1,1,1,0,-1,0,0,2,6},
    {-1,-1,0,0,-1,1,0,0,2,3},
    {0,1,0,0,-1,0,-1,1,2,7},
    {0,-1,-1,0,0,-1,1,0,2,1},
    {0,0,-1,-1,0,0,-1,1,2,7},
    {0,1,0,0,-1,0,-1,-1,2,7},
    {0,-1,1,1,0,-1,0,0,2,2},
    {0,1,-1,0,0,1,1,0,2,2},
    {0,1,1,-1,0,1,0,0,2,7},
    {-1,0,0,0,0,0,0,0,0,2},
    {0,1,1,0,0,1,-1,0,2,1},
    {0,0,0,1,0,1,-1,-1,2,1},
    {0,0,1,1,0,0,1,-1,2,7},
    {-1,0,0,1,-1,0,0,-1,2,1},
    {-1,0,-1,1,0,1,0,0,2,0},
    {0,0,1,-1,1,1,0,0,2,6},
    {-1,-1,0,-1,0,0,0,1,2,4},
    {-1,0,0,1,0,0,1,-1,2,2},
    {0,0,0,1,-1,-1,0,-1,2,5},
    {-1,0,0,0,-1,0,-1,1,2,1},
    {0,0,1,-1,1,1,0,0,2,4},
    {0,0,1,1,0,0,-1,1,2,3},
    {0,0,0,1,1,-1,-1,0,2,0},
    {0,0,-1,0,0,-1,1,1,2,5},
    {0,0,0,1,1,-1,-1,0,2,1},
    {0,-1,0,0,0,1,-1,-1,2,5},
    {0,-1,0,0,1,-1,0,-1,2,7},
    {-1,-1,1,0,0,0,0,1,2,3},
    {-1,0,0,1,0,-1,1,0,2,4},
    {-1,-1,0,0,0,-1,1,0,2,0},
    {0,1,1,0,0,-1,1,0,2,5},
    {1,0,0,1,0,0,-1,-1,2,6},
    {0,-1,0,0,-1,0,-1,-1,2,0},
    {1,0,1,1,0,0,0,1,2,1},
    {-1,0,-1,-1,0,-1,0,0,2,7},
    {0,1,0,0,0,1,-1,-1,2,5},
    {-1,1,0,0,-1,0,0,1,2,1},
    {0,1,-1,-1,0,0,0,-1,2,5},
    {0,-1,0,0,-1,0,-1,-1,2,2},
    {1,0,0,-1,0,1,-1,0,2,5},
    {0,-1,1,0,0,1,1,0,2,4},
    {0,1,1,-1,1,0,0,0,2,1},
    {0,1,0,0,-1,1,0,1,2,2},
    {0,0,0,1,1,-1,0,-1,2,7},
    {1,0,0,0,0,-1,1,1,2,1},
    {1,0,1,1,0,1,0,0,2,7},
    {-1,-1,0,0,0,1,-1,0,2,6},
    {1,-1,0,0,1,1,0,0,2,0},
    {1,-1,0,-1,0,0,-1,0,2,4},
    {1,0,0,-1,1,0,0,1,2,3},
    {0,0,1,1,0,0,-1,1,2,7},
    {0,-1,-1,1,-1,0,0,0,2,5},
    {1,1,-1,0,0,0,0,-1,2,2},
    {1,0,1,1,0,0,1,0,2,5},
    {-1,0,0,1,0,-1,1,0,2,5},
    {0,-1,1,0,0,1,1,0,2,0},
    {0,0,-1,1,1,0,0,-1,2,3},
    {-1,0,0,0,0,1,-1,-1,2,1},
    {-1,0,0,0,1,0,0,0,1,7},
    {0,1,1,-1,1,0,0,0,2,0},
    {0,0,1,0,1,0,1,-1,2,5},
    {-1,-1,1,0,0,0,0,-1,2,1},
    {-1,-1,0,0,0,0,-1,1,2,0},
    {-1,1,0,0,-1,-1,0,0,2,3},
    {-1,-1,1,0,0,0,0,1,2,1},
    {0,-1,-1,1,-1,0,0,0,2,7},
    {-1,1,0,1,0,0,0,1,2,4},
    {0,-1,0,0,-1,0,-1,-1,2,4},
    {0,0,0,1,-1,0,0,0,1,0},
    {-1,1,0,1,0,0,0,-1,2,6},
    {-1,0,0,-1,0,1,1,0,2,3},
    {1,0,1,-1,0,-1,0,0,2,6},
    {0,-1,-1,0,0,1,-1,0,2,4},
    {1,0,1,1,0,1,0,0,2,5},
    {0,0,1,-1,1,1,0,0,2,1},
    {1,1,-1,0,0,0,1,0,2,5},
    {-1,0,-1,1,0,1,0,0,2,4},
    {-1,1,0,1,0,0,1,0,2,3},
    {0,1,0,0,-1,-1,1,0,2,6},
    {0,0,0,0,-1,0,0,0,0,7},
    {-1,-1,1,0,-1,0,0,0,2,0},
    {-1,-1,1,0,0,0,0,1,2,0},
    {0,0,0,-1,-1,0,-1,1,2,1},
    {1,0,0,1,-1,0,0,1,2,3},
    {-1,0,-1,-1,0,-1,0,0,2,4},
    {0,0,-1,0,-1,0,-1,-1,2,6},
    {1,1,0,0,-1,1,0,0,2,1},
    {0,-1,-1,0,-1,0,0,-1,2,6},
    {0,-1,0,0,-1,1,0,1,2,1},
    {1,-1,0,-1,0,0,-1,0,2,2},
    {-1,1,0,1,0,0,1,0,2,2},
    {-1,0,0,0,1,1,-1,0,2,1},
    {0,0,1,-1,0,0,1,1,2,1},
    {0,0,-1,0,0,1,1,-1,2,7},
    {0,0,1,0,0,0,0,-1,1,2},
    {-1,0,-1,1,0,0,0,1,2,3},
    {0,1,1,-1,1,0,0,0,2,5},
    {0,0,1,1,0,-1,1,0,2,2},
    {1,0,1,1,0,1,0,0,2,3},
    {0,0,0,0,0,0,1,0,0,6},
    {-1,-1,1,0,0,0,0,1,2,6},
    {0,0,1,-1,-1,-1,0,0,2,0},
    {0,0,1,-1,1,1,0,0,2,2},
    {0,1,-1,0,1,0,0,-1,2,4},
    {1,-1,0,-1,0,0,0,1,2,1},
    {0,0,1,1,0,0,1,-1,2,2},
    {1,0,1,1,0,-1,0,0,2,1},
    {0,-1,1,1,0,1,0,0,2,5},
    {0,-1,-1,1,-1,0,0,0,2,4},
    {-1,-1,0,0,-1,0,0,-1,2,4},
    {-1,0,0,-1,1,0,0,-1,2,0},
    {0,0,0,1,0,-1,1,1,2,4},
    {0,1,1,-1,1,0,0,0,2,4},
    {0,1,-1,-1,0,0,0,1,2,5},
    {1,0,0,-1,-1,0,0,-1,2,6},
    {0,0,-1,0,1,1,0,1,2,3},
    {0,1,-1,-1,0,0,1,0,2,5},
    {1,-1,0,0,1,0,0,-1,2,2},
    {-1,-1,0,-1,0,0,-1,0,2,6},
    {-1,0,-1,-1,0,-1,0,0,2,1},
    {1,-1,0,-1,0,0,-1,0,2,7},
    {1,0,0,0,0,0,0,0,0,7},
    {0,0,-1,0,1,-1,0,-1,2,5},
    {0,-1,0,0,1,1,-1,0,2,1},
    {0,0,0,-1,0,0,0,0,0,0},
    {1,1,0,1,0,0,0,1,2,2},
    {-1,-1,1,0,0,0,0,1,2,4},
    {-1,1,0,1,0,0,1,0,2,6},
    {-1,-1,0,0,1,-1,0,0,2,6},
    {1,0,0,1,0,0,-1,-1,2,2},
    {0,1,-1,0,0,1,1,0,2,4},
    {0,0,0,-1,-1,-1,1,0,2,7},
    {0,1,0,0,-1,1,0,1,2,0},
    {1,0,0,1,1,1,0,0,2,2},
    {1,1,0,0,-1,1,0,0,2,6},
    {0,-1,0,0,1,0,1,-1,2,2},
    {0,1,1,0,0,-1,1,0,2,4},
    {0,0,0,-1,0,1,-1,-1,2,2},
    {-1,0,0,1,1,0,0,1,2,0},
    {0,1,-1,0,-1,0,0,1,2,7},
    {1,0,0,0,-1,0,-1,1,2,6},
    {0,0,-1,0,0,-1,1,1,2,7},
    {0,-1,0,0,1,0,1,-1,2,5},
    {0,-1,0,0,-1,0,0,0,1,4},
    {0,1,0,0,0,0,0,-1,1,0},
    {0,0,0,-1,0,0,0,1,1,3},
    {-1,0,-1,-1,-1,0,0,0,2,7},
    {0,0,-1,-1,0,0,1,-1,2,1},
    {1,0,0,-1,1,0,0,1,2,7},
    {0,0,0,-1,0,1,0,0,1,7},
    {0,0,0,0,0,0,-1,0,0,4},
    {1,1,0,0,-1,1,0,0,2,5},
    {0,-1,1,0,-1,-1,0,0,2,1},
    {1,0,0,0,0,0,-1,0,1,1},
    {1,0,0,1,0,0,1,1,2,6},
    {-1,0,0,0,0,-1,0,0,1,0},
    {0,0,-1,0,0,0,0,-1,1,4},
    {0,0,0,-1,0,0,0,0,0,6},
    {1,1,0,1,0,1,0,0,2,5},
    {-1,0,0,0,0,0,-1,0,1,0},
    {1,0,0,0,1,1,-1,0,2,6},
    {0,-1,1,1,0,0,0,-1,2,2},
    {-1,1,1,0,-1,0,0,0,2,7},
    {-1,1,0,0,-1,0,0,1,2,7},
    {1,0,0,-1,-1,0,0,-1,2,2},
    {1,1,0,1,0,0,0,-1,2,5},
    {1,0,0,0,0,1,0,0,1,0},
    {0,0,0,1,-1,0,0,0,1,2},
    {0,-1,0,0,0,-1,1,1,2,3},
    {-1,0,0,0,0,0,0,1,1,3},
    {0,0,-1,-1,0,0,-1,1,2,1},
    {0,0,1,0,0,0,0,-1,1,4},
    {1,1,-1,0,0,0,-1,0,2,4},
    {1,0,0,0,-1,0,-1,1,2,3},
    {-1,0,-1,-1,0,1,0,0,2,4},
    {-1,1,0,1,0,-1,0,0,2,3},
    {0,-1,0,0,0,0,-1,0,1,5},
    {0,1,0,0,-1,-1,1,0,2,5},
    {1,0,1,1,1,0,0,0,2,0},
    {0,0,-1,0,0,0,0,-1,1,3},
    {0,0,1,-1,0,0,-1,-1,2,1},
    {1,0,1,-1,0,0,0,1,2,2},
    {0,0,1,0,-1,0,-1,1,2,1},
    {0,0,0,-1,1,0,0,0,1,1},
    {0,0,0,1,0,0,0,1,1,3},
    {-1,-1,1,0,0,0,1,0,2,0},
    {1,0,0,0,0,1,0,0,1,7},
    {0,0,1,0,-1,0,-1,1,2,3},
    {0,1,0,0,0,0,1,0,1,5},
    {0,-1,0,0,0,0,-1,0,1,4},
    {0,0,1,0,0,0,0,-1,1,0},
    {0,0,0,1,-1,0,-1,1,2,3},
    {0,0,-1,0,0,0,0,-1,1,2},
    {1,0,0,0,1,1,-1,0,2,4},
    {0,0,0,-1,1,0,0,0,1,0},
    {1,0,0,0,0,1,-1,-1,2,5},
    {-1,-1,0,0,0,-1,1,0,2,7},
    {0,0,1,0,0,0,0,-1,1,7},
    {0,0,0,1,0,0,1,0,1,7},
    {-1,0,0,0,0,1,0,0,1,4},
    {0,0,1,-1,0,-1,-1,0,2,7},
    {1,0,0,0,0,1,0,0,1,5},
    {1,0,1,1,0,0,1,0,2,3},
    {0,1,0,0,0,0,1,0,1,3},
    {0,0,0,1,-1,1,1,0,2,0},
    {-1,1,1,0,0,0,0,-1,2,3},
    {0,0,1,0,0,0,0,1,1,1},
    {-1,-1,0,-1,0,0,1,0,2,0},
    {0,-1,-1,0,-1,0,0,-1,2,4},
    {0,-1,0,0,0,0,-1,0,1,2},
    {0,0,1,1,0,0,-1,1,2,1},
    {-1,-1,1,0,0,0,1,0,2,2},
    {0,0,1,0,-1,-1,0,-1,2,0},
    {-1,-1,0,0,0,0,1,-1,2,1},
    {0,1,0,0,0,0,-1,0,1,6},
    {0,1,0,0,0,0,-1,0,1,5},
    {0,0,0,1,0,0,0,-1,1,2},
    {0,0,1,0,1,1,0,1,2,6},
    {-1,0,0,0,0,1,0,0,1,6},
    {1,-1,0,-1,0,0,-1,0,2,5},
    {1,0,0,-1,1,0,0,1,2,1},
    {0,1,0,0,1,0,0,0,1,4},
    {0,0,0,-1,0,1,1,-1,2,4},
    {-1,0,-1,1,0,0,-1,0,2,2},
    {-1,0,-1,-1,0,0,0,-1,2,1},
    {1,1,0,1,0,0,-1,0,2,0},
    {1,0,1,1,0,1,0,0,2,1},
    {-1,0,0,0,-1,0,0,0,1,2},
    {0,0,0,1,0,0,0,1,1,0},
    {0,-1,1,1,0,0,-1,0,2,6},
    {0,0,0,0,1,0,0,0,0,2},
    {0,1,0,0,0,-1,-1,1,2,1},
    {0,1,1,-1,1,0,0,0,2,3},
    {1,0,0,0,1,1,0,1,2,7},
    {1,1,0,1,0,-1,0,0,2,2},
    {1,0,0,0,1,0,0,0,1,4},
    {0,-1,-1,0,1,-1,0,0,2,4},
    {0,1,0,0,0,0,-1,0,1,3},
    {0,0,-1,0,-1,0,0,0,1,7},
    {-1,-1,0,0,0,0,1,-1,2,3},
    {-1,0,-1,1,0,-1,0,0,2,4},
    {0,-1,0,0,-1,0,-1,1,2,1},
    {0,-1,0,0,0,-1,-1,1,2,7},
    {0,1,1,0,0,-1,1,0,2,3},
    {0,0,0,1,0,0,0,-1,1,3},
    {-1,0,0,0,-1,1,0,1,2,2},
    {1,0,0,0,0,-1,0,0,1,0},
    {0,0,-1,-1,-1,1,0,0,2,6},
    {1,0,1,-1,0,-1,0,0,2,0},
    {0,0,1,1,0,-1,1,0,2,3},
    {0,0,0,-1,0,-1,0,0,1,5},
    {0,0,1,-1,0,-1,-1,0,2,1},
    {0,-1,0,0,-1,0,-1,-1,2,6},
    {-1,1,1,0,0,0,0,1,2,4},
    {-1,1,0,1,0,0,0,-1,2,4},
    {0,1,1,0,0,1,-1,0,2,0},
    {0,1,1,-1,0,0,0,1,2,5},
    {0,0,-1,0,0,0,0,-1,1,1},
    {1,1,-1,0,0,0,0,-1,2,7},
    {0,0,0,1,0,1,1,-1,2,6},
    {0,0,-1,-1,-1,0,0,-1,2,7},
    {-1,0,0,1,1,-1,0,0,2,5},
    {0,1,0,0,0,0,0,1,1,3},
    {-1,0,0,0,0,0,0,0,0,3},
    {1,1,0,1,0,-1,0,0,2,4},
    {0,0,0,-1,1,1,-1,0,2,2},
    {-1,0,0,0,0,0,1,0,1,2},
    {-1,0,-1,1,0,0,1,0,2,0},
    {0,-1,-1,0,1,0,0,1,2,2},
    {-1,0,0,0,0,1,0,0,1,0},
    {0,0,0,1,0,1,0,0,1,4},
    {0,0,1,0,0,1,0,0,1,6},
    {1,0,0,0,0,1,1,-1,2,7},
    {0,0,1,0,1,0,0,0,1,4},
    {0,0,0,-1,0,-1,0,0,1,3},
    {0,-1,0,0,1,-1,-1,0,2,5},
    {0,1,0,0,0,1,0,0,1,7},
    {0,0,0,-1,0,0,0,-1,1,5},
    {1,0,0,-1,-1,1,0,0,2,7},
    {0,1,0,0,1,1,-1,0,2,5},
    {-1,0,0,0,0,0,1,0,1,1},
    {0,0,0,-1,-1,1,1,0,2,0},
    {1,-1,-1,0,0,0,1,0,2,1},
    {0,-1,0,0,0,0,1,0,1,2},
    {0,1,0,0,-1,1,0,1,2,3},
    {1,0,0,0,0,0,0,0,0,1},
    {0,0,-1,0,-1,0,0,0,1,6},
    {1,1,-1,0,-1,0,0,0,2,3},
    {-1,0,0,0,0,0,1,0,1,0},
    {0,-1,0,0,0,0,0,1,1,6},
    {0,0,1,-1,-1,-1,0,0,2,3},
    {0,0,1,1,-1,0,0,-1,2,7},
    {0,1,0,0,0,0,0,-1,1,6},
    {1,-1,-1,0,-1,0,0,0,2,3},
    {1,-1,0,-1,0,0,-1,0,2,6},
    {0,0,-1,0,1,0,1,-1,2,5},
    {0,1,-1,0,0,0,1,1,2,4},
    {0,0,0,1,0,1,0,0,1,3},
    {0,0,0,1,-1,-1,1,0,2,1},
    {1,-1,0,0,1,1,0,0,2,4},
    {-1,-1,1,0,0,0,0,-1,2,6},
    {-1,-1,0,0,1,0,0,1,2,2},
    {0,1,1,0,0,-1,1,0,2,1},
    {0,-1,1,0,0,-1,-1,0,2,0},
    {0,1,0,0,0,-1,1,1,2,4},
    {-1,0,0,1,-1,0,0,-1,2,7},
    {1,-1,0,0,1,1,0,0,2,1},
    {1,0,0,0,0,0,-1,0,1,3},
    {0,-1,1,0,1,0,0,-1,2,2},
    {-1,1,1,0,0,0,0,-1,2,6},
    {0,0,1,0,0,1,0,0,1,3},
    {0,0,0,-1,-1,1,1,0,2,3},
    {1,0,0,0,-1,0,0,0,1,6},
    {1,0,0,0,-1,0,-1,-1,2,7},
    {-1,0,0,0,1,-1,-1,0,2,2},
    {0,0,1,0,0,0,-1,0,1,3},
    {0,1,0,0,0,0,0,-1,1,4},
    {0,1,0,0,1,0,1,-1,2,1},
    {0,0,1,0,0,0,0,0,0,6},
    {1,0,0,0,1,0,0,0,1,0},
    {0,0,0,1,0,1,0,0,1,5},
    {1,0,0,0,0,1,-1,-1,2,2},
    {-1,0,0,0,0,0,1,0,1,5},
    {0,0,0,1,1,-1,0,-1,2,0},
    {-1,0,0,-1,0,1,1,0,2,7},
    {-1,-1,0,0,0,1,-1,0,2,5},
    {0,0,-1,0,0,0,1,0,1,5},
    {1,0,0,-1,0,0,1,-1,2,7},
    {0,0,0,-1,-1,1,1,0,2,6},
    {1,0,1,1,0,1,0,0,2,0},
    {0,0,0,-1,1,1,0,1,2,1},
    {0,-1,0,0,0,0,0,1,1,3},
    {-1,0,0,1,-1,0,0,-1,2,2},
    {0,-1,1,0,1,0,0,-1,2,0},
    {0,0,-1,0,0,0,-1,0,1,6},
    {0,0,0,1,0,-1,0,0,1,1},
    {0,0,-1,0,-1,0,0,0,1,1},
    {0,1,0,0,0,0,0,-1,1,3},
    {0,0,1,0,1,0,0,0,1,1},
    {1,1,0,1,0,0,0,-1,2,4},
    {0,-1,0,0,0,0,-1,0,1,6},
    {1,0,0,0,-1,-1,0,-1,2,7},
    {0,0,0,0,0,0,0,1,0,1},
    {-1,0,0,0,0,0,1,0,1,4},
    {1,0,0,0,-1,0,0,0,1,3},
    {1,-1,0,0,-1,-1,0,0,2,5},
    {1,0,0,0,0,0,-1,0,1,5},
    {1,0,0,0,1,1,-1,0,2,5},
    {0,0,0,1,0,0,0,1,1,7},
    {0,0,0,1,0,0,0,-1,1,4},
    {0,0,0,1,-1,-1,0,-1,2,7},
    {1,0,0,0,0,0,0,1,1,6},
    {0,0,-1,1,0,0,-1,-1,2,0},
    {0,0,-1,0,-1,0,0,0,1,0},
    {1,0,0,0,0,0,0,0,0,0},
    {1,0,1,1,0,1,0,0,2,2},
    {0,-1,0,0,0,0,0,1,1,1},
    {1,1,0,0,0,0,-1,1,2,3},
    {0,1,1,0,-1,0,0,-1,2,0},
    {1,1,-1,0,1,0,0,0,2,6},
    {-1,-1,1,0,0,0,1,0,2,1},
    {-1,1,0,0,0,1,1,0,2,6},
    {0,-1,0,0,0,0,1,0,1,4},
    {-1,0,-1,1,0,0,0,-1,2,5},
    {1,0,0,-1,1,0,0,1,2,5},
    {1,0,0,0,-1,0,0,0,1,4},
    {0,0,1,0,0,-1,0,0,1,4},
    {1,0,0,0,0,0,1,0,1,5},
    {0,-1,0,0,0,0,0,1,1,2},
    {0,1,0,0,0,0,0,0,0,6},
    {0,0,0,0,-1,0,0,0,0,1},
    {0,-1,0,0,0,0,0,1,1,7},
    {0,-1,0,0,-1,1,1,0,2,2},
    {1,-1,-1,0,0,0,0,-1,2,7},
    {0,0,1,0,0,1,0,0,1,2},
    {0,0,1,0,0,0,0,0,0,4},
    {0,0,0,1,0,1,0,0,1,1},
    {0,0,0,1,0,0,0,0,0,2},
    {1,0,0,-1,0,1,-1,0,2,1},
    {1,1,0,0,1,-1,0,0,2,6},
    {-1,0,0,0,0,0,0,1,1,2},
    {1,1,-1,0,0,0,0,-1,2,5},
    {0,0,-1,0,0,0,0,1,1,2},
    {-1,0,-1,-1,0,-1,0,0,2,2},
    {-1,1,0,1,0,0,1,0,2,5},
    {0,1,0,0,1,0,0,0,1,6},
    {-1,0,0,0,0,0,0,1,1,1},
    {1,-1,-1,0,0,0,0,1,2,2},
    {0,-1,0,0,-1,0,0,0,1,7},
    {0,1,0,0,0,0,0,0,0,1},
    {0,0,0,-1,0,0,-1,0,1,3},
    {0,0,0,1,1,0,0,0,1,3},
    {-1,0,0,0,1,1,0,1,2,0},
    {1,0,0,-1,0,1,-1,0,2,6},
    {0,0,0,-1,-1,-1,0,-1,2,5},
    {0,0,0,-1,0,0,1,0,1,5},
    {1,0,0,0,0,0,0,-1,1,1},
    {0,0,0,1,0,0,1,0,1,3},
    {0,1,0,0,0,0,0,1,1,0},
    {0,-1,0,0,0,-1,0,0,1,5},
    {0,0,0,1,-1,-1,0,-1,2,2},
    {0,0,-1,0,0,0,0,0,0,1},
    {0,0,0,1,0,0,-1,0,1,0},
    {0,-1,1,1,0,-1,0,0,2,5},
    {1,0,1,1,0,0,0,-1,2,6},
    {0,0,1,0,0,0,1,0,1,3},
    {0,0,1,1,0,0,1,-1,2,6},
    {0,1,0,0,1,-1,0,-1,2,6},
    {0,0,0,-1,0,-1,0,0,1,2},
    {-1,0,0,0,-1,-1,1,0,2,0},
    {0,1,0,0,0,0,0,1,1,4},
    {0,-1,-1,0,0,0,1,-1,2,0},
    {-1,0,0,0,-1,1,1,0,2,7},
    {0,1,0,0,0,1,0,0,1,4},
    {0,0,1,-1,-1,0,0,1,2,0},
    {0,1,0,0,0,-1,0,0,1,5},
    {-1,0,0,0,0,-1,-1,1,2,1},
    {0,1,-1,-1,1,0,0,0,2,4},
    {0,0,1,0,1,0,1,1,2,0},
    {0,-1,-1,0,0,0,-1,1,2,0},
    {0,-1,0,0,1,0,1,-1,2,7},
    {0,-1,0,0,0,0,0,0,0,5},
    {0,0,-1,0,-1,-1,1,0,2,5},
    {0,0,0,1,0,-1,0,0,1,5},
    {0,0,0,-1,0,0,0,-1,1,1},
    {0,0,0,-1,0,-1,0,0,1,1},
    {0,-1,1,0,-1,0,0,1,2,0},
    {1,-1,0,0,-1,-1,0,0,2,4},
    {1,0,0,1,-1,0,0,1,2,2},
    {-1,0,0,0,-1,0,0,0,1,4},
    {0,1,-1,0,1,1,0,0,2,7},
    {-1,0,0,0,0,-1,1,1,2,4},
    {0,0,1,0,-1,0,0,0,1,1},
    {1,-1,0,0,-1,-1,0,0,2,1},
    {1,1,-1,0,0,0,0,-1,2,3},
    {0,0,0,1,0,0,1,0,1,0},
    {0,0,0,-1,0,1,0,0,1,4},
    {1,0,0,1,0,0,1,1,2,3},
    {0,0,0,1,0,0,-1,0,1,6},
    {0,1,0,0,0,1,1,-1,2,4},
    {0,-1,0,0,-1,0,0,0,1,3},
    {0,0,-1,-1,-1,1,0,0,2,4},
    {0,1,0,0,0,1,0,0,1,5},
    {0,1,-1,0,0,-1,-1,0,2,5},
    {1,0,0,0,0,0,0,1,1,7},
    {0,0,0,1,1,-1,0,-1,2,5},
    {0,1,0,0,1,0,1,-1,2,2},
    {0,0,0,1,0,1,-1,-1,2,4},
    {0,0,1,0,1,0,0,0,1,5},
    {0,0,-1,0,-1,0,0,0,1,3},
    {0,0,0,-1,1,-1,0,-1,2,3},
    {1,0,0,0,0,0,0,-1,1,3},
    {0,0,0,-1,-1,0,0,0,1,7},
    {1,0,0,0,0,1,1,-1,2,5},
    {0,0,0,-1,0,0,0,0,0,7},
    {1,-1,0,-1,0,0,-1,0,2,3},
    {-1,0,0,0,-1,-1,0,-1,2,1},
    {1,1,-1,0,0,0,-1,0,2,2},
    {-1,0,0,0,0,0,0,-1,1,1},
    {-1,-1,1,0,0,0,0,1,2,2},
    {1,0,0,0,0,-1,0,0,1,7},
    {1,0,1,1,0,1,0,0,2,6},
    {0,0,0,1,0,-1,0,0,1,7},
    {0,0,1,0,-1,1,1,0,2,7},
    {0,1,0,0,0,1,1,-1,2,2},
    {0,1,1,0,0,-1,1,0,2,6},
    {0,0,0,-1,-1,0,-1,-1,2,5},
    {0,0,-1,0,1,0,0,0,1,5},
    {0,-1,1,1,1,0,0,0,2,1},
    {0,-1,1,1,1,0,0,0,2,6},
    {-1,0,0,0,0,1,0,0,1,5},
    {1,0,1,-1,0,0,0,1,2,3},
    {0,0,0,1,0,0,0,-1,1,1},
    {1,0,0,0,0,-1,1,1,2,4},
    {0,1,1,-1,1,0,0,0,2,6},
    {-1,-1,0,0,0,0,-1,1,2,1},
    {-1,-1,0,0,1,-1,0,0,2,7},
    {0,0,0,-1,-1,1,1,0,2,2},
    {0,-1,-1,1,1,0,0,0,2,5},
    {0,1,-1,-1,0,1,0,0,2,7},
    {0,0,0,1,1,0,1,-1,2,6},
    {-1,-1,0,-1,0,0,1,0,2,5},
    {0,-1,0,0,0,0,1,0,1,7},
    {0,-1,0,0,1,-1,0,-1,2,4},
    {1,0,0,0,0,1,1,-1,2,6},
    {0,1,1,-1,1,0,0,0,2,7},
    {1,-1,0,0,0,0,-1,-1,2,7},
    {1,1,0,1,0,0,-1,0,2,2},
    {1,1,0,0,0,-1,1,0,2,6},
    {0,0,1,0,0,0,1,0,1,2},
    {0,1,-1,0,1,1,0,0,2,3},
    {-1,0,0,0,1,0,1,-1,2,7},
    {-1,0,0,0,-1,0,0,0,1,1},
    {0,0,1,-1,-1,0,0,1,2,2},
    {-1,0,0,1,0,-1,1,0,2,3},
    {0,-1,1,1,-1,0,0,0,2,5},
    {0,-1,0,0,-1,0,-1,-1,2,3},
    {1,-1,0,-1,0,0,0,-1,2,4},
    {0,0,-1,0,0,0,0,0,0,5},
    {0,0,1,0,0,1,0,0,1,7},
    {-1,0,-1,1,0,1,0,0,2,6},
    {0,1,0,0,-1,0,-1,1,2,4},
    {0,0,-1,0,0,0,-1,0,1,0},
    {0,-1,0,0,0,0,1,0,1,5},
    {0,0,0,-1,-1,0,0,0,1,0},
    {-1,0,0,0,0,1,-1,-1,2,0},
    {1,0,0,0,0,-1,0,0,1,3},
    {-1,-1,1,0,0,0,0,1,2,5},
    {0,0,1,1,1,-1,0,0,2,5},
    {0,0,0,1,-1,0,-1,1,2,5},
    {0,0,0,-1,1,1,-1,0,2,7},
    {0,0,0,1,0,0,0,1,1,5},
    {1,0,0,0,-1,0,-1,1,2,1},
    {0,0,1,-1,0,0,1,1,2,3},
    {0,1,0,0,0,0,1,0,1,4},
    {0,0,-1,0,0,-1,0,0,1,3},
    {0,-1,-1,1,0,0,1,0,2,0},
    {0,1,0,0,0,1,-1,-1,2,1},
    {0,-1,0,0,0,1,-1,-1,2,3},
    {0,0,0,-1,0,0,0,-1,1,4},
    {-1,-1,0,-1,0,0,0,1,2,6},
    {-1,0,0,0,0,0,1,0,1,3},
    {-1,0,0,0,-1,0,0,0,1,5},
    {0,0,0,1,0,0,0,1,1,1},
    {0,0,0,1,1,1,-1,0,2,4},
    {0,0,1,0,1,1,-1,0,2,4},
    {-1,0,0,0,0,0,0,0,0,6},
    {0,0,0,-1,0,0,0,1,1,6},
    {-1,1,0,0,0,0,1,1,2,1},
    {0,-1,0,0,0,0,0,0,0,4},
    {0,1,-1,-1,0,0,-1,0,2,0},
    {1,0,0,0,1,-1,-1,0,2,2},
    {1,0,0,0,0,-1,0,0,1,2},
    {0,0,1,0,0,0,1,0,1,7},
    {-1,0,0,0,0,0,-1,0,1,3},
    {0,0,1,0,-1,-1,0,-1,2,6},
    {0,-1,-1,1,0,-1,0,0,2,1},
    {-1,0,0,0,0,0,-1,0,1,2},
    {1,0,0,0,1,-1,-1,0,2,5},
    {0,1,0,0,-1,0,0,0,1,7},
    {0,0,1,0,0,0,0,1,1,2},
    {-1,0,0,0,0,0,0,-1,1,6},
    {-1,0,-1,-1,0,0,-1,0,2,1},
    {0,0,0,1,0,-1,1,1,2,0},
    {0,-1,-1,1,1,0,0,0,2,4},
    {0,0,-1,0,-1,1,0,1,2,2},
    {0,1,0,0,1,1,0,1,2,6},
    {0,0,1,-1,1,0,0,-1,2,2},
    {0,0,1,0,0,0,0,-1,1,1},
    {0,1,0,0,0,-1,0,0,1,7},
    {0,0,1,0,1,0,0,0,1,3},
    {0,-1,-1,1,0,1,0,0,2,7},
    {1,1,0,1,1,0,0,0,2,4},
    {0,0,0,1,0,0,1,0,1,2},
    {-1,0,0,0,0,0,0,1,1,5},
    {-1,0,-1,1,0,-1,0,0,2,3},
    {0,0,0,1,1,0,1,-1,2,4},
    {1,1,0,1,0,0,1,0,2,2},
    {1,1,-1,0,0,0,0,-1,2,1},
    {1,0,0,0,0,-1,-1,1,2,7},
    {0,1,-1,-1,0,-1,0,0,2,4},
    {0,-1,1,1,0,0,0,1,2,4},
    {1,0,1,-1,0,-1,0,0,2,1},
    {0,0,0,1,0,0,-1,0,1,7},
    {1,1,0,0,0,0,-1,1,2,7},
    {0,1,1,-1,0,-1,0,0,2,2},
    {0,0,1,1,-1,0,0,-1,2,1},
    {0,1,0,0,0,0,0,-1,1,1},
    {0,0,0,1,0,0,0,-1,1,7},
    {0,0,0,-1,1,1,-1,0,2,0},
    {0,0,1,0,-1,1,0,1,2,7},
    {0,0,0,1,0,0,0,0,0,0},
    {-1,0,0,0,1,0,0,0,1,3},
    {0,1,0,0,-1,-1,1,0,2,1},
    {0,0,-1,0,1,0,1,-1,2,6},
    {1,0,0,0,1,0,0,0,1,2},
    {0,0,-1,-1,0,1,-1,0,2,6},
    {0,0,-1,0,0,0,0,1,1,3},
    {1,-1,-1,0,-1,0,0,0,2,7},
    {0,0,1,-1,0,1,1,0,2,3},
    {-1,-1,0,0,0,0,1,-1,2,4},
    {1,1,0,0,0,0,-1,1,2,4},
    {-1,-1,0,0,1,-1,0,0,2,5},
    {0,1,-1,0,1,0,0,-1,2,5},
    {0,-1,0,0,0,1,0,0,1,0},
    {0,0,0,1,0,-1,0,0,1,3},
    {-1,0,0,0,0,0,-1,0,1,6},
    {0,0,1,0,0,-1,0,0,1,1},
    {-1,-1,0,-1,0,0,-1,0,2,3},
    {0,-1,1,0,0,1,1,0,2,6},
    {0,-1,-1,1,0,0,0,1,2,1},
    {0,0,-1,0,0,-1,0,0,1,5},
    {-1,0,0,-1,0,1,1,0,2,2},
    {0,0,-1,0,1,0,1,-1,2,1},
    {-1,-1,0,-1,-1,0,0,0,2,7},
    {-1,-1,0,0,-1,1,0,0,2,6},
    {0,-1,0,0,0,0,0,0,0,1},
    {-1,0,0,1,0,-1,1,0,2,0},
    {-1,0,-1,1,0,-1,0,0,2,2},
    {0,0,-1,0,0,1,1,-1,2,3},
    {1,1,0,1,1,0,0,0,2,5},
    {0,-1,0,0,0,-1,1,1,2,6},
    {0,0,-1,0,0,0,0,0,0,7},
    {0,0,0,-1,0,-1,0,0,1,0},
    {0,0,-1,-1,0,0,1,-1,2,2},
    {1,0,0,0,-1,-1,0,-1,2,4},
    {-1,0,0,0,0,-1,0,0,1,5},
    {0,-1,-1,0,0,0,1,-1,2,3},
    {-1,1,0,0,0,0,-1,-1,2,0},
    {0,0,0,1,-1,0,0,0,1,4},
    {1,0,0,0,-1,1,0,1,2,0},
    {0,0,-1,0,0,0,-1,0,1,2},
    {0,-1,0,0,0,0,0,-1,1,6},
    {0,0,1,-1,-1,0,0,1,2,3},
    {1,1,0,1,0,0,-1,0,2,1},
    {-1,0,-1,-1,0,0,0,-1,2,2},
    {0,0,1,0,0,0,0,-1,1,3},
    {0,1,1,0,0,1,-1,0,2,7},
    {1,0,0,0,0,0,-1,0,1,7},
    {1,0,0,0,-1,1,1,0,2,3},
    {0,-1,0,0,-1,0,0,0,1,5},
    {-1,1,0,1,0,1,0,0,2,5},
    {0,-1,0,0,0,-1,0,0,1,3},
    {0,0,1,0,-1,1,0,1,2,0},
    {0,-1,-1,0,1,-1,0,0,2,0},
    {0,1,0,0,0,1,0,0,1,1},
    {1,1,0,0,-1,0,0,-1,2,1},
    {1,1,0,0,1,-1,0,0,2,4},
    {0,0,0,0,0,-1,0,0,0,2},
    {-1,0,0,0,0,1,1,-1,2,6},
    {0,0,1,1,0,1,-1,0,2,4},
    {-1,0,0,0,0,0,1,0,1,6},
    {0,0,1,0,-1,0,-1,-1,2,3},
    {-1,0,0,1,0,0,-1,1,2,5},
    {1,0,0,0,0,0,-1,0,1,6},
    {0,-1,-1,0,1,0,0,1,2,7},
    {0,0,1,0,-1,1,0,1,2,5},
    {0,0,-1,0,0,0,0,-1,1,5},
    {1,1,-1,0,1,0,0,0,2,2},
    {1,0,0,0,0,0,0,1,1,2},
    {-1,0,0,0,0,0,0,1,1,0},
    {0,1,0,0,0,1,1,-1,2,6},
    {0,0,0,-1,0,0,-1,0,1,2},
    {0,0,1,0,0,0,0,0,0,5},
    {0,0,0,-1,-1,-1,1,0,2,6},
    {0,0,0,-1,0,1,-1,-1,2,7},
    {0,1,0,0,0,-1,0,0,1,3},
    {-1,1,1,0,0,-1,0,0,2,2},
    {0,0,1,0,-1,-1,0,-1,2,2},
    {0,1,0,0,0,1,1,-1,2,3},
    {0,0,-1,1,0,0,-1,-1,2,2},
    {0,0,0,-1,1,1,-1,0,2,5},
    {1,-1,-1,0,-1,0,0,0,2,5},
    {0,-1,0,0,0,-1,0,0,1,1},
    {0,0,0,-1,0,-1,-1,1,2,5},
    {1,0,0,-1,-1,0,0,-1,2,1},
    {1,1,-1,0,1,0,0,0,2,4},
    {0,0,0,-1,1,-1,-1,0,2,5},
    {0,0,0,-1,0,0,0,0,0,4},
    {1,0,0,1,-1,0,0,1,2,7},
    {-1,0,0,0,0,0,0,0,0,7},
    {0,-1,0,0,0,0,1,0,1,3},
    {0,0,-1,0,0,0,1,0,1,2},
    {0,-1,1,1,0,1,0,0,2,7},
    {0,0,0,-1,1,-1,-1,0,2,6},
    {0,0,1,0,0,0,0,0,0,2},
    {0,0,0,1,1,0,1,1,2,0},
    {1,0,0,0,0,0,0,0,0,2},
    {0,1,0,0,0,1,0,0,1,2},
    {0,0,-1,0,0,0,-1,0,1,1},
    {0,-1,0,0,0,-1,0,0,1,6},
    {1,-1,0,0,-1,0,0,1,2,0},
    {1,0,0,0,1,0,0,0,1,7},
    {0,0,0,1,0,0,0,1,1,6},
    {-1,-1,0,-1,-1,0,0,0,2,1},
    {0,0,-1,0,0,0,-1,0,1,5},
    {0,0,1,0,-1,1,0,1,2,4},
    {0,0,0,-1,0,0,0,1,1,2},
    {0,-1,0,0,0,1,0,0,1,5},
    {1,1,-1,0,0,1,0,0,2,5},
    {0,-1,0,0,-1,0,0,0,1,2},
    {-1,0,0,0,0,-1,-1,1,2,3},
    {1,-1,0,-1,0,-1,0,0,2,6},
    {-1,0,-1,1,0,0,0,-1,2,7},
    {0,-1,0,0,0,1,0,0,1,3},
    {0,-1,0,0,1,0,0,0,1,2},
    {-1,-1,0,0,0,0,-1,1,2,5},
    {0,0,0,1,1,0,0,0,1,7},
    {1,0,0,-1,0,0,1,-1,2,1},
    {1,0,0,0,1,0,0,0,1,1},
    {1,0,0,0,0,0,-1,0,1,0},
    {0,0,0,1,0,1,0,0,1,0},
    {0,0,-1,0,0,0,-1,0,1,7},
    {-1,0,-1,1,0,0,1,0,2,4},
    {1,1,0,0,-1,0,0,-1,2,6},
    {0,0,0,-1,0,0,-1,0,1,0},
    {0,-1,1,1,0,1,0,0,2,4},
    {-1,0,0,0,-1,-1,0,-1,2,7},
    {-1,0,0,0,0,1,0,0,1,1},
    {-1,0,0,0,-1,0,0,0,1,7},
    {0,0,-1,0,-1,0,0,0,1,4},
    {1,-1,0,0,1,1,0,0,2,7},
    {0,0,0,-1,-1,0,-1,-1,2,2},
    {0,-1,-1,0,1,-1,0,0,2,3},
    {0,0,0,-1,0,0,-1,0,1,5},
    {0,-1,0,0,0,-1,0,0,1,7},
    {0,0,-1,0,0,0,0,1,1,1},
    {-1,1,0,1,0,0,0,-1,2,7},
    {0,0,0,1,0,0,0,-1,1,6},
    {0,0,0,-1,0,0,0,-1,1,7},
    {-1,-1,0,-1,0,1,0,0,2,6},
    {0,0,1,1,0,0,1,-1,2,0},
    {0,1,0,0,0,1,0,0,1,3},
    {-1,0,-1,-1,0,0,0,1,2,4},
    {0,-1,0,0,0,0,1,0,1,1},
    {-1,-1,0,-1,0,0,-1,0,2,2},
    {0,0,0,1,0,0,1,0,1,5},
    {0,1,-1,-1,-1,0,0,0,2,2},
    {1,1,0,0,-1,0,0,-1,2,0},
    {0,-1,0,0,-1,0,0,0,1,1},
    {-1,1,0,1,-1,0,0,0,2,4},
    {1,1,0,1,1,0,0,0,2,1},
    {0,1,-1,0,0,0,-1,-1,2,2},
    {0,-1,-1,1,0,0,1,0,2,5},
    {1,0,0,0,0,0,1,0,1,3},
    {0,0,-1,1,0,0,1,1,2,1},
    {0,1,1,-1,0,1,0,0,2,0},
    {0,1,0,0,1,0,0,0,1,3},
    {0,1,0,0,1,0,0,0,1,2},
    {0,0,0,0,0,0,1,0,0,1},
    {0,-1,0,0,0,1,1,-1,2,7},
    {0,0,0,0,0,0,1,0,0,4},
    {1,1,0,0,0,0,1,-1,2,5},
    {0,1,1,0,1,-1,0,0,2,0},
    {-1,0,-1,1,0,1,0,0,2,3},
    {0,0,-1,0,0,1,0,0,1,1},
    {0,0,0,1,-1,-1,1,0,2,7},
    {0,0,0,-1,1,1,-1,0,2,4},
    {1,0,0,0,0,0,0,-1,1,4},
    {1,0,0,0,0,0,-1,0,1,2},
    {-1,-1,0,-1,0,0,1,0,2,3},
    {0,1,-1,-1,-1,0,0,0,2,6},
    {-1,0,0,0,1,-1,-1,0,2,5},
    {-1,0,0,-1,0,0,1,1,2,3},
    {0,0,-1,0,0,0,0,0,0,3},
    {0,0,0,-1,0,-1,-1,1,2,0},
    {-1,0,0,0,1,1,-1,0,2,5},
    {0,1,-1,0,0,0,-1,-1,2,3},
    {0,-1,1,1,1,0,0,0,2,4},
    {1,0,0,0,0,0,0,1,1,1},
    {-1,-1,1,0,0,0,0,-1,2,4},
    {0,1,0,0,0,0,1,0,1,6},
    {0,0,1,0,0,1,0,0,1,1},
    {0,0,0,1,0,1,0,0,1,2},
    {0,0,1,1,1,0,0,1,2,5},
    {0,-1,-1,1,-1,0,0,0,2,0},
    {-1,-1,1,0,0,1,0,0,2,3},
    {0,0,0,1,0,0,1,0,1,1},
    {0,0,0,1,0,1,0,0,1,7},
    {-1,0,0,0,0,1,-1,-1,2,7},
    {0,0,0,1,1,-1,-1,0,2,6},
    {0,-1,0,0,-1,0,0,0,1,0},
    {0,0,1,0,-1,0,-1,-1,2,0},
    {0,1,-1,-1,-1,0,0,0,2,7},
    {0,0,0,1,0,0,0,0,0,6},
    {1,0,0,0,1,0,1,-1,2,0},
    {0,0,0,1,0,0,-1,0,1,3},
    {0,-1,0,0,-1,0,-1,-1,2,7},
    {-1,0,-1,1,0,-1,0,0,2,5},
    {-1,0,-1,1,0,0,-1,0,2,4},
    {0,0,-1,0,1,0,1,1,2,6},
    {-1,0,0,0,0,-1,0,0,1,4},
    {0,0,0,-1,0,0,1,0,1,2},
    {-1,0,0,0,0,-1,1,1,2,2},
    {0,-1,1,1,0,0,-1,0,2,2},
    {0,0,-1,1,0,0,-1,-1,2,6},
    {0,-1,0,0,-1,-1,0,-1,2,4},
    {0,0,0,1,-1,-1,1,0,2,0},
    {0,0,-1,0,1,-1,-1,0,2,0},
    {0,0,0,-1,0,1,0,0,1,1},
    {-1,0,0,0,0,0,0,1,1,6},
    {0,0,0,1,0,0,0,0,0,7},
    {0,0,0,-1,0,0,0,1,1,4},
    {0,0,-1,0,0,-1,0,0,1,0},
    {0,-1,1,0,0,0,-1,-1,2,7},
    {0,-1,0,0,0,0,0,1,1,0},
    {0,0,0,-1,0,0,-1,0,1,1},
    {1,0,0,0,0,0,0,-1,1,7},
    {-1,1,0,1,0,0,1,0,2,4},
    {0,1,-1,0,-1,0,0,1,2,2},
    {0,0,1,0,0,1,0,0,1,4},
    {0,-1,1,0,0,0,1,1,2,0},
    {1,-1,0,-1,0,0,1,0,2,6},
    {0,1,0,0,0,0,0,-1,1,5},
    {-1,-1,0,-1,0,1,0,0,2,0},
    {-1,0,0,0,-1,0,-1,-1,2,4},
    {1,0,0,0,0,0,0,-1,1,0},
    {0,0,0,-1,0,0,-1,0,1,4},
    {0,-1,0,0,-1,-1,0,-1,2,7},
    {-1,0,-1,-1,0,-1,0,0,2,6},
    {1,1,0,1,-1,0,0,0,2,3},
    {-1,-1,1,0,0,0,1,0,2,6},
    {-1,0,0,-1,1,1,0,0,2,2},
    {0,0,1,0,0,0,1,0,1,4},
    {-1,1,1,0,0,0,-1,0,2,1},
    {0,0,0,-1,0,0,0,-1,1,3},
    {0,0,1,1,0,0,1,-1,2,4},
    {0,0,-1,0,1,1,0,1,2,1},
    {-1,0,0,0,0,0,0,-1,1,5},
    {0,0,1,0,1,1,-1,0,2,5},
    {-1,1,0,0,0,0,-1,-1,2,2},
    {0,1,0,0,1,-1,0,-1,2,5},
    {0,0,1,0,-1,0,-1,-1,2,4},
    {1,0,0,0,0,-1,0,0,1,1},
    {-1,1,1,0,0,0,1,0,2,5},
    {0,0,1,0,1,1,0,1,2,1},
    {0,-1,0,0,-1,0,-1,-1,2,1},
    {0,0,1,1,1,-1,0,0,2,3},
    {-1,0,0,1,-1,1,0,0,2,5},
    {-1,1,1,0,0,0,0,-1,2,2},
    {-1,0,0,-1,0,1,1,0,2,1},
    {-1,0,0,0,0,-1,1,1,2,5},
    {0,0,-1,-1,1,-1,0,0,2,1},
    {1,-1,-1,0,0,0,0,-1,2,1},
    {0,0,-1,0,-1,0,-1,-1,2,3},
    {0,0,-1,0,-1,0,-1,1,2,7},
    {-1,-1,0,0,1,-1,0,0,2,2},
    {0,0,0,1,0,0,1,0,1,6},
    {0,0,1,0,0,0,0,1,1,5},
    {-1,0,-1,1,0,1,0,0,2,2},
    {0,0,1,0,0,0,0,-1,1,6},
    {-1,0,-1,1,0,-1,0,0,2,1},
    {-1,-1,1,0,0,0,1,0,2,7},
    {0,0,-1,0,0,0,1,0,1,6},
    {0,0,1,0,-1,-1,1,0,2,7},
    {0,1,0,0,0,0,-1,0,1,0},
    {0,0,-1,1,0,0,1,1,2,2},
    {0,1,0,0,-1,0,0,0,1,1},
    {-1,0,0,0,0,-1,0,0,1,2},
    {0,0,0,-1,0,1,0,0,1,0},
    {-1,0,0,1,0,0,1,-1,2,6},
    {-1,0,0,0,-1,0,0,0,1,6},
    {1,-1,-1,0,0,0,0,1,2,1},
    {1,0,0,-1,0,0,-1,1,2,2},
    {-1,0,0,1,0,0,-1,1,2,0},
    {1,0,0,1,0,1,1,0,2,2},
    {0,0,-1,0,0,0,0,1,1,6},
    {-1,0,-1,1,0,0,0,-1,2,1},
    {-1,0,0,0,0,0,0,0,0,1},
    {1,0,0,0,1,0,0,0,1,5},
    {0,0,0,-1,-1,0,0,0,1,5},
    {0,1,1,0,1,-1,0,0,2,5},
    {0,0,-1,0,0,0,0,1,1,7},
    {-1,0,0,0,0,1,0,0,1,3},
    {1,0,0,0,0,-1,0,0,1,4},
    {0,1,0,0,0,0,0,1,1,7},
    {0,1,0,0,-1,0,0,0,1,4},
    {-1,0,-1,1,-1,0,0,0,2,7},
    {0,0,1,-1,0,0,-1,-1,2,3},
    {1,0,0,-1,0,0,-1,1,2,1},
    {0,-1,0,0,1,0,1,-1,2,3},
    {1,0,0,-1,0,0,-1,1,2,3},
    {0,-1,-1,1,0,0,0,-1,2,6},
    {0,0,-1,1,-1,0,0,1,2,2},
    {0,0,-1,0,0,-1,0,0,1,1},
    {0,0,0,-1,-1,-1,0,-1,2,1},
    {1,-1,-1,0,0,0,-1,0,2,5},
    {1,0,0,0,-1,0,0,0,1,2},
    {-1,0,0,1,0,1,-1,0,2,0},
    {0,1,1,0,0,0,1,-1,2,4},
    {1,0,1,-1,0,-1,0,0,2,4},
    {-1,0,0,1,1,0,0,1,2,3},
    {0,-1,-1,0,1,-1,0,0,2,5},
    {0,-1,-1,0,0,-1,1,0,2,4},
    {0,0,1,0,-1,0,0,0,1,5},
    {0,0,-1,0,1,0,1,-1,2,2},
    {1,0,0,-1,-1,0,0,-1,2,5},
    {-1,1,0,1,0,0,-1,0,2,5},
    {-1,0,0,0,0,-1,1,1,2,0},
    {0,1,0,0,0,0,0,0,0,0},
    {-1,0,0,0,0,0,0,-1,1,7},
    {1,1,0,0,0,0,1,-1,2,6},
    {0,1,0,0,1,-1,-1,0,2,3},
    {-1,0,0,0,0,1,-1,-1,2,4},
    {1,0,1,-1,0,0,0,1,2,0},
    {0,-1,0,0,0,0,-1,0,1,7},
    {0,0,1,0,-1,1,0,1,2,2},
    {1,0,0,0,0,0,0,1,1,3},
    {0,-1,-1,1,-1,0,0,0,2,1},
    {1,1,-1,0,0,0,0,1,2,2},
    {1,0,1,-1,0,-1,0,0,2,2},
    {1,1,0,1,0,0,-1,0,2,4},
    {0,-1,0,0,1,1,0,1,2,6},
    {1,-1,-1,0,1,0,0,0,2,7},
    {0,-1,0,0,-1,-1,1,0,2,7},
    {-1,-1,0,0,-1,1,0,0,2,2},
    {0,0,0,1,0,0,0,-1,1,0},
    {1,0,0,0,-1,-1,0,-1,2,5},
    {0,0,-1,-1,1,0,0,1,2,2},
    {0,0,1,0,0,0,-1,0,1,2},
    {-1,0,-1,-1,0,-1,0,0,2,5},
    {0,0,-1,-1,0,-1,1,0,2,3},
    {0,0,1,0,0,1,-1,-1,2,0},
    {0,0,1,1,-1,0,0,-1,2,2},
    {0,0,0,-1,0,0,0,-1,1,2},
    {1,0,0,0,0,1,0,0,1,4},
    {0,-1,-1,1,0,0,0,1,2,5},
    {-1,0,0,1,1,-1,0,0,2,0},
    {0,-1,0,0,0,1,0,0,1,4},
    {0,0,-1,0,1,1,-1,0,2,6},
    {1,-1,0,-1,-1,0,0,0,2,4},
    {0,-1,0,0,-1,0,-1,1,2,5},
    {1,0,0,-1,0,0,-1,1,2,6},
    {0,-1,-1,1,-1,0,0,0,2,6},
    {0,1,0,0,0,1,0,0,1,6},
    {0,1,0,0,-1,-1,0,-1,2,6},
    {1,-1,-1,0,-1,0,0,0,2,4},
    {1,0,0,0,-1,0,0,0,1,7},
    {0,-1,-1,0,0,-1,1,0,2,3},
    {0,1,0,0,0,1,1,-1,2,7},
    {0,-1,0,0,-1,0,-1,1,2,3},
    {0,0,1,0,0,0,0,1,1,4},
    {0,0,-1,0,1,0,1,1,2,3},
    {0,0,0,1,-1,0,0,0,1,1},
    {0,1,0,0,0,-1,0,0,1,6},
    {0,0,1,0,-1,0,-1,1,2,7},
    {-1,-1,0,-1,0,0,-1,0,2,0},
    {0,0,0,1,-1,0,0,0,1,7},
    {0,0,-1,0,0,0,1,0,1,4},
    {1,-1,-1,0,0,0,0,1,2,3},
    {1,0,0,0,0,-1,0,0,1,6},
    {-1,-1,0,0,1,-1,0,0,2,3},
    {0,-1,1,1,-1,0,0,0,2,7},
    {0,0,-1,0,1,0,0,0,1,3},
    {-1,0,0,0,0,0,-1,0,1,7},
    {-1,0,-1,1,0,0,0,1,2,1},
    {-1,-1,0,0,-1,1,0,0,2,1},
    {-1,0,-1,1,0,0,-1,0,2,7},
    {0,0,0,1,1,1,0,1,2,7},
    {0,0,0,-1,1,1,-1,0,2,6},
    {0,0,1,-1,0,0,1,1,2,4},
    {0,0,1,0,1,-1,0,-1,2,0},
    {-1,0,0,-1,-1,-1,0,0,2,3},
    {0,1,0,0,0,-1,0,0,1,4},
    {0,1,0,0,-1,0,0,0,1,2},
    {0,0,0,0,0,0,-1,0,0,5},
    {0,1,0,0,0,0,0,0,0,5},
    {0,0,1,0,0,0,-1,0,1,6},
    {1,0,0,0,1,-1,0,-1,2,0},
    {0,-1,0,0,0,0,0,-1,1,1},
    {-1,-1,1,0,0,1,0,0,2,4},
    {0,1,0,0,0,-1,0,0,1,0},
    {-1,1,0,1,0,0,0,1,2,5},
    {0,0,0,-1,1,1,0,1,2,0},
    {0,0,1,0,-1,0,0,0,1,2},
    {0,0,0,-1,1,-1,0,-1,2,7},
    {0,0,1,0,0,0,-1,0,1,7},
    {-1,1,1,0,0,0,0,-1,2,7},
    {0,1,0,0,0,1,0,0,1,0},
    {1,1,0,1,0,0,1,0,2,7},
    {1,1,0,1,0,0,0,-1,2,0},
    {0,1,0,0,0,-1,0,0,1,1},
    {1,-1,0,0,0,0,1,1,2,7},
    {-1,0,0,0,1,0,0,0,1,1},
    {0,0,-1,0,0,1,0,0,1,4},
    {1,0,0,1,0,0,1,1,2,5},
    {-1,0,0,-1,1,1,0,0,2,6},
    {-1,0,0,0,0,-1,0,0,1,3},
    {-1,0,0,1,0,1,-1,0,2,5},
    {-1,0,0,0,1,0,1,1,2,0},
    {-1,0,0,-1,0,0,-1,-1,2,6},
    {0,0,0,1,0,1,0,0,1,6},
    {0,1,-1,0,0,1,1,0,2,1},
    {0,-1,0,0,0,1,0,0,1,1},
    {0,0,1,0,1,0,1,-1,2,7},
    {0,0,1,-1,1,0,0,-1,2,0},
    {0,0,-1,0,0,0,1,0,1,7},
    {0,1,-1,0,1,0,0,-1,2,6},
    {-1,1,0,0,0,-1,-1,0,2,7},
    {0,-1,0,0,1,1,0,1,2,3},
    {0,0,0,-1,0,0,0,1,1,0},
    {0,0,0,-1,-1,0,0,0,1,3},
    {0,0,0,1,-1,0,0,0,1,6},
    {1,0,1,1,-1,0,0,0,2,4},
    {1,0,0,0,-1,0,0,0,1,5},
    {0,0,1,1,1,-1,0,0,2,0},
    {0,1,0,0,1,1,0,1,2,0},
    {0,0,0,-1,0,0,0,1,1,7},
    {0,1,1,-1,-1,0,0,0,2,6},
    {0,-1,0,0,0,0,-1,0,1,0},
    {0,0,1,0,1,0,0,0,1,7},
    {0,0,0,1,1,-1,-1,0,2,5},
    {0,0,0,-1,1,0,0,0,1,5},
    {0,-1,-1,0,-1,0,0,-1,2,1},
    {-1,0,0,1,0,-1,1,0,2,1},
    {0,0,0,-1,0,0,0,0,0,3},
    {1,0,0,0,1,0,0,0,1,6},
    {0,1,0,0,0,0,1,0,1,7},
    {0,0,1,0,0,-1,-1,1,2,6},
    {0,-1,0,0,0,-1,-1,1,2,5},
    {0,1,0,0,-1,0,0,0,1,5},
    {0,0,0,1,1,-1,0,-1,2,1},
    {0,-1,0,0,0,0,1,0,1,0},
    {1,0,1,-1,0,0,-1,0,2,6},
    {0,-1,0,0,1,0,0,0,1,7},
    {0,-1,0,0,1,0,0,0,1,5},
    {-1,1,1,0,0,0,-1,0,2,0},
    {0,0,-1,0,0,0,0,-1,1,0},
    {1,0,0,0,0,0,1,0,1,4},
    {0,0,-1,-1,-1,0,0,-1,2,5},
    {0,0,1,-1,1,0,0,-1,2,7},
    {-1,0,0,0,1,0,1,-1,2,1},
    {0,0,0,1,-1,1,0,1,2,3},
    {-1,-1,0,0,0,0,-1,1,2,7},
    {-1,-1,1,0,0,1,0,0,2,2},
    {-1,-1,0,-1,0,-1,0,0,2,0},
    {0,0,-1,0,0,1,0,0,1,6},
    {0,1,0,0,-1,0,0,0,1,6},
    {-1,0,0,0,0,0,0,-1,1,0},
    {0,0,-1,-1,0,0,-1,1,2,6},
    {0,1,1,0,0,0,1,-1,2,6},
    {0,0,0,-1,0,0,1,0,1,3},
    {1,0,0,0,0,0,0,-1,1,2},
    {1,1,0,1,0,0,1,0,2,5},
    {1,-1,-1,0,0,0,-1,0,2,6},
    {1,-1,0,-1,0,-1,0,0,2,3},
    {0,0,1,1,0,-1,1,0,2,6},
    {0,0,1,0,0,0,0,1,1,0},
    {0,0,1,1,0,-1,1,0,2,7},
    {0,-1,-1,0,0,0,1,-1,2,2},
    {-1,0,0,1,0,0,-1,1,2,4},
    {0,-1,0,0,1,0,0,0,1,0},
    {0,0,0,-1,0,0,1,0,1,4},
    {0,0,0,1,0,-1,0,0,1,6},
    {-1,1,0,0,0,0,-1,-1,2,7},
    {-1,0,0,0,0,0,0,-1,1,2},
    {0,0,-1,1,-1,0,0,1,2,4},
    {0,0,1,0,0,-1,0,0,1,7},
    {0,0,1,0,0,-1,0,0,1,0},
    {0,0,0,-1,0,0,1,0,1,6},
    {1,0,0,0,1,-1,-1,0,2,4},
    {-1,0,0,-1,1,1,0,0,2,1},
    {1,1,-1,0,0,0,0,-1,2,6},
    {0,1,0,0,1,0,1,1,2,2},
    {0,0,-1,1,1,0,0,-1,2,0},
    {1,0,1,-1,0,-1,0,0,2,5},
    {0,0,1,0,0,0,0,0,0,0},
    {0,0,1,-1,1,1,0,0,2,0},
    {0,0,1,0,0,-1,1,1,2,2},
    {1,0,0,0,0,1,1,-1,2,0},
    {0,1,0,0,-1,0,0,0,1,3},
    {0,0,0,0,-1,0,0,0,0,3},
    {0,0,0,1,0,0,0,0,0,3},
    {1,0,0,0,0,0,0,1,1,5},
    {-1,0,0,-1,0,-1,-1,0,2,7},
    {0,0,1,0,0,0,1,0,1,6},
    {1,-1,-1,0,0,0,0,-1,2,0},
    {-1,1,0,1,0,1,0,0,2,6},
    {1,1,0,0,0,1,-1,0,2,6},
    {0,0,0,1,0,0,-1,0,1,4},
    {0,0,-1,0,-1,1,1,0,2,3},
    {0,0,1,1,1,0,0,1,2,7},
    {1,0,0,1,0,0,1,1,2,2},
    {0,-1,0,0,1,0,0,0,1,3},
    {0,-1,1,0,0,0,1,1,2,7},
    {0,-1,0,0,0,0,-1,0,1,3},
    {0,0,1,0,0,0,-1,0,1,0},
    {0,0,1,0,0,-1,0,0,1,2},
    {-1,0,0,0,1,-1,-1,0,2,7},
    {-1,0,0,0,1,0,1,1,2,7},
    {1,0,0,-1,-1,1,0,0,2,6},
    {0,0,0,1,0,0,-1,0,1,1},
    {-1,1,0,0,-1,-1,0,0,2,7},
    {0,-1,0,0,1,0,0,0,1,6},
    {0,1,0,0,0,0,0,1,1,1},
    {0,0,0,1,0,1,1,-1,2,4},
    {0,0,-1,0,0,1,0,0,1,2},
    {0,-1,1,0,-1,0,0,1,2,5},
    {0,-1,-1,1,0,0,1,0,2,3},
    {1,0,0,0,0,0,0,0,0,4},
    {0,0,0,0,0,0,-1,0,0,1},
    {0,0,1,-1,1,0,0,-1,2,5},
    {0,0,-1,0,0,0,0,0,0,0},
    {0,0,-1,0,1,0,1,1,2,0},
    {1,0,0,1,1,1,0,0,2,0},
    {0,0,-1,-1,0,-1,1,0,2,0},
    {0,0,0,-1,-1,1,1,0,2,7},
    {0,-1,1,1,0,0,-1,0,2,4},
    {0,0,1,0,0,0,0,1,1,7},
    {0,-1,0,0,0,0,0,0,0,2},
    {1,-1,0,0,1,1,0,0,2,3},
    {0,0,-1,0,0,-1,0,0,1,4},
    {0,-1,1,0,0,-1,-1,0,2,6},
    {-1,1,0,0,0,1,1,0,2,3},
    {1,0,0,0,-1,0,0,0,1,1},
    {0,0,1,0,-1,0,0,0,1,4},
    {0,1,-1,0,0,1,1,0,2,6},
    {0,0,-1,0,-1,1,0,1,2,5},
    {-1,1,1,0,-1,0,0,0,2,1},
    {0,0,1,0,0,-1,0,0,1,3},
    {1,0,0,0,0,0,1,0,1,1},
    {-1,0,-1,1,1,0,0,0,2,7},
    {-1,1,1,0,0,-1,0,0,2,0},
    {0,1,1,0,0,0,-1,1,2,5},
    {1,0,1,-1,0,0,-1,0,2,4},
    {-1,1,0,1,1,0,0,0,2,2},
    {0,0,0,-1,0,0,0,-1,1,0},
    {0,-1,0,0,0,0,0,0,0,0},
    {0,0,0,1,0,-1,0,0,1,4},
    {0,0,1,1,0,1,-1,0,2,5},
    {0,1,0,0,-1,0,-1,1,2,3},
    {0,-1,0,0,1,0,0,0,1,1},
    {0,0,-1,0,1,0,0,0,1,6},
    {0,1,0,0,0,-1,0,0,1,2},
    {0,0,1,0,0,0,-1,0,1,5},
    {0,0,0,1,1,-1,-1,0,2,4},
    {-1,-1,0,0,-1,0,0,-1,2,6},
    {0,0,-1,0,1,0,0,0,1,7},
    {0,0,0,1,1,-1,-1,0,2,3},
    {1,0,0,1,1,0,0,-1,2,6},
    {1,0,0,0,0,0,0,-1,1,6},
    {-1,0,0,0,1,1,0,1,2,7},
    {0,0,0,1,0,-1,0,0,1,0},
    {1,0,0,0,0,1,0,0,1,1},
    {0,-1,0,0,1,0,0,0,1,4},
    {0,0,0,-1,0,0,0,1,1,5},
    {0,-1,0,0,0,0,0,-1,1,5},
    {1,0,1,1,0,-1,0,0,2,6},
    {0,0,0,1,-1,1,1,0,2,7},
    {0,0,0,1,1,0,1,-1,2,5},
    {1,0,0,0,0,0,1,0,1,0},
    {0,0,-1,0,1,0,0,0,1,0},
    {0,1,0,0,0,0,0,0,0,3},
    {0,0,1,0,0,0,0,1,1,3},
    {-1,0,0,0,1,0,0,0,1,0},
    {0,1,0,0,-1,-1,0,-1,2,0},
    {1,0,1,-1,-1,0,0,0,2,6},
    {1,-1,0,0,0,0,1,1,2,0},
    {-1,-1,0,0,-1,0,0,-1,2,2},
    {0,0,-1,0,1,0,0,0,1,1},
    {0,0,1,0,1,0,1,1,2,1},
    {-1,0,0,0,0,0,-1,0,1,5},
    {0,1,1,-1,0,0,0,1,2,6},
    {1,0,1,-1,0,1,0,0,2,3},
    {0,-1,-1,0,0,0,-1,1,2,2},
    {0,-1,0,0,0,0,1,0,1,6},
    {0,0,-1,0,0,0,0,0,0,2},
    {0,1,-1,0,-1,-1,0,0,2,1},
    {0,0,-1,0,1,0,0,0,1,2},
    {0,0,-1,-1,1,0,0,1,2,5},
    {0,0,0,-1,-1,0,0,0,1,2},
    {0,0,1,0,-1,1,1,0,2,0},
    {0,0,1,0,0,0,0,0,0,7},
    {0,0,0,-1,1,0,1,-1,2,3},
    {1,1,0,0,0,0,-1,1,2,0},
    {0,-1,0,0,0,0,-1,0,1,1},
    {0,-1,1,0,0,0,1,1,2,3},
    {0,0,-1,0,1,-1,-1,0,2,6},
    {1,0,0,0,-1,1,0,1,2,4},
    {0,0,1,0,-1,0,0,0,1,3},
    {0,-1,1,0,-1,0,0,1,2,2},
    {0,0,1,-1,0,0,-1,-1,2,0},
    {0,0,1,-1,-1,0,0,1,2,6},
    {1,0,0,0,0,0,1,0,1,7},
    {0,0,-1,-1,-1,0,0,-1,2,3},
    {0,0,0,-1,0,1,0,0,1,6},
    {0,1,0,0,0,1,-1,-1,2,0},
    {-1,0,0,-1,0,-1,-1,0,2,4},
    {-1,0,0,0,1,0,0,0,1,2},
    {0,0,0,-1,0,0,1,0,1,1},
    {0,0,0,1,-1,0,0,0,1,3},
    {0,0,1,0,0,0,0,0,0,1},
    {1,0,0,0,0,0,0,0,0,5},
    {0,0,-1,-1,1,-1,0,0,2,4},
    {-1,1,1,0,0,0,0,-1,2,5},
    {1,0,0,0,1,0,1,1,2,3},
    {-1,0,0,0,0,-1,0,0,1,1},
    {0,0,0,1,1,1,0,1,2,6},
    {-1,0,0,0,0,0,0,0,0,5},
    {-1,-1,0,0,1,0,0,1,2,6},
    {-1,1,0,1,0,0,1,0,2,7},
    {0,0,1,0,0,0,0,1,1,6},
    {0,0,-1,0,1,-1,0,-1,2,1},
    {-1,1,1,0,0,0,0,-1,2,1},
    {-1,0,0,-1,0,1,1,0,2,5},
    {0,1,0,0,0,0,1,0,1,0},
    {1,0,0,0,0,1,0,0,1,2},
    {-1,0,-1,-1,1,0,0,0,2,7},
    {-1,1,0,0,0,1,1,0,2,7},
    {-1,0,0,0,0,1,1,-1,2,3},
    {1,-1,-1,0,0,0,0,-1,2,6},
    {0,0,-1,-1,1,-1,0,0,2,2},
    {0,0,-1,0,0,0,0,-1,1,6},
    {0,0,0,-1,0,1,-1,-1,2,5},
    {1,1,0,0,0,0,-1,1,2,5},
    {0,-1,-1,0,1,0,0,1,2,1},
    {0,-1,0,0,0,1,-1,-1,2,0},
    {-1,0,0,0,-1,0,0,0,1,3},
    {1,-1,-1,0,0,0,0,1,2,7},
    {0,1,-1,-1,0,0,1,0,2,3},
    {0,-1,1,0,-1,0,0,1,2,6},
    {1,1,0,1,0,0,1,0,2,3},
    {0,-1,1,1,0,0,0,1,2,0},
    {1,0,0,0,0,1,0,0,1,3},
    {-1,-1,0,-1,0,0,-1,0,2,7},
    {0,1,0,0,1,0,1,-1,2,0},
    {-1,0,0,1,0,1,-1,0,2,7},
    {0,0,1,1,1,-1,0,0,2,1},
    {0,1,1,0,-1,0,0,-1,2,4},
    {0,1,0,0,1,0,0,0,1,5},
    {1,-1,0,0,0,0,1,1,2,1},
    {0,0,0,1,0,1,1,-1,2,2},
    {1,1,0,0,0,0,-1,1,2,6},
    {0,0,-1,-1,-1,1,0,0,2,5},
    {1,0,0,-1,0,-1,1,0,2,0},
    {-1,-1,0,-1,0,1,0,0,2,3},
    {0,1,-1,-1,1,0,0,0,2,2},
    {0,0,-1,-1,-1,1,0,0,2,2},
    {-1,1,0,1,0,0,-1,0,2,4},
    {0,-1,-1,0,1,0,0,1,2,4},
    {0,0,0,1,0,0,0,0,0,5},
    {-1,-1,0,0,0,0,1,-1,2,6},
    {0,1,1,0,-1,0,0,-1,2,5},
    {1,1,-1,0,0,0,-1,0,2,1},
    {1,0,0,-1,0,-1,1,0,2,1},
    {0,-1,1,1,0,0,1,0,2,0},
    {0,0,0,0,0,0,-1,0,0,3},
    {0,0,0,0,0,0,1,0,0,2},
    {1,0,0,0,-1,-1,1,0,2,3},
    {-1,-1,0,-1,0,0,1,0,2,2},
    {1,0,0,-1,1,-1,0,0,2,1},
    {1,1,-1,0,0,0,0,1,2,4},
    {1,1,0,1,0,0,0,-1,2,6},
    {-1,0,0,0,1,0,0,0,1,5},
    {0,-1,-1,0,1,0,0,1,2,5},
    {0,0,0,0,0,0,-1,0,0,0},
    {-1,-1,0,0,0,0,1,-1,2,7},
    {-1,0,0,1,0,1,-1,0,2,1},
    {0,0,0,1,1,1,-1,0,2,2},
    {0,0,1,-1,0,-1,-1,0,2,4},
    {-1,0,0,1,0,0,1,-1,2,0},
    {0,1,1,0,-1,0,0,-1,2,6},
    {0,-1,0,0,-1,0,-1,1,2,6},
    {1,0,0,-1,0,-1,1,0,2,2},
    {-1,0,0,1,0,1,-1,0,2,2},
    {1,1,0,0,0,0,-1,1,2,1},
    {0,1,0,0,0,-1,-1,1,2,3},
    {0,-1,-1,1,0,0,0,1,2,3},
    {0,0,0,-1,0,0,0,0,0,2},
    {0,-1,1,0,-1,-1,0,0,2,5},
    {-1,0,0,0,0,0,-1,0,1,1},
    {-1,0,0,1,0,1,-1,0,2,3},
    {0,0,1,1,-1,1,0,0,2,3},
    {1,1,0,0,0,0,-1,1,2,2},
    {1,0,0,-1,0,-1,1,0,2,4},
    {0,-1,-1,0,1,0,0,1,2,0},
    {-1,0,0,1,0,1,-1,0,2,4},
    {0,0,1,1,1,-1,0,0,2,6},
    {-1,1,1,0,0,0,0,1,2,5},
    {0,1,-1,0,-1,0,0,1,2,5},
    {0,0,-1,1,1,1,0,0,2,3},
    {0,1,-1,-1,0,0,0,-1,2,2},
    {0,-1,1,0,1,0,0,-1,2,5},
    {1,0,0,0,-1,0,-1,-1,2,6},
    {1,0,0,1,0,-1,-1,0,2,1},
    {0,1,-1,0,0,-1,-1,0,2,2},
    {0,-1,-1,1,0,0,0,1,2,0},
    {1,0,1,-1,-1,0,0,0,2,3},
    {0,0,1,-1,-1,-1,0,0,2,4},
    {0,1,-1,0,0,0,-1,-1,2,4},
    {0,1,-1,-1,0,1,0,0,2,2},
    {1,0,1,1,-1,0,0,0,2,2},
    {0,1,-1,0,-1,0,0,1,2,6},
    {0,0,-1,1,1,1,0,0,2,4},
    {1,-1,0,0,1,0,0,-1,2,5},
    {0,-1,1,0,1,0,0,-1,2,6},
    {0,1,-1,-1,0,0,0,1,2,2},
    {0,0,-1,0,0,0,0,-1,1,7},
    {1,0,0,1,0,-1,-1,0,2,2},
    {0,0,0,-1,0,1,-1,-1,2,1},
    {0,0,0,1,1,0,0,0,1,0},
    {-1,0,0,1,1,-1,0,0,2,1},
    {0,0,1,-1,-1,-1,0,0,2,5},
    {-1,0,-1,-1,0,1,0,0,2,2},
    {0,0,-1,1,1,1,0,0,2,5},
    {0,-1,1,0,1,0,0,-1,2,7},
    {0,-1,0,0,0,-1,0,0,1,4},
    {-1,1,0,0,0,0,-1,-1,2,1},
    {1,0,0,1,0,-1,-1,0,2,3},
    {0,0,-1,1,-1,-1,0,0,2,1},
    {0,1,0,0,0,1,-1,-1,2,4},
    {-1,0,-1,1,0,1,0,0,2,5},
    {1,0,0,0,0,-1,0,0,1,5},
    {0,-1,-1,1,0,0,0,1,2,6},
    {1,-1,0,0,0,0,1,1,2,2},
    {-1,-1,0,0,1,-1,0,0,2,1},
    {0,0,-1,-1,0,-1,1,0,2,7},
    {0,1,1,0,1,0,0,1,2,5},
    {-1,0,0,-1,0,0,-1,-1,2,4},
    {-1,0,0,-1,0,1,1,0,2,4},
    {1,1,-1,0,-1,0,0,0,2,2},
    {1,0,0,1,1,1,0,0,2,7},
    {1,0,0,1,0,-1,-1,0,2,4},
    {0,0,1,-1,-1,-1,0,0,2,7},
    {-1,0,0,-1,0,1,1,0,2,0},
    {0,1,-1,0,-1,0,0,1,2,1},
    {1,-1,0,0,0,0,1,1,2,3},
    {0,-1,1,1,0,0,1,0,2,4},
    {0,0,-1,0,1,0,0,0,1,4},
    {0,-1,1,0,1,0,0,-1,2,1},
    {-1,1,0,0,0,0,-1,-1,2,3},
    {-1,0,0,-1,-1,0,0,1,2,5},
    {0,1,0,0,1,0,0,0,1,7},
    {0,0,1,1,0,-1,1,0,2,1},
    {-1,0,0,0,-1,0,0,0,1,0},
    {1,0,0,1,0,-1,-1,0,2,5},
    {0,0,1,0,-1,0,-1,1,2,6},
    {1,-1,0,0,0,0,1,1,2,4},
    {-1,-1,1,0,0,0,-1,0,2,4},
    {0,1,-1,0,0,0,1,1,2,1},
    {0,0,-1,0,0,-1,-1,1,2,1},
    {-1,0,0,-1,0,1,1,0,2,6},
    {-1,1,0,0,0,0,-1,-1,2,4},
    {-1,0,0,0,1,0,1,1,2,1},
    {-1,0,0,0,0,0,1,0,1,7},
    {1,0,0,1,0,-1,-1,0,2,6},
    {-1,0,-1,-1,0,0,1,0,2,2},
    {0,1,0,0,-1,0,-1,-1,2,0},
    {0,-1,-1,0,1,-1,0,0,2,7},
    {0,0,-1,0,1,0,1,-1,2,4},
    {0,0,1,-1,-1,-1,0,0,2,1},
    {0,1,-1,0,-1,0,0,1,2,3},
    {0,0,0,1,0,-1,-1,1,2,2},
    {0,0,0,0,0,1,0,0,0,5},
    {1,0,1,1,0,-1,0,0,2,0},
    {0,-1,0,0,0,-1,1,1,2,4},
    {1,-1,0,0,-1,-1,0,0,2,7},
    {0,0,-1,-1,1,0,0,1,2,7},
    {0,0,0,-1,0,1,1,-1,2,1},
    {1,-1,0,0,0,0,1,1,2,5},
    {0,0,-1,1,1,1,0,0,2,1},
    {1,-1,0,0,0,-1,-1,0,2,1},
    {0,0,1,-1,-1,-1,0,0,2,2},
    {0,0,1,0,1,1,-1,0,2,7},
    {1,-1,0,0,1,1,0,0,2,5},
    {0,0,1,-1,1,1,0,0,2,5},
    {1,-1,0,0,0,0,1,1,2,6},
    {0,1,1,-1,-1,0,0,0,2,7},
    {-1,1,0,0,0,0,-1,-1,2,6},
    {0,0,0,-1,0,1,0,0,1,2},
    {0,0,1,-1,-1,0,0,1,2,1},
    {1,0,0,1,0,-1,-1,0,2,0},
    {0,0,0,0,-1,0,0,0,0,4},
    {0,0,0,0,0,0,1,0,0,0},
    {0,-1,0,0,1,1,-1,0,2,7},
    {-1,1,0,0,1,1,0,0,2,6},
    {0,0,0,0,0,1,0,0,0,2},
    {0,0,0,0,1,0,0,0,0,4},
    {0,0,0,0,0,0,0,-1,0,6},
    {-1,-1,1,0,1,0,0,0,2,5},
    {0,-1,1,0,0,0,1,1,2,6},
    {0,0,-1,0,-1,0,-1,1,2,5},
    {0,0,0,0,-1,0,0,0,0,5},
    {0,0,0,0,0,1,0,0,0,3},
    {0,0,1,-1,0,1,1,0,2,2},
    {0,0,0,0,1,0,0,0,0,5},
    {0,0,0,0,0,0,0,-1,0,7},
    {-1,0,-1,1,0,0,0,1,2,5},
    {0,0,0,0,0,-1,0,0,0,3},
    {0,0,0,0,0,1,0,0,0,4},
    {0,0,0,0,1,0,0,0,0,6},
    {0,0,0,0,0,0,0,-1,0,0},
    {-1,0,-1,-1,1,0,0,0,2,5},
    {-1,-1,0,-1,0,0,-1,0,2,4},
    {1,-1,-1,0,0,1,0,0,2,0},
    {0,-1,0,0,-1,0,0,0,1,6},
    {0,0,0,0,0,0,-1,0,0,2},
    {0,0,0,0,0,-1,0,0,0,4},
    {1,-1,-1,0,0,-1,0,0,2,0},
    {0,0,1,-1,0,1,1,0,2,6},
    {0,0,0,0,0,0,1,0,0,3},
    {-1,0,0,0,1,1,-1,0,2,0},
    {0,1,0,0,0,-1,1,1,2,5},
    {-1,0,0,-1,1,1,0,0,2,0},
    {0,-1,-1,1,-1,0,0,0,2,2},
    {0,1,0,0,1,0,1,-1,2,5},
    {0,0,0,0,0,-1,0,0,0,5},
    {-1,0,0,0,1,-1,-1,0,2,4},
    {1,-1,-1,0,0,0,1,0,2,0},
    {0,0,0,0,-1,0,0,0,0,0},
    {0,0,0,0,0,0,0,1,0,2},
    {0,0,0,0,1,0,0,0,0,0},
    {1,0,0,1,-1,-1,0,0,2,0},
    {-1,-1,0,-1,0,0,0,-1,2,1},
    {0,0,0,0,0,0,0,-1,0,2},
    {0,0,-1,0,-1,0,-1,-1,2,7},
    {0,0,0,0,0,-1,0,0,0,6},
    {0,-1,0,0,-1,0,-1,-1,2,5},
    {0,0,0,0,0,0,1,0,0,5},
    {0,1,1,0,1,-1,0,0,2,7},
    {-1,1,0,1,0,0,-1,0,2,7},
    {0,0,0,0,0,-1,0,0,0,7},
    {0,0,0,1,0,1,-1,-1,2,0},
    {1,1,0,1,1,0,0,0,2,0},
    {0,0,0,0,-1,0,0,0,0,2},
    {1,0,1,-1,0,0,0,1,2,1},
    {-1,-1,0,0,0,0,1,-1,2,5},
    {0,0,1,1,1,0,0,1,2,4},
    {-1,0,0,-1,1,1,0,0,2,3},
    {0,0,-1,0,0,0,0,0,0,4},
    {0,0,-1,1,1,1,0,0,2,6},
    {0,0,0,0,0,0,0,1,0,4},
    {0,0,0,0,0,1,0,0,0,0},
    {0,0,0,0,0,0,0,-1,0,4},
    {0,0,-1,-1,0,-1,1,0,2,1},
    {0,0,0,0,0,0,-1,0,0,6},
    {0,1,0,0,0,0,0,0,0,4},
    {0,0,0,0,0,-1,0,0,0,0},
    {-1,0,0,0,-1,1,0,1,2,3},
    {0,0,0,0,0,0,0,1,0,5},
    {1,0,0,-1,1,-1,0,0,2,4},
    {0,0,0,0,0,0,1,0,0,7},
    {0,0,0,0,1,0,0,0,0,3},
    {1,0,0,-1,-1,1,0,0,2,2},
    {0,0,0,0,0,0,0,-1,0,5},
    {0,0,0,1,1,1,0,1,2,3},
    {-1,-1,0,-1,0,0,-1,0,2,1},
    {0,0,0,0,0,0,-1,0,0,7},
    {0,0,0,0,0,-1,0,0,0,1},
    {-1,1,0,0,0,0,1,1,2,7},
    {1,1,0,1,0,1,0,0,2,1},
    {1,0,1,1,0,-1,0,0,2,3},
    {-1,0,0,-1,0,-1,-1,0,2,1},
    {-1,0,0,0,0,0,0,-1,1,4},
    {-1,1,0,0,0,0,1,1,2,0},
    {1,-1,0,0,0,0,-1,-1,2,0},
    {0,-1,-1,0,-1,1,0,0,2,6},
    {-1,0,0,-1,0,-1,-1,0,2,2},
    {1,0,1,-1,0,0,1,0,2,5},
    {1,0,0,1,0,1,1,0,2,5},
    {-1,-1,0,0,-1,0,0,-1,2,5},
    {1,0,0,1,0,1,1,0,2,3},
    {0,1,-1,0,1,0,0,-1,2,7},
    {1,-1,0,0,0,0,-1,-1,2,1},
    {-1,0,0,0,-1,-1,1,0,2,3},
    {0,1,0,0,1,0,1,1,2,1},
    {-1,0,0,-1,0,-1,-1,0,2,3},
    {0,0,0,1,-1,1,1,0,2,6},
    {1,0,0,0,0,0,0,-1,1,5},
    {-1,1,0,0,0,0,1,1,2,2},
    {-1,-1,1,0,0,0,-1,0,2,6},
    {0,0,0,1,-1,-1,0,-1,2,1},
    {0,0,0,-1,1,1,0,1,2,6},
    {0,0,0,0,0,0,0,1,0,7},
    {0,1,-1,0,1,0,0,-1,2,0},
    {0,0,0,0,0,1,0,0,0,7},
    {-1,0,0,0,0,0,0,0,0,4},
    {-1,-1,0,0,1,0,0,1,2,0},
    {1,-1,0,0,0,0,-1,-1,2,2},
    {0,0,-1,1,-1,-1,0,0,2,6},
    {0,1,0,0,0,-1,-1,1,2,6},
    {0,0,1,0,1,0,0,0,1,0},
    {0,-1,1,0,-1,0,0,1,2,1},
    {1,1,0,0,1,-1,0,0,2,0},
    {-1,1,0,0,0,0,1,1,2,3},
    {0,0,1,-1,1,1,0,0,2,7},
    {0,1,-1,0,1,0,0,-1,2,1},
    {0,0,1,1,0,-1,1,0,2,4},
    {0,0,1,1,0,1,-1,0,2,0},
    {-1,1,0,0,1,0,0,-1,2,0},
    {0,0,0,0,0,0,0,-1,0,3},
    {1,-1,0,0,0,0,-1,-1,2,3},
    {0,0,1,-1,0,0,1,1,2,5},
    {0,0,0,-1,1,-1,-1,0,2,2},
    {-1,0,0,-1,0,-1,-1,0,2,5},
    {-1,0,0,-1,0,0,-1,-1,2,1},
    {-1,1,0,0,0,0,1,1,2,4},
    {1,0,0,1,0,1,1,0,2,6},
    {0,1,-1,0,1,0,0,-1,2,2},
    {1,0,0,1,0,-1,-1,0,2,7},
    {1,-1,0,0,0,0,-1,-1,2,4},
    {0,0,-1,1,0,1,1,0,2,4},
    {0,0,1,-1,-1,0,0,1,2,5},
    {0,1,1,-1,0,0,0,-1,2,7},
    {-1,0,0,-1,0,-1,-1,0,2,6},
    {0,-1,1,1,0,0,0,1,2,5},
    {0,1,0,0,-1,0,-1,-1,2,6},
    {0,0,-1,1,-1,-1,0,0,2,0},
    {0,-1,1,0,-1,0,0,1,2,3},
    {0,1,0,0,1,1,-1,0,2,1},
    {-1,1,0,0,0,0,1,1,2,5},
    {0,0,1,0,1,-1,-1,0,2,0},
    {1,-1,0,0,0,-1,-1,0,2,3},
    {0,1,-1,0,1,0,0,-1,2,3},
    {1,0,0,-1,1,-1,0,0,2,6},
    {1,-1,0,0,0,0,-1,-1,2,5},
    {0,1,1,-1,0,0,0,-1,2,2},
    {0,0,-1,0,0,-1,-1,1,2,4},
    {0,-1,1,0,-1,0,0,1,2,4},
    {1,0,1,1,1,0,0,0,2,1},
    {-1,1,0,0,0,0,1,1,2,6},
    {0,1,0,0,0,0,1,0,1,2},
    {0,-1,1,0,0,0,1,1,2,1},
    {1,0,0,1,0,1,1,0,2,0},
    {1,1,0,0,-1,1,0,0,2,7},
    {1,-1,0,0,0,0,-1,-1,2,6},
    {-1,-1,1,0,0,-1,0,0,2,7},
    {-1,0,0,-1,0,-1,-1,0,2,0},
    {0,0,-1,1,-1,-1,0,0,2,2},
    {0,0,1,0,0,-1,1,1,2,0},
    {0,-1,1,0,0,0,-1,-1,2,6},
    {-1,1,0,0,0,1,1,0,2,1},
    {1,0,1,-1,0,1,0,0,2,0},
    {1,0,1,-1,0,1,0,0,2,4},
    {0,1,-1,-1,1,0,0,0,2,6},
    {0,0,-1,1,0,-1,-1,0,2,6},
    {1,-1,0,-1,0,0,0,-1,2,5},
    {-1,1,1,0,0,0,0,1,2,1},
    {0,0,0,-1,-1,-1,1,0,2,1},
    {1,0,1,-1,0,1,0,0,2,5},
    {0,1,0,0,1,0,1,-1,2,4},
    {-1,1,1,0,0,0,0,1,2,2},
    {1,1,0,1,0,0,1,0,2,4},
    {1,0,1,-1,0,1,0,0,2,6},
    {-1,0,0,0,1,-1,-1,0,2,6},
    {0,0,0,-1,1,0,0,0,1,6},
    {0,-1,1,1,-1,0,0,0,2,0},
    {-1,1,1,0,0,0,0,1,2,3},
    {1,0,1,-1,0,1,0,0,2,7},
    {0,1,-1,-1,1,0,0,0,2,1},
    {-1,-1,0,-1,0,0,-1,0,2,5},
    {-1,0,-1,1,0,-1,0,0,2,7},
    {1,0,0,0,1,1,-1,0,2,2},
    {1,0,1,1,-1,0,0,0,2,7},
    {-1,1,1,0,0,0,1,0,2,2},
    {0,-1,0,0,0,0,0,-1,1,3},
    {0,-1,1,1,-1,0,0,0,2,1},
    {1,1,0,1,0,0,1,0,2,6},
    {0,0,1,0,0,1,-1,-1,2,2},
    {0,1,1,0,1,0,0,1,2,3},
    {1,-1,-1,0,0,0,0,-1,2,4},
    {0,-1,1,1,-1,0,0,0,2,2},
    {0,1,1,0,0,0,1,-1,2,3},
    {0,1,-1,-1,1,0,0,0,2,3},
    {1,-1,-1,0,0,0,0,-1,2,5},
    {-1,0,0,0,1,0,1,1,2,2},
    {0,-1,1,1,-1,0,0,0,2,3},
    {0,0,-1,1,0,1,1,0,2,1},
    {-1,1,1,0,0,0,0,1,2,6},
    {1,1,0,1,0,0,1,0,2,0},
    {1,0,1,-1,0,1,0,0,2,2},
    {0,0,0,1,0,0,-1,0,1,5},
    {0,0,0,-1,0,0,0,0,0,5},
    {0,-1,1,1,-1,0,0,0,2,4},
    {1,1,0,1,0,0,1,0,2,1},
    {0,1,-1,-1,1,0,0,0,2,5},
    {0,1,1,-1,-1,0,0,0,2,2},
    {-1,0,-1,-1,0,1,0,0,2,0},
    {0,-1,-1,1,1,0,0,0,2,2},
    {-1,1,0,0,1,0,0,-1,2,4},
    {0,1,0,0,1,0,1,1,2,7},
    {0,1,1,0,0,1,-1,0,2,4},
    {0,1,1,-1,-1,0,0,0,2,3},
    {1,-1,0,-1,1,0,0,0,2,0},
    {1,1,-1,0,0,0,0,1,2,5},
    {1,-1,0,-1,0,0,1,0,2,7},
    {-1,0,-1,-1,0,1,0,0,2,1},
    {0,1,0,0,1,0,1,-1,2,3},
    {0,-1,1,0,1,1,0,0,2,7},
    {0,-1,-1,1,1,0,0,0,2,3},
    {1,-1,0,0,0,-1,-1,0,2,7},
    {-1,-1,1,0,0,0,0,-1,2,5},
    {1,-1,0,0,1,0,0,-1,2,1},
    {0,0,0,-1,1,1,0,1,2,5},
    {1,1,-1,0,0,0,0,1,2,6},
    {1,-1,0,-1,0,0,1,0,2,0},
    {-1,1,0,1,0,0,-1,0,2,0},
    {0,0,-1,-1,0,-1,1,0,2,4},
    {1,0,1,1,0,-1,0,0,2,2},
    {0,1,1,-1,-1,0,0,0,2,5},
    {0,0,1,1,1,-1,0,0,2,7},
    {1,1,-1,0,0,0,0,1,2,7},
    {0,0,1,-1,0,-1,-1,0,2,2},
    {1,-1,0,-1,0,0,1,0,2,1},
    {-1,0,-1,-1,0,1,0,0,2,3},
    {0,0,0,-1,1,0,0,0,1,4},
    {-1,-1,1,0,0,0,0,-1,2,7},
    {0,-1,-1,0,-1,0,0,-1,2,0},
    {0,1,-1,0,0,0,-1,-1,2,0},
    {-1,1,0,1,0,0,-1,0,2,1},
    {1,1,-1,0,0,0,0,1,2,0},
    {1,-1,0,-1,0,0,1,0,2,2},
    {0,-1,-1,1,1,0,0,0,2,6},
    {0,1,-1,0,0,0,-1,-1,2,7},
    {-1,0,0,0,1,-1,0,-1,2,1},
    {0,0,0,-1,0,0,1,0,1,7},
    {-1,-1,1,0,0,0,0,-1,2,0},
    {1,0,0,0,1,-1,-1,0,2,1},
    {-1,1,0,1,0,0,-1,0,2,2},
    {0,0,0,-1,-1,-1,1,0,2,5},
    {0,0,0,1,-1,1,0,1,2,7},
    {0,-1,1,0,0,-1,-1,0,2,5},
    {0,-1,-1,0,0,0,1,-1,2,1},
    {1,1,-1,0,0,0,0,1,2,1},
    {0,-1,-1,0,0,0,1,-1,2,5},
    {0,0,0,1,-1,1,0,1,2,5},
    {-1,0,-1,-1,0,1,0,0,2,5},
    {0,0,-1,-1,1,0,0,1,2,0},
    {1,1,-1,0,0,1,0,0,2,4},
    {-1,0,-1,-1,0,-1,0,0,2,3},
    {0,-1,-1,1,1,0,0,0,2,7},
    {0,0,0,-1,1,0,1,1,2,1},
    {1,1,-1,0,0,0,1,0,2,0},
    {1,0,0,0,1,1,-1,0,2,1},
    {1,-1,0,-1,0,0,1,0,2,4},
    {-1,0,-1,-1,0,1,0,0,2,6},
    {1,0,0,-1,0,1,-1,0,2,7},
    {0,0,0,1,1,1,-1,0,2,3},
    {0,-1,1,1,0,0,0,-1,2,4},
    {0,0,1,0,-1,0,-1,1,2,0},
    {0,0,1,-1,1,0,0,-1,2,4},
    {-1,-1,1,0,0,0,0,-1,2,2},
    {0,0,0,-1,0,1,1,-1,2,5},
    {1,1,-1,0,0,0,0,1,2,3},
    {0,0,-1,0,1,0,1,1,2,7},
    {1,-1,0,-1,0,0,1,0,2,5},
    {-1,0,-1,-1,0,1,0,0,2,7},
    {1,0,1,1,0,-1,0,0,2,7},
    {0,0,0,1,0,1,1,-1,2,3},
    {-1,-1,0,0,0,0,1,-1,2,0},
    {0,0,0,-1,-1,-1,1,0,2,4},
    {0,-1,0,0,0,-1,1,1,2,0},
    {0,0,-1,-1,-1,1,0,0,2,3},
    {0,0,-1,0,-1,1,0,1,2,6},
    {0,0,0,-1,0,-1,0,0,1,4},
    {0,-1,-1,0,1,-1,0,0,2,1},
    {0,-1,0,0,1,0,1,1,2,0},
    {0,0,-1,0,-1,0,-1,1,2,4},
    {0,1,-1,0,1,1,0,0,2,6},
    {0,0,1,0,1,-1,0,-1,2,6},
    {1,0,1,-1,0,0,-1,0,2,1},
    {0,-1,-1,1,0,0,0,1,2,7},
    {1,0,0,0,0,-1,-1,1,2,2},
    {-1,1,0,0,0,1,1,0,2,0},
    {-1,1,0,0,1,1,0,0,2,0},
    {0,0,0,1,1,1,-1,0,2,5},
    {1,0,0,0,-1,1,1,0,2,4},
    {1,0,1,-1,-1,0,0,0,2,0},
    {1,0,0,-1,0,0,1,-1,2,5},
    {0,0,1,0,1,-1,0,-1,2,7},
    {0,0,1,0,1,-1,0,-1,2,1},
    {0,1,0,0,-1,0,-1,-1,2,1},
    {0,0,-1,-1,0,1,-1,0,2,0},
    {0,0,0,1,1,1,-1,0,2,6},
    {0,1,0,0,-1,0,-1,-1,2,2},
    {0,0,-1,0,0,-1,-1,1,2,6},
    {0,-1,0,0,0,0,0,-1,1,7},
    {1,0,0,0,0,-1,-1,1,2,4},
    {-1,0,0,-1,-1,0,0,1,2,0},
    {0,0,0,1,0,-1,1,1,2,2},
    {-1,1,1,0,0,0,1,0,2,4},
    {0,0,-1,0,1,-1,0,-1,2,3},
    {0,0,-1,0,-1,1,0,1,2,1},
    {0,0,0,1,1,-1,-1,0,2,7},
    {-1,-1,0,0,0,-1,1,0,2,1},
    {1,1,0,1,0,0,0,1,2,7},
    {-1,0,0,0,0,1,1,-1,2,5},
    {-1,0,0,0,0,1,1,-1,2,7},
    {0,1,-1,-1,0,0,-1,0,2,7},
    {0,0,1,-1,0,0,-1,-1,2,4},
    {0,0,0,1,1,0,0,0,1,5},
    {0,1,0,0,-1,0,-1,-1,2,3},
    {1,0,0,0,0,-1,-1,1,2,5},
    {0,0,0,-1,-1,-1,1,0,2,0},
    {0,0,1,0,-1,-1,1,0,2,1},
    {-1,1,0,1,-1,0,0,0,2,0},
    {0,-1,0,0,1,0,1,1,2,4},
    {0,0,1,0,1,-1,0,-1,2,2},
    {0,1,0,0,-1,0,-1,-1,2,4},
    {1,1,0,0,-1,0,0,-1,2,7},
    {1,0,0,0,0,-1,-1,1,2,6},
    {0,0,-1,0,-1,1,0,1,2,3},
    {0,-1,0,0,1,0,1,1,2,5},
    {0,0,0,-1,-1,-1,0,-1,2,3},
    {0,0,-1,0,0,0,0,0,0,6},
    {0,0,0,1,1,1,-1,0,2,1},
    {1,1,-1,0,1,0,0,0,2,5},
    {0,0,1,0,1,-1,0,-1,2,3},
    {0,1,0,0,-1,0,-1,-1,2,5},
    {-1,0,-1,1,0,0,-1,0,2,1},
    {0,0,0,-1,-1,-1,1,0,2,2},
    {-1,0,0,0,0,1,1,-1,2,0},
    {0,0,0,-1,1,0,0,0,1,7},
    {-1,0,-1,1,1,0,0,0,2,0},
    {0,0,0,1,0,1,-1,-1,2,6},
    {0,1,1,0,1,-1,0,0,2,4},
    {1,0,0,0,0,0,-1,0,1,4},
    {0,0,1,0,1,-1,0,-1,2,4},
    {0,0,0,-1,-1,-1,1,0,2,3},
    {0,0,1,0,0,-1,1,1,2,1},
    {-1,-1,1,0,0,0,1,0,2,5},
    {-1,0,0,0,0,-1,0,0,1,7},
    {-1,0,0,0,0,1,1,-1,2,1},
    {0,0,1,0,1,-1,0,-1,2,5},
    {0,1,0,0,1,0,0,0,1,1},
    {-1,1,1,0,0,0,0,1,2,0},
    {0,-1,1,0,0,1,1,0,2,1},
    {-1,1,0,0,1,1,0,0,2,3},
    {1,0,0,1,1,0,0,-1,2,5},
    {0,0,0,-1,0,1,0,0,1,3},
    {0,0,1,-1,0,0,-1,-1,2,7},
    {0,1,-1,0,0,-1,-1,0,2,1},
    {1,0,0,1,-1,-1,0,0,2,2},
    {1,-1,0,0,-1,-1,0,0,2,3},
    {1,1,-1,0,0,-1,0,0,2,7},
    {1,1,0,0,0,-1,1,0,2,2},
    {-1,0,0,-1,-1,0,0,1,2,6},
    {0,0,-1,1,0,0,1,1,2,0},
    {0,-1,0,0,0,1,0,0,1,6},
    {0,0,-1,0,0,-1,0,0,1,6},
    {0,-1,1,0,0,1,1,0,2,2},
    {-1,1,0,0,1,1,0,0,2,4},
    {0,-1,-1,1,0,0,1,0,2,6},
    {-1,0,0,-1,1,0,0,-1,2,2},
    {-1,0,0,-1,-1,0,0,1,2,7},
    {0,1,1,-1,0,1,0,0,2,5},
    {0,0,0,-1,0,0,0,-1,1,6},
    {0,0,0,-1,1,-1,-1,0,2,0},
    {0,-1,1,0,0,1,1,0,2,3},
    {0,0,-1,-1,0,0,1,-1,2,7},
    {0,1,0,0,0,-1,1,1,2,1},
    {-1,1,0,0,1,1,0,0,2,5},
    {0,0,0,0,-1,0,0,0,0,6},
    {0,0,1,0,-1,0,0,0,1,6},
    {1,0,0,1,1,0,0,-1,2,7},
    {0,1,-1,0,0,-1,-1,0,2,3},
    {0,0,1,1,0,0,1,-1,2,1},
    {0,0,1,-1,0,0,-1,-1,2,2},
    {0,1,-1,0,0,-1,-1,0,2,4},
    {1,-1,0,0,-1,-1,0,0,2,6},
    {0,-1,0,0,1,1,0,1,2,5},
    {1,-1,0,-1,0,0,-1,0,2,0},
    {0,-1,1,0,0,1,1,0,2,5},
    {1,0,0,0,0,1,-1,-1,2,1},
    {0,-1,0,0,0,-1,-1,1,2,4},
    {0,1,0,0,-1,-1,0,-1,2,3},
    {-1,1,0,0,1,1,0,0,2,7},
    {0,0,0,1,0,0,0,0,0,4},
    {1,0,0,1,1,0,0,-1,2,1},
    {1,0,0,0,0,1,1,-1,2,4},
    {-1,1,1,0,0,0,0,-1,2,0},
    {-1,0,0,-1,-1,0,0,1,2,2},
    {0,0,-1,1,0,0,1,1,2,4},
    {-1,-1,0,-1,-1,0,0,0,2,4},
    {0,-1,1,0,1,0,0,-1,2,4},
    {1,0,0,1,1,0,0,-1,2,2},
    {-1,1,1,0,1,0,0,0,2,2},
    {0,0,1,0,0,1,1,-1,2,3},
    {0,1,-1,0,0,-1,-1,0,2,6},
    {0,1,0,0,0,0,0,0,0,7},
    {0,0,0,1,-1,1,0,1,2,4},
    {-1,0,0,-1,-1,0,0,1,2,3},
    {0,0,-1,-1,-1,0,0,-1,2,1},
    {0,1,-1,0,1,1,0,0,2,0},
    {0,0,-1,-1,0,0,-1,1,2,3},
    {0,0,-1,1,0,0,1,1,2,5},
    {0,-1,1,0,0,1,1,0,2,7},
    {0,0,1,0,1,1,0,1,2,2},
    {0,0,1,0,0,0,0,-1,1,5},
    {0,0,1,-1,0,0,-1,-1,2,5},
    {0,1,-1,0,0,-1,-1,0,2,7},
    {0,0,-1,1,0,-1,-1,0,2,4},
    {-1,0,0,-1,-1,0,0,1,2,4},
    {0,-1,0,0,0,0,0,0,0,6},
    {0,0,-1,-1,0,-1,1,0,2,5},
    {1,0,0,0,0,0,1,0,1,6},
    {0,0,-1,1,0,0,1,1,2,6},
    {0,0,-1,0,1,1,0,1,2,7},
    {0,0,0,1,-1,1,0,1,2,6},
    {-1,1,0,0,1,1,0,0,2,2},
    {1,0,0,1,1,0,0,-1,2,4},
    {0,0,1,-1,0,0,-1,-1,2,6},
    {0,1,-1,0,0,-1,-1,0,2,0},
    {-1,-1,0,-1,0,-1,0,0,2,6},
    {-1,0,0,0,0,-1,1,1,2,6},
    {1,0,0,0,0,1,-1,-1,2,6},
    {0,0,1,0,0,0,1,0,1,0},
    {1,0,0,0,-1,1,1,0,2,0},
    {0,0,0,1,1,0,1,-1,2,2},
    {0,-1,1,0,-1,-1,0,0,2,3},
    {-1,0,0,0,-1,-1,1,0,2,4},
    {0,0,-1,0,-1,-1,0,-1,2,2},
    {-1,0,0,0,0,1,0,0,1,7},
    {1,0,0,1,-1,-1,0,0,2,4},
    {-1,0,0,0,0,-1,1,1,2,7},
    {1,-1,0,0,-1,0,0,1,2,6},
    {-1,0,0,0,1,-1,-1,0,2,3},
    {0,0,1,0,1,1,0,1,2,3},
    {0,0,0,1,1,1,-1,0,2,7},
    {1,0,0,0,0,1,-1,-1,2,7},
    {0,0,0,-1,1,1,0,1,2,2},
    {0,0,-1,0,0,1,0,0,1,7},
    {0,0,0,-1,1,-1,-1,0,2,1},
    {-1,-1,0,0,0,1,-1,0,2,2},
    {0,0,-1,1,1,0,0,-1,2,1},
    {-1,-1,0,-1,0,0,0,-1,2,5},
    {1,0,0,-1,-1,1,0,0,2,3},
    {0,0,-1,0,-1,-1,0,-1,2,3},
    {0,1,0,0,1,0,1,-1,2,6},
    {1,0,0,0,0,1,-1,-1,2,0},
    {0,1,0,0,-1,0,-1,1,2,6},
    {-1,-1,0,0,1,0,0,1,2,5},
    {0,0,-1,0,1,-1,0,-1,2,4},
    {0,0,-1,1,1,0,0,-1,2,2},
    {1,0,0,0,0,0,0,0,0,3},
    {0,0,-1,0,-1,-1,0,-1,2,4},
    {-1,1,1,0,0,1,0,0,2,0},
    {-1,0,0,0,0,-1,1,1,2,1},
    {0,0,0,1,-1,0,-1,-1,2,2},
    {0,0,0,1,-1,1,1,0,2,3},
    {-1,0,0,0,1,1,-1,0,2,3},
    {0,0,0,1,-1,0,0,0,1,5},
    {0,0,1,0,1,1,0,1,2,5},
    {1,1,0,1,0,-1,0,0,2,6},
    {-1,0,0,0,-1,0,-1,-1,2,7},
    {0,1,0,0,1,0,1,-1,2,7},
    {0,0,0,1,0,0,1,0,1,4},
    {0,0,0,-1,1,-1,-1,0,2,3},
    {0,0,-1,0,-1,-1,0,-1,2,5},
    {0,-1,0,0,-1,0,-1,1,2,7},
    {0,0,-1,-1,0,1,-1,0,2,5},
    {0,-1,0,0,0,-1,-1,1,2,6},
    {0,0,0,1,-1,1,1,0,2,4},
    {1,0,0,-1,1,0,0,1,2,0},
    {0,-1,0,0,-1,0,-1,1,2,0},
    {-1,0,0,0,0,-1,1,1,2,3},
    {1,0,0,1,-1,-1,0,0,2,3},
    {0,0,0,1,-1,1,1,0,2,5},
    {1,0,0,-1,-1,0,0,-1,2,0},
    {0,0,1,0,1,1,0,1,2,7},
    {-1,0,0,-1,0,0,-1,-1,2,5},
    {-1,1,0,1,0,1,0,0,2,0},
    {0,0,-1,0,0,-1,-1,1,2,0},
    {0,0,-1,0,-1,-1,0,-1,2,7},
    {0,0,1,0,1,1,0,1,2,0},
    {1,0,0,0,0,1,-1,-1,2,4},
    {0,0,0,1,1,1,-1,0,2,0},
    {0,0,-1,0,-1,-1,0,-1,2,0},
    {0,-1,0,0,-1,0,-1,1,2,2},
    {0,0,0,-1,1,-1,-1,0,2,7},
    {-1,0,-1,1,1,0,0,0,2,5},
    {0,0,-1,1,-1,-1,0,0,2,3},
    {-1,0,0,-1,1,1,0,0,2,4},
    {0,0,-1,0,-1,-1,0,-1,2,1},
    {-1,-1,1,0,0,-1,0,0,2,2},
    {0,1,1,0,-1,1,0,0,2,4},
    {0,0,-1,0,0,0,1,0,1,3},
    {0,0,1,-1,-1,-1,0,0,2,6},
    {-1,0,0,-1,-1,-1,0,0,2,2},
    {1,1,0,0,1,0,0,1,2,6},
    {0,-1,0,0,0,0,0,1,1,5},
    {0,0,1,0,1,0,1,1,2,6},
    {1,0,0,-1,0,0,1,-1,2,0},
    {0,0,-1,-1,0,1,-1,0,2,2},
    {0,1,1,0,-1,1,0,0,2,5},
    {-1,-1,0,-1,0,0,0,1,2,0},
    {0,0,-1,0,0,1,1,-1,2,6},
    {0,1,-1,-1,0,0,0,-1,2,4},
    {1,1,0,0,1,0,0,1,2,7},
    {0,0,-1,-1,0,1,-1,0,2,3},
    {0,0,1,1,1,-1,0,0,2,2},
    {1,0,0,-1,-1,1,0,0,2,0},
    {-1,-1,0,0,0,1,-1,0,2,1},
    {-1,0,0,1,0,0,-1,1,2,1},
    {0,-1,1,1,0,0,0,-1,2,6},
    {0,0,-1,-1,0,0,1,-1,2,3},
    {-1,1,0,1,0,1,0,0,2,7},
    {1,1,-1,0,0,0,-1,0,2,5},
    {-1,0,-1,-1,-1,0,0,0,2,1},
    {1,0,1,1,1,0,0,0,2,2},
    {0,1,1,-1,0,0,0,-1,2,4},
    {1,1,-1,0,0,0,-1,0,2,6},
    {0,-1,-1,1,0,0,0,1,2,4},
    {1,0,0,-1,-1,0,0,-1,2,4},
    {0,-1,-1,0,-1,1,0,0,2,4},
    {1,0,1,1,1,0,0,0,2,3},
    {0,1,1,-1,0,-1,0,0,2,0},
    {-1,0,0,-1,0,0,-1,-1,2,7},
    {0,1,1,-1,0,0,0,-1,2,5},
    {1,-1,0,-1,0,-1,0,0,2,1},
    {-1,0,-1,-1,-1,0,0,0,2,3},
    {0,1,0,0,0,0,-1,0,1,7},
    {-1,1,0,1,0,1,0,0,2,2},
    {0,0,-1,0,0,1,1,-1,2,0},
    {1,0,1,1,1,0,0,0,2,4},
    {0,0,0,0,1,0,0,0,0,7},
    {0,1,1,-1,0,0,0,-1,2,6},
    {1,1,-1,0,0,0,-1,0,2,0},
    {0,1,-1,-1,0,0,0,1,2,7},
    {0,-1,-1,1,0,1,0,0,2,2},
    {1,-1,0,-1,0,-1,0,0,2,2},
    {-1,0,-1,-1,-1,0,0,0,2,4},
    {-1,1,0,1,0,1,0,0,2,3},
    {1,1,0,0,-1,0,0,-1,2,3},
    {1,0,1,1,0,0,-1,0,2,0},
    {1,0,1,1,1,0,0,0,2,5},
    {-1,1,1,0,0,0,1,0,2,0},
    {-1,-1,1,0,0,0,1,0,2,4},
    {1,0,1,-1,0,0,0,-1,2,5},
    {-1,0,-1,-1,-1,0,0,0,2,5},
    {-1,1,0,1,0,1,0,0,2,4},
    {1,0,1,-1,-1,0,0,0,2,1},
    {1,0,1,1,1,0,0,0,2,6},
    {0,0,0,-1,0,-1,-1,1,2,6},
    {1,1,0,0,0,1,-1,0,2,0},
    {0,1,-1,0,0,0,1,1,2,5},
    {0,1,1,-1,0,0,0,-1,2,0},
    {1,-1,0,-1,0,-1,0,0,2,4},
    {-1,0,-1,-1,-1,0,0,0,2,6},
    {0,1,1,-1,0,0,0,-1,2,3},
    {1,0,1,1,1,0,0,0,2,7},
    {0,0,0,1,-1,1,1,0,2,2},
    {0,1,-1,-1,0,0,0,1,2,4},
    {1,-1,0,-1,0,-1,0,0,2,5},
    {1,1,0,0,0,1,-1,0,2,7},
    {0,0,1,0,0,0,1,0,1,1},
    {-1,-1,1,0,0,0,1,0,2,3},
    {0,0,-1,1,-1,-1,0,0,2,5},
    {-1,0,-1,-1,-1,0,0,0,2,0},
    {0,0,0,0,1,0,0,0,0,1},
    {0,-1,-1,1,0,0,0,1,2,2},
    {0,1,1,0,-1,1,0,0,2,6},
    {0,-1,0,0,0,-1,0,0,1,0},
    {0,0,1,1,0,-1,1,0,2,5},
    {-1,0,0,0,0,-1,0,0,1,6},
    {1,0,0,-1,0,0,1,-1,2,2},
    {1,0,1,1,-1,0,0,0,2,6},
    {0,0,-1,-1,0,1,-1,0,2,4},
    {0,-1,-1,0,1,-1,0,0,2,6},
    {0,1,0,0,1,-1,-1,0,2,6},
    {-1,-1,0,0,-1,0,0,-1,2,0},
    {0,0,1,0,0,1,-1,-1,2,7},
    {-1,-1,0,-1,0,-1,0,0,2,2},
    {-1,0,0,1,0,0,-1,1,2,2},
    {1,1,0,0,0,1,-1,0,2,3},
    {1,-1,0,-1,0,0,0,-1,2,3},
    {-1,1,0,1,0,0,-1,0,2,6},
    {0,1,1,0,1,-1,0,0,2,2},
    {1,1,0,0,-1,0,0,-1,2,4},
    {0,-1,-1,0,-1,1,0,0,2,2},
    {-1,-1,0,0,1,0,0,1,2,4},
    {0,0,1,1,0,1,-1,0,2,1},
    {0,-1,0,0,1,-1,-1,0,2,3},
    {0,1,1,0,1,-1,0,0,2,3},
    {0,0,0,1,1,0,0,0,1,6},
    {1,1,0,0,-1,0,0,-1,2,5},
    {0,-1,-1,0,0,1,-1,0,2,2},
    {1,0,0,-1,0,0,-1,1,2,7},
    {0,-1,-1,0,0,-1,1,0,2,6},
    {0,-1,-1,0,-1,1,0,0,2,3},
    {-1,0,0,1,0,0,1,-1,2,7},
    {0,0,1,1,0,1,-1,0,2,2},
    {-1,1,0,1,0,0,1,0,2,0},
    {1,-1,0,-1,0,-1,0,0,2,0},
    {1,0,0,-1,0,0,-1,1,2,0},
    {0,0,-1,-1,0,-1,1,0,2,2},
    {0,0,-1,-1,0,0,-1,1,2,5},
    {-1,0,0,1,0,0,1,-1,2,4},
    {-1,-1,0,-1,0,0,1,0,2,7},
    {0,0,1,1,0,1,-1,0,2,3},
    {0,0,0,1,0,-1,-1,1,2,0},
    {0,0,-1,1,0,0,-1,-1,2,3},
    {0,-1,-1,0,-1,1,0,0,2,5},
    {-1,-1,0,0,1,0,0,1,2,7},
    {0,0,0,1,-1,0,-1,1,2,1},
    {-1,0,0,1,0,0,1,-1,2,1},
    {-1,1,0,0,1,0,0,-1,2,6},
    {-1,1,1,0,0,0,-1,0,2,7},
    {1,0,0,0,-1,1,1,0,2,6},
    {0,1,1,0,1,-1,0,0,2,6},
    {0,-1,-1,0,-1,1,0,0,2,1},
    {0,0,0,-1,1,0,0,0,1,3},
    {0,-1,-1,0,-1,1,0,0,2,7},
    {0,1,-1,-1,0,0,1,0,2,7},
    {0,0,1,0,0,0,-1,0,1,4},
    {0,0,-1,0,0,1,0,0,1,5},
    {0,0,1,1,0,1,-1,0,2,6},
    {-1,0,-1,-1,0,0,0,1,2,7},
    {1,1,0,0,-1,0,0,-1,2,2},
    {1,0,0,-1,0,0,-1,1,2,4},
    {0,0,-1,-1,0,-1,1,0,2,6},
    {-1,0,0,-1,0,0,1,1,2,5},
    {0,0,-1,-1,0,0,1,-1,2,5},
    {0,-1,-1,0,-1,1,0,0,2,0},
    {1,0,0,0,-1,-1,1,0,2,0},
    {0,1,1,0,1,-1,0,0,2,1},
    {0,1,-1,0,-1,0,0,1,2,4},
    {0,1,0,0,0,0,0,-1,1,2},
    {1,0,0,-1,0,0,-1,1,2,5},
    {1,0,0,-1,0,1,-1,0,2,3},
    {-1,-1,0,0,1,0,0,1,2,3},
    {0,0,0,-1,0,1,-1,-1,2,3},
    {0,1,-1,0,0,0,-1,-1,2,1},
    {-1,0,0,1,0,0,1,-1,2,5},
    {0,0,0,1,0,0,0,1,1,4},
    {0,1,1,0,-1,1,0,0,2,7},
    {1,0,0,-1,0,0,1,-1,2,3},
    {-1,-1,0,0,-1,0,0,-1,2,1},
    {-1,0,0,1,0,0,-1,1,2,3},
    {1,1,0,1,0,-1,0,0,2,3},
    {-1,1,1,0,0,0,-1,0,2,2},
    {0,1,-1,-1,0,0,0,1,2,0},
    {1,1,-1,0,-1,0,0,0,2,5},
    {0,1,1,-1,0,0,1,0,2,5},
    {1,-1,-1,0,0,0,1,0,2,2},
    {0,-1,1,0,-1,-1,0,0,2,2},
    {-1,-1,0,-1,0,1,0,0,2,4},
    {0,0,0,1,0,1,-1,-1,2,5},
    {-1,0,-1,1,1,0,0,0,2,6},
    {0,0,0,1,1,0,0,0,1,1},
    {0,-1,1,1,0,0,0,-1,2,0},
    {0,0,-1,0,0,-1,0,0,1,7},
    {-1,1,1,0,0,0,-1,0,2,3},
    {1,1,0,1,0,-1,0,0,2,5},
    {1,0,0,-1,1,-1,0,0,2,2},
    {1,0,1,-1,-1,0,0,0,2,7},
    {1,0,0,0,1,-1,-1,0,2,6},
    {0,1,0,0,0,-1,-1,1,2,7},
    {0,1,-1,-1,0,0,0,1,2,1},
    {0,-1,-1,1,0,-1,0,0,2,2},
    {-1,-1,0,-1,0,1,0,0,2,5},
    {0,-1,1,1,0,0,0,-1,2,1},
    {0,1,1,0,0,0,1,-1,2,7},
    {1,-1,0,0,0,1,1,0,2,6},
    {-1,1,1,0,0,0,-1,0,2,4},
    {0,1,0,0,0,0,0,-1,1,7},
    {0,0,0,-1,1,-1,0,-1,2,2},
    {1,0,1,-1,0,0,0,-1,2,2},
    {1,-1,-1,0,0,0,1,0,2,4},
    {0,-1,0,0,0,0,0,0,0,3},
    {-1,1,1,0,0,0,-1,0,2,5},
    {-1,1,0,1,0,-1,0,0,2,4},
    {1,1,0,1,0,-1,0,0,2,7},
    {0,0,-1,-1,0,1,-1,0,2,1},
    {1,-1,-1,0,0,0,1,0,2,5},
    {-1,-1,0,-1,0,1,0,0,2,7},
    {0,-1,0,0,0,0,0,0,0,7},
    {-1,0,-1,1,1,0,0,0,2,1},
    {-1,1,1,0,0,0,-1,0,2,6},
    {-1,1,0,0,0,-1,-1,0,2,0},
    {1,1,0,1,0,-1,0,0,2,0},
    {0,0,1,0,0,1,-1,-1,2,1},
    {0,1,1,-1,1,0,0,0,2,2},
    {-1,0,-1,1,1,0,0,0,2,2},
    {0,-1,-1,0,1,0,0,1,2,6},
    {1,1,0,1,0,-1,0,0,2,1},
    {-1,1,0,1,-1,0,0,0,2,1},
    {-1,-1,0,0,-1,1,0,0,2,0},
    {0,0,0,-1,0,0,-1,0,1,6},
    {0,1,1,-1,-1,0,0,0,2,1},
    {0,0,1,0,1,1,0,1,2,4},
    {-1,-1,0,-1,0,1,0,0,2,1},
    {-1,0,0,0,-1,1,0,1,2,0},
    {-1,0,-1,1,1,0,0,0,2,3},
    {0,-1,1,1,0,0,0,-1,2,5},
    {0,1,1,0,-1,1,0,0,2,1},
    {0,1,0,0,0,-1,1,1,2,0},
    {1,0,1,-1,-1,0,0,0,2,4},
    {-1,0,0,0,0,1,1,-1,2,2},
    {0,0,-1,0,-1,0,-1,-1,2,1},
    {-1,0,-1,-1,0,0,0,-1,2,6},
    {0,1,-1,-1,0,0,0,1,2,6},
    {-1,0,-1,1,1,0,0,0,2,4},
    {0,-1,1,0,1,1,0,0,2,5},
    {1,0,0,0,0,1,0,0,1,6},
    {0,-1,0,0,1,0,1,1,2,2},
    {0,0,0,-1,1,-1,0,-1,2,0},
    {1,1,0,0,1,0,0,1,2,2},
    {1,0,0,-1,0,0,1,-1,2,4},
    {1,1,0,0,1,0,0,1,2,3},
    {-1,1,0,0,0,0,-1,-1,2,5},
    {-1,-1,0,0,-1,0,0,-1,2,3},
    {0,0,-1,-1,0,1,-1,0,2,7},
    {1,0,0,0,-1,1,1,0,2,7},
    {-1,0,0,0,1,0,1,-1,2,0},
    {1,0,0,0,-1,1,0,1,2,1},
    {0,0,1,0,-1,0,-1,1,2,4},
    {1,0,1,-1,0,0,-1,0,2,3},
    {0,0,1,-1,0,1,1,0,2,0},
    {1,1,0,1,0,0,0,1,2,1},
    {0,-1,1,0,0,-1,-1,0,2,3},
    {0,0,-1,0,-1,0,0,0,1,5},
    {0,1,0,0,0,-1,1,1,2,6},
    {0,-1,0,0,0,1,-1,-1,2,6},
    {0,0,1,0,-1,0,-1,1,2,5},
    {0,1,0,0,0,-1,1,1,2,7},
    {1,0,0,0,-1,1,1,0,2,1},
    {0,0,0,-1,1,1,0,1,2,3},
    {0,-1,0,0,0,1,-1,-1,2,7},
    {-1,0,0,0,1,-1,-1,0,2,1},
    {1,-1,-1,0,0,0,0,-1,2,3},
    {1,0,0,0,-1,1,1,0,2,2},
    {1,0,0,-1,1,-1,0,0,2,0},
    {0,0,-1,1,-1,-1,0,0,2,7},
    {0,1,1,0,0,0,-1,1,2,2},
    {0,0,-1,0,1,-1,0,-1,2,0},
    {0,0,-1,0,0,0,-1,0,1,3},
    {0,0,-1,0,1,0,1,-1,2,7},
    {0,-1,0,0,0,0,0,-1,1,2},
    {0,-1,0,0,0,1,-1,-1,2,1},
    {0,1,0,0,0,-1,1,1,2,2},
    {0,0,-1,0,1,0,1,-1,2,0},
    {0,-1,0,0,0,1,-1,-1,2,2},
    {0,0,-1,0,-1,1,1,0,2,1},
    {-1,0,-1,-1,0,0,1,0,2,6},
    {1,1,-1,0,0,1,0,0,2,3},
    {0,0,0,1,-1,-1,0,-1,2,6},
    {0,1,0,0,0,-1,1,1,2,3},
    {1,0,0,0,-1,1,1,0,2,5},
    {1,0,1,1,0,-1,0,0,2,4},
    {-1,0,0,-1,-1,-1,0,0,2,6},
    {0,0,0,1,-1,-1,0,-1,2,4},
    {0,0,1,0,-1,0,-1,1,2,2},
    {0,-1,0,0,0,1,-1,-1,2,4},
    {0,0,0,1,-1,-1,0,-1,2,0},
    {0,-1,1,0,0,-1,-1,0,2,2},
    {0,1,1,0,0,1,-1,0,2,5},
    {0,0,1,1,0,-1,1,0,2,0},
    {1,-1,0,0,1,1,0,0,2,6},
    {0,-1,1,0,0,0,-1,-1,2,4},
    {0,1,1,0,-1,1,0,0,2,2},
    {1,1,0,0,1,0,0,1,2,4},
    {1,0,0,-1,0,0,1,-1,2,6},
    {0,1,1,0,0,0,1,-1,2,0},
    {-1,1,1,0,0,-1,0,0,2,5},
    {0,1,0,0,0,0,0,1,1,2},
    {0,-1,-1,0,1,-1,0,0,2,2},
    {1,1,0,0,0,0,1,-1,2,4},
    {1,0,0,1,-1,0,0,1,2,5},
    {-1,0,0,1,0,0,-1,1,2,6},
    {-1,1,0,0,-1,0,0,1,2,4},
    {0,1,-1,0,1,1,0,0,2,2},
    {1,0,1,1,0,0,-1,0,2,7},
    {1,1,-1,0,0,-1,0,0,2,3},
    {0,0,0,0,0,0,0,1,0,6},
    {0,0,-1,-1,-1,0,0,-1,2,0},
    {1,-1,0,0,1,0,0,-1,2,4},
    {0,-1,0,0,-1,-1,0,-1,2,0},
    {1,-1,-1,0,0,0,0,-1,2,2},
    {-1,1,0,0,-1,0,0,1,2,5},
    {1,-1,0,0,-1,-1,0,0,2,0},
    {1,0,0,1,0,0,1,1,2,7},
    {0,0,-1,0,1,-1,-1,0,2,4},
    {0,-1,-1,0,0,0,-1,1,2,4},
    {0,0,-1,1,0,0,-1,-1,2,4},
    {0,0,-1,1,0,-1,-1,0,2,1},
    {0,0,1,0,0,0,-1,0,1,1},
    {1,0,0,1,0,0,1,1,2,0},
    {0,1,-1,0,1,1,0,0,2,4},
    {1,-1,0,0,1,0,0,-1,2,6},
    {-1,0,0,-1,0,0,-1,-1,2,0},
    {0,0,-1,1,0,-1,-1,0,2,2},
    {0,-1,1,0,-1,-1,0,0,2,4},
    {1,0,0,1,0,0,1,1,2,1},
    {0,0,1,0,0,1,0,0,1,5},
    {0,1,-1,0,1,1,0,0,2,5},
    {-1,1,0,1,0,0,1,0,2,1},
    {1,-1,0,0,1,0,0,-1,2,7},
    {0,0,-1,0,-1,0,-1,-1,2,0},
    {0,0,0,-1,0,1,1,-1,2,7},
    {-1,1,1,0,1,0,0,0,2,5},
    {-1,1,0,0,-1,0,0,1,2,0},
    {1,0,0,1,0,1,1,0,2,4},
    {1,1,0,0,1,-1,0,0,2,2},
    {0,0,1,-1,0,1,1,0,2,4},
    {0,1,-1,-1,0,1,0,0,2,4},
    {0,1,1,0,0,0,-1,1,2,0},
    {1,-1,0,0,1,0,0,-1,2,0},
    {0,-1,0,0,0,-1,1,1,2,1},
    {-1,0,0,-1,0,0,-1,-1,2,2},
    {0,-1,1,0,-1,-1,0,0,2,6},
    {0,0,0,-1,-1,0,-1,1,2,4},
    {0,1,-1,-1,1,0,0,0,2,7},
    {0,0,-1,-1,1,0,0,1,2,6},
    {0,0,1,-1,0,1,1,0,2,5},
    {0,0,0,1,0,0,0,1,1,2},
    {-1,0,0,-1,0,0,-1,-1,2,3},
    {0,0,-1,1,0,-1,-1,0,2,5},
    {0,0,-1,0,0,1,0,0,1,0},
    {0,-1,1,0,-1,-1,0,0,2,7},
    {-1,-1,0,0,-1,1,0,0,2,7},
    {1,1,0,0,1,0,0,1,2,1},
    {-1,1,0,0,-1,0,0,1,2,2},
    {1,0,0,1,0,0,1,1,2,4},
    {0,-1,-1,0,-1,0,0,-1,2,5},
    {0,-1,1,0,-1,-1,0,0,2,0},
    {0,1,-1,-1,0,0,-1,0,2,6},
    {-1,1,0,0,-1,0,0,1,2,6},
    {-1,0,0,0,-1,-1,1,0,2,6},
    {0,1,1,0,0,0,-1,1,2,6},
    {1,1,0,1,1,0,0,0,2,3},
    {0,1,1,-1,0,0,0,-1,2,1},
    {-1,0,0,0,1,0,0,0,1,4},
    {0,0,-1,0,1,-1,0,-1,2,7},
    {0,0,0,1,-1,1,1,0,2,1},
    {-1,1,0,0,-1,0,0,1,2,3},
    {1,0,1,1,0,0,0,1,2,4},
    {0,0,1,-1,0,1,1,0,2,7},
    {0,1,-1,0,1,1,0,0,2,1},
    {0,1,1,0,-1,1,0,0,2,3},
    {1,-1,-1,0,0,0,0,1,2,4},
    {0,0,0,-1,1,0,0,0,1,2},
    {-1,0,0,1,0,0,-1,1,2,7},
    {-1,-1,0,-1,0,0,0,-1,2,7},
    {0,0,1,1,-1,1,0,0,2,2},
    {1,-1,0,0,1,0,0,-1,2,3},
    {0,0,1,0,1,0,1,1,2,7},
    {-1,0,0,0,1,0,0,0,1,6},
    {0,1,0,0,0,1,1,-1,2,1},
    {0,0,0,1,0,1,1,-1,2,5},
    {0,0,0,1,1,0,0,0,1,4},
    {1,0,0,0,1,1,-1,0,2,3},
    {1,1,-1,0,0,-1,0,0,2,4},
    {0,0,0,-1,1,-1,0,-1,2,5},
    {0,-1,0,0,0,-1,-1,1,2,1},
    {0,0,0,-1,1,-1,0,-1,2,6},
    {0,1,1,-1,-1,0,0,0,2,4},
    {0,-1,0,0,0,-1,-1,1,2,3},
    {1,-1,0,-1,0,0,0,-1,2,1},
    {-1,0,0,0,-1,-1,1,0,2,5},
    {-1,0,-1,-1,0,0,-1,0,2,0},
    {0,-1,1,1,-1,0,0,0,2,6},
    {0,0,0,1,-1,1,0,1,2,0},
    {1,1,-1,0,0,1,0,0,2,6},
    {0,0,1,0,1,0,1,1,2,2},
    {0,0,-1,0,-1,0,-1,-1,2,2},
    {0,0,0,1,-1,1,0,1,2,1},
    {1,-1,0,-1,1,0,0,0,2,1},
    {-1,0,0,0,0,0,0,1,1,4},
    {0,1,0,0,0,1,1,-1,2,5},
    {1,0,0,0,1,1,-1,0,2,7},
    {0,0,0,-1,1,-1,0,-1,2,1},
    {-1,0,0,0,-1,-1,1,0,2,7},
    {0,0,1,0,1,0,1,1,2,5},
    {0,0,0,1,0,-1,1,1,2,1},
    {0,0,-1,0,-1,0,-1,-1,2,5},
    {0,0,-1,-1,-1,1,0,0,2,0},
    {0,1,0,0,0,1,1,-1,2,0},
    {0,0,0,-1,1,-1,0,-1,2,4},
    {-1,-1,1,0,0,1,0,0,2,1},
    {0,0,-1,1,0,0,-1,-1,2,5},
    {0,-1,0,0,0,-1,-1,1,2,0},
    {-1,0,0,0,-1,-1,1,0,2,2},
    {0,0,1,1,1,0,0,1,2,0},
    {0,1,1,0,0,0,1,-1,2,2},
    {1,1,0,0,0,1,-1,0,2,4},
    {-1,-1,0,0,0,-1,1,0,2,4},
    {0,1,0,0,1,0,0,0,1,0},
    {-1,0,0,1,-1,1,0,0,2,6},
    {1,0,0,1,0,1,1,0,2,7},
    {0,0,1,1,1,0,0,1,2,1},
    {1,0,0,0,-1,0,-1,1,2,0},
    {1,0,0,-1,1,-1,0,0,2,7},
    {0,1,1,-1,0,0,-1,0,2,5},
    {0,-1,-1,1,1,0,0,0,2,1},
    {0,-1,-1,0,0,0,-1,1,2,3},
    {1,1,0,0,0,1,-1,0,2,2},
    {-1,0,0,1,-1,1,0,0,2,7},
    {0,1,0,0,0,0,-1,0,1,1},
    {1,0,0,0,-1,0,0,0,1,0},
    {0,0,1,1,1,0,0,1,2,2},
    {0,0,1,0,1,-1,-1,0,2,4},
    {-1,0,0,0,-1,0,-1,1,2,0},
    {0,0,-1,-1,-1,0,0,-1,2,2},
    {0,1,0,0,0,0,0,1,1,5},
    {-1,-1,0,0,0,-1,1,0,2,6},
    {-1,0,0,1,-1,1,0,0,2,0},
    {-1,0,0,0,-1,0,-1,1,2,6},
    {0,0,1,1,1,0,0,1,2,3},
    {0,0,-1,0,0,0,0,1,1,0},
    {1,0,0,-1,-1,1,0,0,2,1},
    {1,1,0,1,-1,0,0,0,2,0},
    {0,0,1,0,-1,0,0,0,1,7},
    {0,1,1,0,0,0,1,-1,2,5},
    {1,-1,0,-1,0,1,0,0,2,0},
    {0,-1,-1,0,0,0,-1,1,2,5},
    {-1,0,0,1,-1,1,0,0,2,1},
    {-1,-1,0,-1,0,0,0,1,2,7},
    {0,0,-1,-1,-1,0,0,-1,2,4},
    {0,-1,-1,0,0,0,-1,1,2,6},
    {0,0,0,-1,0,0,1,0,1,0},
    {-1,0,0,1,-1,1,0,0,2,2},
    {1,0,0,-1,1,-1,0,0,2,3},
    {-1,0,0,1,-1,1,0,0,2,3},
    {0,0,1,1,1,0,0,1,2,6},
    {0,0,-1,-1,-1,0,0,-1,2,6},
    {1,1,0,0,1,0,0,1,2,5},
    {-1,-1,0,0,0,-1,1,0,2,2},
    {-1,0,0,1,-1,1,0,0,2,4},
    {0,1,1,0,0,0,1,-1,2,1},
    {0,0,1,1,-1,1,0,0,2,6},
    {1,0,0,-1,1,-1,0,0,2,5},
    {0,1,0,0,1,-1,0,-1,2,2},
    {0,-1,-1,0,0,0,-1,1,2,1},
    {0,-1,-1,1,0,1,0,0,2,5},
    {-1,-1,0,0,0,1,-1,0,2,4},
    {-1,1,0,1,0,-1,0,0,2,0},
    {0,0,1,1,-1,0,0,-1,2,0},
    {0,1,1,0,0,0,-1,1,2,3},
    {1,1,0,0,0,-1,1,0,2,5},
    {0,0,-1,-1,1,0,0,1,2,1},
    {0,1,0,0,0,0,-1,0,1,4},
    {-1,0,0,1,1,-1,0,0,2,7},
    {0,1,1,0,0,0,-1,1,2,4},
    {-1,0,-1,1,0,0,-1,0,2,5},
    {1,1,0,0,0,-1,1,0,2,7},
    {0,1,0,0,1,0,1,1,2,0},
    {0,0,-1,-1,1,0,0,1,2,3},
    {0,-1,-1,1,1,0,0,0,2,0},
    {0,1,1,0,0,0,-1,1,2,1},
    {1,-1,0,-1,0,0,1,0,2,3},
    {-1,0,0,-1,-1,0,0,1,2,1},
    {-1,-1,0,0,0,1,-1,0,2,7},
    {0,0,-1,1,0,-1,-1,0,2,3},
    {0,0,-1,-1,1,0,0,1,2,4},
    {0,-1,0,0,1,0,1,-1,2,0},
    {0,-1,-1,0,0,0,1,-1,2,6},
    {-1,-1,0,0,0,1,-1,0,2,0},
    {-1,0,0,1,1,-1,0,0,2,2},
    {0,0,1,1,-1,0,0,-1,2,4},
    {0,1,1,0,0,0,-1,1,2,7},
    {0,1,0,0,1,-1,-1,0,2,7},
    {0,-1,-1,0,0,0,1,-1,2,7},
    {-1,0,0,1,1,-1,0,0,2,3},
    {0,0,1,1,0,0,-1,1,2,5},
    {0,0,1,1,-1,0,0,-1,2,5},
    {1,0,0,-1,-1,1,0,0,2,4},
    {-1,0,0,1,1,-1,0,0,2,4},
    {1,0,1,1,-1,0,0,0,2,3},
    {0,0,-1,1,-1,0,0,1,2,7},
    {1,1,0,0,0,-1,1,0,2,3},
    {-1,-1,0,0,0,1,-1,0,2,3},
    {0,-1,1,1,0,0,0,-1,2,3},
    {0,0,1,-1,-1,0,0,1,2,4},
    {-1,0,-1,-1,1,0,0,0,2,6},
    {0,1,-1,-1,1,0,0,0,2,0},
    {0,1,-1,0,0,0,1,1,2,6},
    {-1,-1,0,-1,0,0,1,0,2,6},
    {1,-1,0,0,0,1,1,0,2,0},
    {0,-1,-1,0,0,1,-1,0,2,1},
    {0,1,1,0,0,-1,1,0,2,7},
    {0,0,0,0,0,1,0,0,0,1},
    {1,0,0,1,0,1,1,0,2,1},
    {0,0,-1,1,1,0,0,-1,2,4},
    {0,0,0,0,0,0,0,-1,0,1},
    {1,-1,0,0,0,1,1,0,2,1},
    {0,1,0,0,0,0,-1,0,1,2},
    {0,0,-1,1,1,0,0,-1,2,5},
    {0,0,0,1,0,0,-1,0,1,2},
    {0,1,-1,0,0,0,1,1,2,0},
    {1,-1,0,0,0,1,1,0,2,2},
    {0,0,-1,1,1,0,0,-1,2,6},
    {0,0,1,0,0,0,1,0,1,5},
    {0,-1,1,0,0,0,-1,-1,2,0},
    {-1,1,0,0,0,-1,-1,0,2,2},
    {-1,0,-1,1,0,0,0,-1,2,3},
    {0,0,0,-1,0,-1,0,0,1,7},
    {0,0,1,-1,-1,0,0,1,2,7},
    {0,1,0,0,-1,0,-1,1,2,5},
    {1,0,0,0,0,1,1,-1,2,3},
    {0,0,-1,1,1,0,0,-1,2,7},
    {-1,1,0,0,0,-1,-1,0,2,3},
    {1,0,0,1,-1,-1,0,0,2,5},
    {-1,1,1,0,0,0,0,1,2,7},
    {0,1,-1,0,0,0,1,1,2,2},
    {0,-1,1,0,0,0,-1,-1,2,2},
    {0,0,1,0,1,0,0,0,1,2},
    {0,1,-1,0,0,0,1,1,2,3},
    {1,0,1,-1,-1,0,0,0,2,5},
    {0,0,1,0,0,0,0,0,0,3},
    {1,-1,0,0,0,1,1,0,2,5},
    {-1,0,0,-1,1,1,0,0,2,7},
    {-1,0,-1,-1,0,0,1,0,2,3},
    {0,-1,1,0,0,0,-1,-1,2,3},
    {-1,1,0,0,0,-1,-1,0,2,5},
    {1,0,0,1,-1,-1,0,0,2,7},
    {0,-1,0,0,1,1,-1,0,2,2},
    {-1,1,0,0,0,-1,-1,0,2,6},
    {0,-1,1,0,0,0,-1,-1,2,5},
    {0,-1,0,0,-1,1,1,0,2,0},
    {1,0,0,0,0,-1,-1,1,2,0},
    {-1,-1,0,0,0,-1,1,0,2,5},
    {1,0,0,1,-1,-1,0,0,2,1},
    {1,1,0,0,0,-1,1,0,2,0},
    {0,1,-1,0,0,0,-1,-1,2,6},
    {1,0,0,1,1,1,0,0,2,3},
    {0,0,0,1,0,-1,1,1,2,7},
    {0,0,-1,0,0,-1,1,1,2,3},
    {0,0,0,-1,1,0,1,1,2,3},
    {0,0,-1,0,0,1,-1,-1,2,2},
    {1,0,0,0,0,-1,-1,1,2,1},
    {0,0,-1,1,-1,0,0,1,2,5},
    {-1,1,0,0,0,1,1,0,2,2},
    {0,0,0,-1,0,-1,0,0,1,6},
    {1,0,0,1,1,1,0,0,2,4},
    {0,-1,0,0,1,0,1,1,2,7},
    {0,0,1,-1,1,0,0,-1,2,6},
    {0,-1,0,0,0,1,0,0,1,2},
    {0,0,1,0,0,-1,0,0,1,6},
    {1,-1,0,0,0,-1,-1,0,2,2},
    {-1,0,0,-1,-1,-1,0,0,2,4},
    {0,0,0,1,0,-1,-1,1,2,4},
    {0,0,-1,1,-1,0,0,1,2,6},
    {1,-1,-1,0,0,0,-1,0,2,7},
    {1,0,0,1,1,1,0,0,2,5},
    {-1,0,0,-1,-1,-1,0,0,2,5},
    {-1,0,0,1,0,1,-1,0,2,6},
    {1,0,0,1,1,1,0,0,2,6},
    {0,-1,1,0,0,0,1,1,2,2},
    {1,0,0,0,1,1,-1,0,2,0},
    {-1,1,0,0,0,1,1,0,2,4},
    {0,0,-1,0,-1,0,0,0,1,2},
    {1,-1,0,0,0,-1,-1,0,2,4},
    {0,0,-1,1,-1,0,0,1,2,0},
    {-1,1,0,0,0,1,1,0,2,5},
    {-1,0,0,0,1,1,-1,0,2,7},
    {0,0,1,-1,1,0,0,-1,2,1},
    {0,0,1,-1,0,1,1,0,2,1},
    {1,-1,0,0,0,-1,-1,0,2,5},
    {-1,0,0,-1,-1,-1,0,0,2,7},
    {0,0,-1,1,-1,0,0,1,2,1}};

static const std::string S3_TABLE_VALUES[S3_TABLE_SIZE] = 
   {"STHSTHSSSTHSSSTWWWWWW",
    "STHSTHSSSTHSSSTWWWWW",
    "STHSTHSSSTHSSSTW",
    "STHSTHSSSTHSSSWWWWW",
    "STHSTHSSSTHSSSWWWW",
    "STHSTHSSSTHSSSWWW",
    "STHSTHSSSTHSSSWW",
    "STHSTHSSSTHSSTWWWWWWW",
    "STHSTHSSSTHSSTWWWWWW",
    "STHSTHSSSTHSSTWWWWW",
    "STHSTHSSSTHSSTWWW",
    "STHSTHSSSTHSSTWW",
    "STHSTHSSSTHSSWW",
    "STHSTHSSSTHSSW",
    "STHSTHSSSTHSS",
    "STHSTHSSSTHSTWWWWWWW",
    "STHSTHSSSTHSTWWWWWW",
    "STHSTHSSSTHSTWWWW",
    "STHSTHSSSTHSTWWW",
    "STHSTHSSSTHST",
    "STHSTHSSSTHSWWWWWWW",
    "STHSTHSSSTHSWWWWWW",
    "STHSTHSSSTHSWW",
    "STHSTHSSSTHS",
    "STHSTHSSSTHTWWWWWW",
    "STHSTHSSSTHTWWWWW",
    "STHSTHSSSTHTWW",
    "STHSTHSSSTHT",
    "STHSTHSSSTHWWWWWW",
    "STHSTHSSSTHWWWWW",
    "STHSTHSSSTHWWW",
    "STHSTHSSSTHWW",
    "STHSTHSSSTHW",
    "STHSTHSSTHSSSTWWWWW",
    "STHSTHSSTHSSSTWWWW",
    "STHSTHSSTHSSSWWWWWW",
    "STHSTHSSTHSSSWWWW",
    "STHSTHSSTHSSSWW",
    "STHSTHSSTHSSTWWWWWWW",
    "STHSTHSSTHSSTWWWWW",
    "STHSTHSSTHSSTWWW",
    "STHSTHSSTHSSTWW",
    "STHSTHSSTHSSSW",
    "STHSTHSSTHSSWWWWWWW",
    "STHSTHSSTHSSWWWWWW",
    "STHSTHSSTHSSWWW",
    "STHSTHSSTHSSW",
    "STHSTHSSTHSTWWWWWWW",
    "STHSTHSSTHSTWWWWWW",
    "STHSTHSSTHSTWWWWW",
    "STHSTHSSTHSTWWW",
    "STHSTHSSTHST",
    "STHSTHSSTHSWWWWWWW",
    "STHSTHSSTHSWWWWWW",
    "STHSTHSSTHSWWWWW",
    "STHSTHSSTHSWWWW",
    "STHSTHSSTHSW",
    "STHSTHSSTHTWWWWWWW",
    "STHSTHSSTHT",
    "STHSTHSSTHWWWWWWW",
    "STHSTHSSTHWWWW",
    "STHSTHSSTHWW",
    "STHSTHSSTHW",
    "STHSTHSTHSSSTWWWWWWW",
    "STHSTHSTHSSSTWWWWW",
    "STHSTHSTHSSSTWWW",
    "STHSTHSTHSSSTWW",
    "STHSTHSTHSSSWWWWWWW",
    "STHSTHSTHSSSWWWWW",
    "STHSTHSTHSSSWWWW",
    "STHSTHSTHSSSWW",
    "STHSTHSTHSSS",
    "STHSTHSTHSSTWWWWWWW",
    "STHSTHSTHSSTWWWWWW",
    "STHSTHSTHSSTWWWWW",
    "STHSTHSTHSSTWW",
    "STHSTHSTHSST",
    "STHSTHSTHSSWWWWWWW",
    "STHSTHSTHSSWWW",
    "STHSTHSTHSSW",
    "STHSTHSTHSS",
    "STHSTHSTHSTWWWWWWW",
    "STHSTHSTHSTWWWWW",
    "STHSTHSTHSTWWW",
    "STHSTHSTHSWWWWWW",
    "STHSTHSTHSWWWW",
    "STHSTHSTHTWWWWWWW",
    "STHSTHSTHTWWWWWW",
    "STHSTHSSSTHSST",
    "STHSTHSTHTWWW",
    "STHSTHSTHWWWWWW",
    "STHSTHSTHWWWWW",
    "STHSTHSTHWWWW",
    "STHSTHSTHWWW",
    "STHSTHSTHW",
    "STHSTHTHSSSTWWWWWWW",
    "STHSTHTHSSSTWWW",
    "STHSTHTHSSSWWWWWW",
    "STHSTHTHSSSWW",
    "STHSTHTHSSS",
    "STHSTHTHSSTWWWWWWW",
    "STHSTHTHSSTWWWWW",
    "STHSTHTHSSTWWW",
    "STHSTHTHSST",
    "STHSTHTHSSWWWWWW",
    "STHSTHTHSSWWWWW",
    "STHSTHTHSSWWW",
    "STHSTHTHSS",
    "STHSTHTHSTWWWW",
    "STHSTHTHSWWWWWW",
    "STHSTHTHSWWW",
    "STHSTHTHSW",
    "STHSTHTHTWWWWWWW",
    "STHSTHTHTWWW",
    "STHSTHTHTWW",
    "STHSTHTHWWWWWW",
    "STHSTHTHWWW",
    "STHSTHTHWW",
    "STHTHSSSTHSSSTWWWWWWW",
    "STHTHSSSTHSSSTWWWWWW",
    "STHTHSSSTHSSSTWWWWW",
    "STHTHSSSTHSSSTWWW",
    "STHTHSSSTHSSSTW",
    "STHTHSSSTHSSSWWWWWWW",
    "STHTHSSSTHSSSWWW",
    "STHTHSSSTHSSSWW",
    "STHTHSSSTHSSTWWWWWW",
    "STHTHSSSTHSSTWWW",
    "STHTHSSSTHSSTWW",
    "STHTHSSSTHSSTW",
    "STHTHSSSTHSSWWWW",
    "STHTHSSSTHSSWWW",
    "STHTHSSSTHSSWW",
    "STHTHSSSTHSTWWWWWW",
    "STHTHSSSTHSTWWWWW",
    "STHTHSSSTHSTWWWW",
    "STHTHSSSTHSTWWW",
    "STHTHSSSTHSWWWWWW",
    "STHTHSSSTHSWWWW",
    "STHTHSSSTHSW",
    "STHTHSSSTHS",
    "STHTHSSSTHTWWWWWWW",
    "STHTHSSSTHTWWWWWW",
    "STHTHSSSTHTWWWWW",
    "STHTHSSSTHTWW",
    "STHTHSSSTHTW",
    "STHTHSSSTHT",
    "STHTHSSSTHWWWWWW",
    "STHTHSSSTHWWWWW",
    "STHTHSSSTHWWWW",
    "STHTHSSSTHWWW",
    "STHTHSSSTHWW",
    "STHTHSSSTH",
    "STHTHSSTHSSSTWWWWW",
    "STHTHSSTHSSST",
    "STHTHSSTHSSSWWWWWW",
    "STHTHSSTHSSSWWWWW",
    "STHTHSSTHSSSWWWW",
    "STHTHSSTHSSSWW",
    "STHTHSSTHSSSW",
    "STHTHSSTHSSS",
    "STHTHSSTHSSTWWWW",
    "STHTHSSTHSSTWWW",
    "STHTHSSTHSSTWW",
    "STHTHSSTHSSTW",
    "STHTHSSTHSST",
    "STHTHSSTHSSWWWWWW",
    "STHTHSSTHSSWWWWW",
    "STHTHSSTHSSW",
    "STHTHSSTHSS",
    "STHTHSSTHSTWWWWWW",
    "STHTHSSTHSTWWWWW",
    "STHTHSSTHSTWWW",
    "STHTHSSTHSTWW",
    "STHSTHSSSTHWWWWWWW",
    "STHTHSSTHSTW",
    "STHTHSSTHSWWWWWWW",
    "STHTHSSTHSWWWWWW",
    "STHTHSSTHSWWWWW",
    "STHTHSSTHSWWWW",
    "STHTHSSTHSWW",
    "STHTHSSTHTWWW",
    "STHTHSSTHTWW",
    "STHSTHSSTHSSSTW",
    "STHTHSSTHT",
    "STHTHSSTHWWWWWWW",
    "STHTHSSTHWWWW",
    "STHTHSSTHWW",
    "STHTHSSTHW",
    "STHTHSSTH",
    "STHTHSTHSSSTWWWWWW",
    "STHTHSTHSSSTWWWW",
    "STHTHSTHSSSTWWW",
    "STHTHSTHSSSTWW",
    "STHTHSTHSSSTW",
    "STHTHSTHSSSWWWWWWW",
    "STHTHSTHSSSWWWWW",
    "STHTHSTHSSSWWWW",
    "STHTHSTHSSSW",
    "STHTHSTHSSTWWWWW",
    "STHTHSTHSSTWW",
    "STHTHSTHSST",
    "STHTHSTHSSWWW",
    "STHTHSTHSSWW",
    "STHTHSTHSTWWWWWWW",
    "STHSTHSSTHSSST",
    "STHTHSTHSTWWWW",
    "STHSTHSSTHSSSTWWWWWWW",
    "STHTHSTHSTWW",
    "STHTHSTHST",
    "STHTHSTHSWWWWWWW",
    "STHTHSTHSWWW",
    "STHTHSTHSWW",
    "STHTHSTHS",
    "STHTHSTHTWWWWWW",
    "STHTHSSSTHWWWWWWW",
    "STHTHSTHTWWWWW",
    "STHTHSTHTWW",
    "STHTHSTHWWWWWWW",
    "STHTHSTHWWW",
    "STHTHSTHWW",
    "STHTHSTHW",
    "STHTHSTHSWWWW",
    "STHTHTHSSSTWWWW",
    "STHTHTHSSSTWW",
    "STHTHTHSSST",
    "STHTHTHSSSWWWWWW",
    "STHTHTHSSSWWWWW",
    "STHTHTHSSSWWWW",
    "STHTHTHSSSWW",
    "STHTHSSSTHSSSWWWWW",
    "STHTHTHSSSW",
    "STHTHTHSSTWWWWWWW",
    "STHTHTHSSTWWW",
    "STHTHTHSSWWWWWWW",
    "STHTHTHSSWWWWWW",
    "STHTHTHSSWWWWW",
    "STHTHTHSSWWWW",
    "STHTHTHSSWW",
    "STHTHTHSTWWWWWWW",
    "STHTHTHSTWWWWWW",
    "STHTHTHSTWWWW",
    "STHTHTHSTW",
    "STHTHTHSWWWWWWW",
    "STHTHTHSWWWWW",
    "STHTHTHSWWWW",
    "STHTHTHSWWW",
    "STHTHTHS",
    "STHTHTHTWWWWWWW",
    "STHTHTHTWWWWWW",
    "STHTHTHTWWWW",
    "STHTHTHTWWW",
    "STHTHTHT",
    "STHTHTHWWWWWWW",
    "STHTHTHWWW",
    "STHTHTHW",
    "SHSTHSSSTHSSSTWWWWWWW",
    "SHSTHSSSTHSSSTWWW",
    "STHSTHTHSTWWWWWW",
    "SHSTHSSSTHSSSTWW",
    "SHSTHSSSTHSSSTW",
    "SHSTHSSSTHSSSWWWWWWW",
    "SHSTHSSSTHSSSWWWWWW",
    "SHSTHSSSTHSSSWWW",
    "SHSTHSSSTHSSS",
    "STHSTHTHSSSWWWW",
    "SHSTHSSSTHSSTWWWWWWW",
    "SHSTHSSSTHSSTWWWWWW",
    "SHSTHSSSTHSSTWWWWW",
    "STHSTHSSTHTWWWWWW",
    "SHSTHSSSTHSSTW",
    "SHSTHSSSTHSST",
    "STHSTHSSSTHSWWW",
    "SHSTHSSSTHSSWWWWWW",
    "SHSTHSSSTHSSWWWWW",
    "SHSTHSSSTHSSWWW",
    "STHSTHSSTHSSSWWW",
    "SHSTHSSSTHSTWWWWWWW",
    "SHSTHSSSTHSTWWWW",
    "SHSTHSSSTHSTWWW",
    "SHSTHSSSTHSWWWWWW",
    "SHSTHSSSTHSWWWW",
    "SHSTHSSSTHSWW",
    "SHSTHSSSTHTWWWWW",
    "SHSTHSSSTHTWW",
    "SHSTHSSSTHSSSW",
    "SHSTHSSSTHT",
    "SHSTHSSSTHWWWWWWW",
    "SHSTHSSSTHWWWWWW",
    "SHSTHSSSTHWWWWW",
    "SHSTHSSSTHWWWW",
    "SHSTHSSSTHWW",
    "SHSTHSSTHSSSTWWWWW",
    "SHSTHSSTHSSSTWWWW",
    "SHSTHSSTHSSSTWWW",
    "SHSTHSSTHSSST",
    "SHSTHSSTHSSSWWWWWW",
    "SHSTHSSTHSSSWWWW",
    "SHSTHSSTHSSSWWW",
    "SHSTHSSTHSSSWW",
    "SHSTHSSTHSSS",
    "SHSTHSSTHSSTWWWWWW",
    "SHSTHSSTHSSTWWWWW",
    "STHTHSTHT",
    "SHSTHSSTHSSTWWWW",
    "SHSTHSSTHSSTW",
    "SHSTHSSTHSSWWWWW",
    "SHSTHSSTHSSWWW",
    "SHSTHSSTHSSWW",
    "SHSTHSSTHSSW",
    "SHSTHSSTHSS",
    "STHSTHSSTHSSSTWW",
    "SHSTHSSTHSTWWWWWWW",
    "SHSTHSSTHSTWWWWW",
    "SHSTHSSTHSTWWWW",
    "SHSTHSSTHSTWWW",
    "SHSTHSSTHSTWW",
    "SHSTHSSTHSWWWWWW",
    "SHSTHSSTHSWWWWW",
    "SHSTHSSTHSWW",
    "STHTHSSSTHSSTWWWW",
    "SHSTHSSTHSW",
    "SHSTHSSTHS",
    "SHSTHSSTHSSSWWWWWWW",
    "SHSTHSSTHTWWWW",
    "SHSTHSSTHTWWW",
    "SHSTHSSTHTWW",
    "SHSTHSSTHTW",
    "SHSTHSSTHT",
    "SHSTHSSTHWWWWWW",
    "SHSTHSSTHWWWWW",
    "SHSTHSSTHWWWW",
    "STHSTHSSTHTW",
    "SHSTHSSTHWWW",
    "SHSTHSSTH",
    "SHSTHSTHSSSTWWWWWW",
    "SHSTHSTHSSSTWWWWW",
    "SHSTHSTHSSSTWWWW",
    "SHSTHSTHSSSTWW",
    "SHSTHSTHSSST",
    "SHSTHSTHSSSWWWWWWW",
    "SHSTHSTHSSSWWWWW",
    "SHSTHSTHSSSWWWW",
    "STHSTHSSSTHSTWWWWW",
    "SHSTHSTHSSSWWW",
    "STHSTHTHWWWWW",
    "SHSTHSTHSSSWW",
    "SHSTHSTHSSS",
    "STHSTHSSSTHSSWWWW",
    "SHSTHSTHSSTWWWWWWW",
    "SHSTHSTHSSTWWWWW",
    "SHSTHSTHSSTWWWW",
    "SHSTHSTHSSTWWW",
    "SHSTHSTHSSTWW",
    "SHSTHSTHSST",
    "SHSTHSTHSSWWWWWWW",
    "SHSTHSTHSSWWWW",
    "SHSTHSTHSSWW",
    "SHSTHSTHSSW",
    "SHSTHSTHSS",
    "SHSTHSTHSTWWWWWWW",
    "SHSTHSTHSTWWW",
    "STHTHSTHSSST",
    "SHSTHSTHSTWW",
    "SHSTHSTHSTW",
    "STHSTHTHWWWWWWW",
    "SHSTHSTHST",
    "SHSTHSTHSWWWWWW",
    "SHSTHSTHSWWWWW",
    "SHSTHSTHSWWW",
    "SHSTHSSSTHSSSWWWWW",
    "SHSTHSTHSWW",
    "SHSTHSTHS",
    "STHSTHTHSSTWWWW",
    "SHSTHSTHTWWWWWWW",
    "STHTHTHSSTWWWW",
    "SHSTHSTHTWWW",
    "SHSTHSTHWWWWWWW",
    "SHSTHSTHWWWW",
    "SHSTHTHSSSTWWWWWWW",
    "SHSTHTHSSSTWWWWW",
    "SHSTHTHSSSTWWWW",
    "SHSTHTHSSSWWWWWW",
    "SHSTHTHSSSWWWWW",
    "SHSTHTHSSSWW",
    "STHTHSSSTHSSWWWWWW",
    "SHSTHTHSSSW",
    "SHSTHTHSSTWWWW",
    "SHSTHTHSSTWWW",
    "SHSTHTHSSTWW",
    "SHSTHTHSST",
    "SHSTHTHSSWWWWWW",
    "SHSTHTHSSWWWWW",
    "SHSTHTHSSWWWW",
    "SHSTHTHSSWWW",
    "STHTHSSTHSSWW",
    "SHSTHTHSS",
    "SHSTHTHSTWWW",
    "SHSTHSSSTHTWWWWWWW",
    "SHSTHTHSTWW",
    "STHTHSTHTWWWW",
    "SHSTHTHSTW",
    "SHSTHTHST",
    "SHSTHTHSWWWWWWW",
    "SHSTHTHSWWWWWW",
    "SHSTHTHSWWWWW",
    "SHSTHTHSWWWW",
    "SHSTHTHSWWW",
    "SHSTHTHSW",
    "SHSTHTHS",
    "SHSTHTHTWWWWWW",
    "SHSTHTHTWWWWW",
    "SHSTHTHTWWWW",
    "SHSTHTHTWW",
    "SHSTHTHT",
    "STHTHSTHSSSTWWWWW",
    "SHSTHTHWWWWWW",
    "SHSTHTHWWWW",
    "SHTHSSSTHSSSTWWWWWW",
    "SHTHSSSTHSSSTWWWWW",
    "SHTHSSSTHSSSTWWW",
    "SHTHSSSTHSSST",
    "SHTHSSSTHSSSWWWWWW",
    "SHTHSSSTHSSSWWWW",
    "SHTHSSSTHSSSWWW",
    "SHTHSSSTHSSSW",
    "SHTHSSSTHSSS",
    "SHTHSSSTHSSTWWWWWWW",
    "SHTHSSSTHSSTWWWW",
    "SHTHSSSTHSSTWWW",
    "SHTHSSSTHSSTWW",
    "SHTHSSSTHSSTW",
    "SHTHSSSTHSSWWWWWW",
    "SHTHSSSTHSSWW",
    "SHTHSSSTHSSW",
    "SHSTHSSTHSWWWW",
    "SHTHSSSTHSTWWWWWWW",
    "SHTHSSSTHSTWWWWWW",
    "SHTHSSSTHSTWWWWW",
    "SHTHSSSTHSTWWWW",
    "SHSTHSSSTHSSSWWWW",
    "SHTHSSSTHSTWWW",
    "SHTHSSSTHST",
    "SHTHSSSTHSWWWWWWW",
    "SHTHSSSTHSWWWWW",
    "SHTHSSSTHSWWWW",
    "SHTHSSSTHSWWW",
    "SHTHSSSTHSW",
    "SHTHSSSTHTWWWWWWW",
    "SHTHSSSTHTWWWWWW",
    "SHTHSSSTHTWWWW",
    "SHTHSSSTHTWWW",
    "SHTHSSSTHTW",
    "SHTHSSSTHT",
    "SHTHSSSTHWWWWWWW",
    "SHTHSSSTHWWW",
    "SHTHSSSTHWW",
    "SHSTHSSSTHST",
    "SHTHSSSTHW",
    "SHTHSSSTH",
    "SHTHSSTHSSSTWWWW",
    "SHTHSSTHSSSTWWW",
    "SHTHSSTHSSST",
    "SHTHSSTHSSSWWWWWWW",
    "SHTHSSTHSSSWWWWWW",
    "SHTHSSTHSSSWWWWW",
    "SHTHSSTHSSSWWW",
    "SHTHSSTHSSSWW",
    "SHTHSSTHSSSW",
    "SHTHSSTHSSTWWWWWW",
    "SHTHSSTHSSTWWW",
    "SHTHSSTHSSTWW",
    "SHSTHSSTHTWWWWW",
    "SHTHSSTHSSWWWWWWW",
    "SHTHSSTHSSWWWWWW",
    "SHTHSSTHSSWWWWW",
    "SHTHSSTHSSWWWW",
    "SHTHSSTHSSWWW",
    "SHTHSSTHSSW",
    "SHTHSSTHSTWWWWWW",
    "SHTHSSTHSTWWWWW",
    "SHTHSSTHSTWWWW",
    "STHTHTHSSSTWWWWWW",
    "SHTHSSTHSTWWW",
    "SHTHSSTHSTWW",
    "SHTHSSTHSTW",
    "SHTHSSTHST",
    "SHTHSSTHSWWWWWWW",
    "SHTHSSTHSWWWWWW",
    "SHTHSSTHSWWWWW",
    "SHTHSSTHSWWWW",
    "SHTHSSTHSWWW",
    "SHTHSSTHSWW",
    "SHTHSSTHSW",
    "SHTHSSTHS",
    "SHTHSSTHTWWWWW",
    "SHTHSSTHTWWWW",
    "SHTHSSTHTW",
    "SHTHSSTHT",
    "SHTHSSTHWWWWWWW",
    "SHTHSSTHWWWWWW",
    "SHTHSSTHWWWWW",
    "SHTHSSTHWWWW",
    "SHTHSSTHWWW",
    "SHTHSSSTHSS",
    "SHTHSSTHWW",
    "SHTHSSTHW",
    "SHTHSSTH",
    "SHTHSTHSSSTWWWWWWW",
    "SHSTHSSSTHSTWWWWW",
    "SHTHSTHSSSTWWWWW",
    "SHTHSTHSSSTWWWW",
    "SHTHSTHSSST",
    "SHTHSTHSSSWWWWWW",
    "SHTHSTHSSSWWWWW",
    "SHTHSTHSSSWWW",
    "STHSTHSTHSWWWWW",
    "SHTHSTHSSTWWWWWWW",
    "SHSTHSTHSTWWWW",
    "SHTHSTHSSTWWWWW",
    "SHTHSTHSSTWWWW",
    "SHTHSTHSSTW",
    "SHTHSTHSST",
    "SHTHSTHSSWWWWW",
    "SHTHSTHSSWWWW",
    "SHTHSTHSSWWW",
    "SHTHSTHSTWWWWWWW",
    "SHTHSTHSTWWWWW",
    "SHTHSTHSTWWWW",
    "SHSTHSTHWWWWW",
    "SHTHSTHSTW",
    "SHTHSTHSWWWWWW",
    "SHTHSTHSWWWWW",
    "SHTHSTHSW",
    "SHTHSTHTWWWWWWW",
    "SHTHSTHTWWWWW",
    "SHTHSTHTWWWW",
    "SHSTHSSSTHSWWW",
    "SHTHSTHTWW",
    "SHTHSTHT",
    "SHTHSTHWWWWW",
    "SHTHSTHWWWW",
    "SHTHSTHWWW",
    "STHSTHTHSSTW",
    "SHTHSTHWW",
    "SHTHSTHW",
    "SHTHTHSSSTWWWWWWW",
    "SHTHTHSSSTWWWW",
    "SHTHSSSTHSST",
    "SHTHTHSSSTWWW",
    "SHTHTHSSSTWW",
    "SHTHTHSSSTW",
    "SHTHTHSSST",
    "SHTHTHSSSWWWWWW",
    "SHTHTHSSSWWWWW",
    "SHTHSSTHSSTW",
    "SHTHTHSSSWWWW",
    "STHTHSSTHSSWWWW",
    "SHTHTHSSSWW",
    "SHTHTHSSSW",
    "SHTHTHSSS",
    "SHTHTHSSTWWWWWW",
    "SHTHTHSSTWWWW",
    "STHTHSSSTHSWWWWW",
    "SHTHTHSSTWW",
    "SHTHTHSSTW",
    "SHTHTHSST",
    "SHTHTHSSWWWWW",
    "SHTHTHSSWW",
    "SHTHSSSTHSSTWWWWW",
    "SHTHSTHSS",
    "SHTHTHSS",
    "SHTHTHSTWWWWW",
    "SHTHTHSTWWWW",
    "SHTHTHSTWW",
    "SHTHTHSTW",
    "SHTHTHSWWWW",
    "STHSTHTHST",
    "SHTHTHSWWW",
    "SHTHTHSWW",
    "SHTHTHSW",
    "SHTHTHTWWWWWWW",
    "SHTHTHTWWWWW",
    "SHTHTHTWWWW",
    "SHTHTHTW",
    "SHTHTHWWWWWWW",
    "SHTHTHWWWWWW",
    "SHTHTHWWWW",
    "STHSTHSTHSW",
    "SHTHTHWW",
    "SHTHTHW",
    "THSTHSSSTHSSSTWWWWWW",
    "THSTHSSSTHSSSTWWWWW",
    "THSTHSSSTHSSSTW",
    "THSTHSSSTHSSST",
    "THSTHSSSTHSSSWWWWWW",
    "THSTHSSSTHSSSWWWWW",
    "THSTHSSSTHSSSWW",
    "STHSTHSTHSSWWWWW",
    "THSTHSSSTHSSTWWWWWWW",
    "THSTHSSSTHSSTWWWWWW",
    "STHTHTHSW",
    "THSTHSSSTHSSTWWW",
    "THSTHSSSTHSSTWW",
    "STHSTHSTHSWWW",
    "THSTHSSSTHSST",
    "SHSTHSSTHSSSTWWWWWWW",
    "SHTHTHST",
    "THSTHSSSTHSSWWWW",
    "THSTHSSSTHSTWWWWWW",
    "THSTHSSSTHSTWWWWW",
    "THSTHSSSTHSTWW",
    "THSTHSSSTHST",
    "THSTHSSSTHSWWWWWWW",
    "THSTHSSSTHS",
    "THSTHSSSTHTWWWW",
    "THSTHSSSTHTWWW",
    "STHSTHSSTHSSWWWW",
    "THSTHSSSTHTWW",
    "STHSTHSSTHTWWW",
    "THSTHSSSTHTW",
    "THSTHSSSTHT",
    "THSTHSSSTHWWWWWWW",
    "THSTHSSSTHWWWWWW",
    "STHSTHSSSTHSWWWWW",
    "THSTHSSSTHSWWW",
    "THSTHSSSTHWWWW",
    "THSTHSSSTHWW",
    "THSTHSSSTHW",
    "THSTHSSSHSSSTWWWWWWW",
    "STHSTHSSTHTWWWWW",
    "THSTHSSSHSSSTWWWW",
    "THSTHSSSHSSSTWWW",
    "THSTHSSSTHSSW",
    "THSTHSSSHSSSTWW",
    "THSTHSSSHSSSTW",
    "SHTHTHSSWWW",
    "THSTHSSSHSSSWWWWWWW",
    "THSTHSSSHSSSWWWWWW",
    "THSTHSSSHSSSWWWWW",
    "THSTHSSSHSSSWWWW",
    "THSTHSSSHSSSWWW",
    "THSTHSSSHSSSWW",
    "THSTHSSSHSSSW",
    "STHTHTHSWW",
    "THSTHSSSHSSS",
    "THSTHSSSHSSTWWWWWWW",
    "THSTHSSSHSSTWWWWWW",
    "THSTHSSSTHSS",
    "THSTHSSSHSSTWWWWW",
    "THSTHSSSHSSTWWWW",
    "STHTHSTHSSSTWWWWWWW",
    "THSTHSSSHSSTWWW",
    "THSTHSSSHSSTW",
    "THSTHSSSHSSWWWWWW",
    "THSTHSSSHSSWW",
    "STHSTHSTHSSSTWWWWWW",
    "SHSTHSSSTHTW",
    "THSTHSSSHSS",
    "THSTHSSSHSTWWWWWWW",
    "SHTHSTHSSSTWW",
    "THSTHSSSHSTWWWWWW",
    "STHTHSSTHTWWWWWW",
    "THSTHSSSHSTWWWWW",
    "THSTHSSSHSTWWWW",
    "THSTHSSSHSTWWW",
    "THSTHSSSHSTW",
    "THSTHSSSHST",
    "THSTHSSSHSWWWWWW",
    "THSTHSSSHSWWWW",
    "THSTHSSSHSWW",
    "THSTHSSSHTWWWWWWW",
    "SHSTHTHTWWWWWWW",
    "THSTHSSSHTWWWWWW",
    "THSTHSSSHTWWWW",
    "THSTHSSSHTWW",
    "SHSTHTHSTWWWW",
    "THSTHSSSHWWWWWWW",
    "SHSTHSTHSSTWWWWWW",
    "THSTHSSSHWWWWWW",
    "THSTHSSSHWWWWW",
    "THSTHSSSHWW",
    "THSTHSSSHW",
    "THSTHSSSH",
    "THSTHSSTHSSSTWWWWWWW",
    "THSTHSSTHSSSTWWWWWW",
    "THSTHSSTHSSSTWWWWW",
    "THSTHSSTHSSSTWWWW",
    "THSTHSSTHSSSTWWW",
    "THSTHSSTHSSSTWW",
    "THSTHSSTHSSST",
    "STHSTHSSTHSWW",
    "SHTHTHT",
    "THSTHSSTHSSSWWWWWWW",
    "THSTHSSTHSSSWWWWW",
    "THSTHSSTHSSSWWWW",
    "THSTHSSTHSSSW",
    "THSTHSSTHSSS",
    "SHTHTHSWWWWWWW",
    "THSTHSSTHSSTWWWW",
    "STHSTHSSTHSS",
    "THSTHSSTHSSTWW",
    "THSTHSSTHSSTW",
    "SHSTHSTHSWWWW",
    "THSTHSSTHSST",
    "THSTHSSTHSSWWWWWWW",
    "STHTHSTHWWWW",
    "THSTHSSTHSSWWWWWW",
    "STHTHTHTWW",
    "THSTHSSTHSSWW",
    "THSTHSSTHSS",
    "THSTHSSTHSTWWWWWWW",
    "THSTHSSTHSTWWWWWW",
    "STHSTHSSTHSTW",
    "STHTHSSSTHW",
    "THSTHSSTHSTWWWWW",
    "THSTHSSTHSTWWW",
    "STHTHSSSTHSSTWWWWW",
    "SHSTHSTHSTWWWWWW",
    "THSTHSSTHSTWW",
    "THSTHSSTHSTW",
    "THSTHSSTHST",
    "THSTHSSTHSWWWWW",
    "THSTHSSTHSWW",
    "THSTHSSTHSW",
    "STHSTHTHSWWWWWWW",
    "SHTHSTHSSSW",
    "THSTHSSTHS",
    "THSTHSSTHTWWWWWWW",
    "THSTHSSTHTWWWWWW",
    "THSTHSSTHTWWWW",
    "THSTHSSTHTWWW",
    "THSTHSSTHTWW",
    "THSTHSSTHT",
    "STHTHSTHSSTWWWW",
    "THSTHSSTHWWWWW",
    "THSTHSSTHWW",
    "THSTHSSTH",
    "THSTHSTHSSSTWWWWWWW",
    "THSTHSTHSSSTWWWWWW",
    "THSTHSSSTHSSSTWW",
    "THSTHSTHSSSTWWWW",
    "THSTHSTHSSSTWW",
    "THSTHSTHSSSTW",
    "THSTHSTHSSSWWWWWW",
    "THSTHSTHSSSWWWWW",
    "THSTHSTHSSSWWWW",
    "THSTHSTHSSSWW",
    "THSTHSTHSSSW",
    "THSTHSTHSSS",
    "THSTHSTHSSTWWWWW",
    "THSTHSTHSSTWW",
    "THSTHSTHSSTW",
    "THSTHSTHSST",
    "THSTHSTHSSWWWWWWW",
    "THSTHSSSHS",
    "THSTHSTHSSWWWWW",
    "THSTHSTHSSWWWW",
    "STHTHTHTWWWWW",
    "THSTHSTHSSWW",
    "THSTHSTHSSW",
    "THSTHSTHSS",
    "STHSTHSTHSTWWWW",
    "THSTHSTHSTWWWWWW",
    "THSTHSTHSTWWW",
    "THSTHSTHSTWW",
    "THSTHSTHSWWWWWW",
    "THSTHSTHSWWWWW",
    "SHTHTHSTWWW",
    "THSTHSTHSWWWW",
    "THSTHSTHSWWW",
    "THSTHSTHSWW",
    "THSTHSTHTWWWW",
    "THSTHSTHTWWW",
    "THSTHSTHTW",
    "THSTHSTHT",
    "SHTHSSSTHTWW",
    "THSTHSTHWWWWWW",
    "THSTHSTHWWWWW",
    "THSTHSTHWWWW",
    "THSTHSTHWW",
    "THSTHSTH",
    "THSTHSSSTHTWWWWW",
    "THSTHTHSSSTWWWWWWW",
    "THSTHTHSSSTWWWWWW",
    "STHTHTHSSSTW",
    "THSTHTHSSSTWWW",
    "THSTHTHSSSTWW",
    "THSTHTHSSSTW",
    "THSTHTHSSSWWWWWW",
    "STHTHTHWW",
    "THSTHTHSSSWWWWW",
    "THSTHTHSSSWW",
    "THSTHTHSSS",
    "THSTHTHSSTWWWWWW",
    "THSTHTHSSTWW",
    "THSTHTHSSTW",
    "THSTHTHSST",
    "THSTHTHSSWWWWWWW",
    "THSTHTHSSWWWWWW",
    "SHTHSSTHSSTWWWWWWW",
    "THSTHTHSSWWWWW",
    "THSTHTHSSWWW",
    "THSTHTHSSWW",
    "THSTHTHSSW",
    "SHSTHSSTHSWWWWWWW",
    "THSTHTHSTWWWWWWW",
    "STHSTHTHSTWW",
    "THSTHTHSTWWWWWW",
    "THSTHTHSTWWWW",
    "THSTHTHSTWW",
    "THSTHTHSTW",
    "THSTHTHSWWWWWWW",
    "THSTHTHSWWWWW",
    "THSTHTHSWWWW",
    "THSTHTHSWWW",
    "THSTHTHSWW",
    "THSTHTHS",
    "THSTHTHTWWWWWW",
    "THSTHTHTWWWWW",
    "THSTHTHTWWWW",
    "SHTHSSTHSSSTW",
    "THSTHSSSHT",
    "THSTHTHTWWW",
    "THSTHTHTW",
    "THSTHTHWWWWWWW",
    "THSTHTHWWWWWW",
    "THSTHTHWWWWW",
    "THSTHTHWWW",
    "THSTHTHWW",
    "THSTHTHW",
    "THSTHTH",
    "THSHSSSTHSSSTWWWWWWW",
    "THSHSSSTHSSSTWWWW",
    "THSHSSSTHSSSTWWW",
    "SHTHSSSTHSSSWW",
    "THSHSSSTHSSSTWW",
    "STHSTHSTHSSSTWWWW",
    "STHSTHSTHTWW",
    "THSHSSSTHSSST",
    "SHTHSTHWWWWWW",
    "THSHSSSTHSSSWWWWWW",
    "THSHSSSTHSSSWWWWW",
    "HSHSTHSWWWW",
    "SHTHSTHS",
    "THTHSHSSTWWWWWW",
    "THSHTHSSTWWWWW",
    "SHSTHTHTWWW",
    "HSHSTHSWWW",
    "HSHSSSHSSTW",
    "HSHSTHSWW",
    "HSHSTH",
    "HSTHSTHSSSTWWWWW",
    "HSHTHSSSTWWWWWW",
    "HSHTHSSSTWWWW",
    "STHSTHSTHTWWWW",
    "HSHTHSSSTWWW",
    "SHSWWWWWW",
    "SSSHSWWW",
    "SHTHSSSTHSSSTW",
    "HSTHSSTHSSSW",
    "HSHTHSSSTWW",
    "HTHSTHSSSTWWW",
    "HSHTHSSSTW",
    "THSHSTHSWWWW",
    "THTHTHS",
    "STHSTHTHSSSWWW",
    "HSHTHSSSWWWW",
    "SHSTHSSTHSWWW",
    "SSSHSSSWWWWWW",
    "HSHTHSSTWWWWWWW",
    "HTHSSSTHSSWWWWW",
    "SSSTHSWW",
    "HSHSSTHSSSTWWWW",
    "HSHTHSSTW",
    "HTHT",
    "HSTHTHSWWWWWW",
    "THSHSSTHSTWW",
    "HSHTHSSWWWW",
    "THSHSSSTHSSSTWWWWWW",
    "THTHSSSHWWWWWW",
    "HSHTHSSWW",
    "STHSTWWWWW",
    "HSTHSSTHTWW",
    "HSHTHSTWWWWWW",
    "SHTHTHSTWWWWWW",
    "HSTHSSTHTW",
    "HSHTHSTWWW",
    "HSTHSSSHW",
    "HSHTHSTW",
    "STHSTHTH",
    "HSHSSTHSSSWWWWW",
    "THTHSSTHWWWW",
    "STHTHSTHSSTWWW",
    "HSHTHSWWWWW",
    "HSHSTHWWW",
    "THSTWWWWW",
    "HSHTHTWWWWWW",
    "STHSSWWWWW",
    "SHSTHTH",
    "HSHTHTWWWWW",
    "SSTHSSTWWW",
    "HSHTHSSSWWWWW",
    "SHSTHSSSTHTWWWW",
    "HSHTHTW",
    "STHTHSTHSSWWWWWW",
    "HSHTHWWWWWW",
    "HTHSSSTHWWWWW",
    "HSHTHWW",
    "STHSTHSSTHSSWWWWW",
    "HSHTH",
    "HTHTHWWW",
    "SSSHSSW",
    "HTHSSSTHSSSTWWWW",
    "HTHSSSTHSSSTWWW",
    "HSSSWWWWW",
    "STHTHSSTHSW",
    "HTHSSSTHSSSWWWWWWW",
    "HSTHSSTHSWWW",
    "SHTHSTHSWWW",
    "THTHSSW",
    "HTHSSSTHSSSWW",
    "HTHSWWWWW",
    "HTHSSSTHSSS",
    "STHTHTHSS",
    "THSSWWWWWWW",
    "HSHSTHT",
    "HSHTHS",
    "HSTHSSSHSSSWWWW",
    "HTHSSSTHSSTWWWW",
    "HTHSSSTHSSTW",
    "HTHSSSTHSSTWW",
    "HSTHSSSHSSSWWW",
    "THSTHSTHSSST",
    "THTHSSSTHTWWWWWW",
    "SHTHSSSTHWWWWWW",
    "HSHTHSSTWW",
    "SHSTHSTHTW",
    "SSSH",
    "SHSTHSSTHTWWWWWW",
    "HSHTHSST",
    "HTHSSSTHSSWW",
    "THTHSSTHTWWWWWW",
    "HTHSSSTHSTWW",
    "SSHSST",
    "HTHSSSTHSTW",
    "HSHTHW",
    "HTHSSSTHSWWWWWWW",
    "HSHSSTHSSWWWW",
    "THTHSTHSSWWW",
    "HSTHSSTHSW",
    "THSHSTHWWWWWW",
    "HTHSSSTHS",
    "SHSTHTHSSTWWWWWWW",
    "HTHSSTHSST",
    "HSHSTHSSSWW",
    "THTHSHSSTWWWWWWW",
    "SHTHSTHSSWWWWWW",
    "THTHSSTHSSSWWW",
    "HTHTHSSWWW",
    "HTHSSSTH",
    "HSHSSTHSSTWWWW",
    "THSHSSTHSTWWWWW",
    "HTHSSSTWWWW",
    "HTHSSSTWW",
    "SHTHTHWWWWW",
    "SHTHTHWWW",
    "HTHSSSTW",
    "STHSTHSSTHSTWW",
    "THTHSTHSSTWW",
    "HTHSSST",
    "THSTHSTHSSTWWWW",
    "THSTHSTHWWW",
    "HTHSHSSWW",
    "HTHSTHSSSWWWW",
    "SHSTHSTHTWWWW",
    "HTHSSSHSSSTWWWWW",
    "SSTHWWWWW",
    "HSTHSSSTHWWWWWWW",
    "HTHSSSHSSSTWWWW",
    "HSHTHTWWWW",
    "HTHSSTHS",
    "HTHSSSHSSSTWWW",
    "SHTHSSTHTWW",
    "HTHSSSHTWW",
    "HTHSSTWW",
    "HTHSSSHSSSTW",
    "HSHSTHWW",
    "HTHTHTWWWWWW",
    "HTHSSSHSSST",
    "HTHSSSHSSSWWWWWWW",
    "THSHTHSSTWW",
    "HTHSSSHSSSWWWWW",
    "SHSTHSTH",
    "THTHSTHWWWWWWW",
    "HTHSSSHSSSWWW",
    "SHTHTHSSSTWWWWW",
    "HTHSSSHSSS",
    "HTHSSSTHWWW",
    "HTHSSSHSST",
    "HSHTHT",
    "STHTHSTH",
    "HTHSSSHSSWWWWW",
    "HTHSSSHSSWWW",
    "HTHSSSHSSW",
    "STHTHTH",
    "HTHSSSHSTWWWWWWW",
    "THSHSSTHTWWWWWW",
    "HTHSSSTHW",
    "HTHSSSHSWWW",
    "THTHSSSTHSSTWWWWW",
    "HTHSSSHS",
    "HSTHSTHSSS",
    "HTHSSSHTW",
    "HTHSSSHT",
    "HSHSSSTHTWWWWWWW",
    "HSHTHSSS",
    "SHSTHSTHSSSWWWWWW",
    "SHSTHSTHSTWWWWW",
    "HTHSSSHWWWWW",
    "THSHSSTHSSSTWW",
    "HTHSSSHWW",
    "HTHSSSWW",
    "HSHSSSTHWWW",
    "HTHSSTHSSSTWWWWWW",
    "STHSSTWWWWWWW",
    "THSTHSSSHTWWW",
    "THSTHSTHSW",
    "SSTHTWWW",
    "HSTHSTHSSTWWWWWWW",
    "THTHSSTHSTWWW",
    "HTHSSSTWWW",
    "SSHSTWWWWWW",
    "HTHSSTHSSSTWW",
    "HTHSSTHSSST",
    "THSTHSTHSTWWWWW",
    "HTHSSTHSSSWWWW",
    "TWWWW",
    "HTHSSTHSSTWWWWWW",
    "THSTHTHWWWW",
    "HTHSSTHSSTWWWWW",
    "HTHSTHSWWW",
    "SHSTHTHSWW",
    "T",
    "THSHSSTHTW",
    "STHSTHSTHSSSWWWWWW",
    "HTHSSTHSSTWWW",
    "STHTHSTHTW",
    "HTHSSSHWWWWWWW",
    "HTHSSTHSSTW",
    "HTHSSTHSSWWWWWW",
    "HTHSSWWWWW",
    "THTHTHSSSWWW",
    "STHSTHSTHTW",
    "HTHSSTHSSWWWW",
    "THSHSTHSTWWWWWWW",
    "HTHSSTHSSWWW",
    "HTHSSSHSTWW",
    "HTHSSSTHSSWWWWWWW",
    "STHSTHSSTHWWW",
    "HTHSSTHSSWW",
    "HTHSSTHSSW",
    "HTHSSTHSTWWWW",
    "SHTHSTHTW",
    "THSHSTHSSSTWW",
    "STHTHTHTW",
    "HTHSSTHST",
    "THSTHTHSSSTWWWW",
    "HTHTHSTWWWW",
    "HTHSSSHSWWWWW",
    "HSHTHWWWW",
    "HTHSSTHSWWWWW",
    "SSHSSSTWWWWW",
    "HSWWWWWW",
    "STHSTHSSSTHTWWWWWWW",
    "THTHSHSSST",
    "HTHSSTHSWWWW",
    "HTHSSTHSWW",
    "THTHSSSTHSST",
    "HSHSTHTWW",
    "THSTHSTHWWWWWWW",
    "STHSSTWWWW",
    "THSTHSSTHSWWWWWWW",
    "HTHTHWWWWW",
    "HTHSSTHTWWW",
    "HTHSSSWWWWWWW",
    "HSHSSTHSTW",
    "HTHSSTHTWW",
    "HSTHSSSHSTWWWW",
    "THTHSWW",
    "HTHSSTHT",
    "SHTHSSTHSSWW",
    "SHTHSTHSWWWW",
    "HTHSSTHWWWWWW",
    "STHSTHSSSTHSSWWWWW",
    "HTHSSTHWWWWW",
    "HTHSSTHWWWW",
    "HTHSSTHWW",
    "SWWWWWWW",
    "HSTHSSSHSSSTWWWWWWW",
    "THSTHSSSHSW",
    "HTHSSSTHT",
    "HTHSSTWWWWWWW",
    "HTHSSTWWWWW",
    "HTHSSTWWWW",
    "SHSTHSSTHTWWWWWWW",
    "SHTHSSTHSSTWWWW",
    "HTHSSSTHSSST",
    "STHSTHTHSSSWWWWW",
    "SHTHSSSTHSWW",
    "HSHTHSSSWW",
    "HSTHTHSSTWW",
    "HTHSSSHSWWWWWW",
    "HTHSSTWWW",
    "SHSTHSSSTHS",
    "HSTHSSSTHSSTWWWW",
    "THTHTHTWWWWW",
    "STHSTHTHSSW",
    "HTHSST",
    "THTHSSSTHSSTW",
    "THSHSSSTHWWWWW",
    "HTHSSWWWWWWW",
    "THSHSSTHSSWW",
    "HSHSTHSSTWWWWWWW",
    "THTHSTHSSSWWWW",
    "HTHSTHSSSTWWWWWWW",
    "THSHSTHSSSWWWWWWW",
    "HTHSSTHSWWWWWWW",
    "HTHSTHSSSTWWWWW",
    "THSTHTHTWW",
    "HSTHSSTHSTWW",
    "THSHSTHSSTW",
    "HSHTHSSSW",
    "HTHSTWWWWW",
    "HTHSSSTHSTWWWWWWW",
    "HSHSTHS",
    "SSSHWW",
    "STHTHSSTHSTWWWWWWW",
    "THSTHSSSTHSSS",
    "THSHSSTHSSWWWWWW",
    "HTHSTHSSTWWWWWWW",
    "THTHTWWWWWW",
    "HTHSSTHSSSW",
    "SHTHSTHWWWWWWW",
    "HTHSTHSSTWWWWWW",
    "HTHSTHSSTWWWW",
    "HSTHSSTHTWWWWWWW",
    "SHSTHTHSTWWWWWWW",
    "HTHSTHSSTWWW",
    "HTHSSSHWWWWWW",
    "HTWWW",
    "THSHTHTWW",
    "THSTHSTHSSSTWWW",
    "HTHSTHSSWWWWWWW",
    "THSTHSSSTHSTW",
    "SSSHSSWWWWWWW",
    "HTHSTHSS",
    "HSHTHSWWWWWWW",
    "SHSTHSSSTHSS",
    "HTHSTHSTWWWWWW",
    "HTHSTHSTWW",
    "THSTHSSSTHSSSWWW",
    "HSHSSSTHS",
    "HTHSTHSTW",
    "STHTHSTHSTWWWWW",
    "THSHSTHW",
    "HTHSTHSWWWWWWW",
    "HTHSTHSWWWWW",
    "HTHTWWWWWWW",
    "THTHSTHSSTWWWW",
    "SHTHSSSTHSWWWWWW",
    "STHSWWWW",
    "STHSTHTHT",
    "HTHSSSTHSSTWWWWWWW",
    "HTHSTHSSTW",
    "SSHSSSWWWWW",
    "HSHSSSHSSWWW",
    "STHSTHTHSSSW",
    "THSHSSSTHSSS",
    "SHTHSSSTHWWWW",
    "HTHSTHTWWWWWWW",
    "HTHSHTW",
    "SHSSSW",
    "SSSWW",
    "HTHSTHTW",
    "HTHSHWW",
    "HTHSTHWWWW",
    "HTHSSSHTWWWWW",
    "HSHSTHSSSTWWWWW",
    "HSHTHSSTWWW",
    "SSTHSW",
    "THTHSSSHSSSTWWWWWW",
    "SHSTHTHSSST",
    "HTHSTWWWWWW",
    "HTHSTWWWW",
    "THTHTHWWWW",
    "HTHSTWWW",
    "STHTHTHSSSTWWW",
    "HTHSTWW",
    "HTHST",
    "HTHSHSSSTWWWWWWW",
    "SHTHTH",
    "SSTHSTWW",
    "THTHSSSTHSSW",
    "SHSTHTHWW",
    "THTHSTHSSSTWWWWW",
    "HSTHSSTHSSSWW",
    "HTHSHSSSWWWWW",
    "HTHSHSSSWWW",
    "HSHSSSHSW",
    "THTHSTHSWWWW",
    "THSHTHSSSTWWWWWW",
    "HSTHSSSTHSSWWWWWW",
    "THSHSSTHSSWWWWWWW",
    "HTHSHSSSWW",
    "HTHSHSSS",
    "THTHSSTHSWWWWWWW",
    "HTHSSSTHWWWWWWW",
    "HTHSHSSTWWWWWWW",
    "HTHSTHSTWWW",
    "STHTHSTHSTW",
    "HTHSHSSTWWWW",
    "STHSTHTHSSSTWWWWWW",
    "SSSHSTWWWW",
    "SHSTHSTHSW",
    "THSWWWW",
    "HTHSHSSSTWWWWW",
    "HSTHSTHSSTWWWWWW",
    "THSTHSSSTHSSTWWWWW",
    "HTHSHSSWWWWWWW",
    "STHST",
    "HTHSSSHST",
    "SHSTHTHSSSWWWWWWW",
    "IWWW",
    "THTW",
    "HTHSSSHSTW",
    "HTHSTHSSW",
    "HTHSHSS",
    "HTHSSSTWWWWWWW",
    "THSHSSTHSTWWW",
    "HTHSSSTWWWWWW",
    "THTHSSSTHSSSWWWWWWW",
    "HTHSHSTWWWW",
    "HSTHSSSHSSW",
    "STHSTHSTHT",
    "HTHSHSWWWWWWW",
    "HTHSHSWWWWWW",
    "SSSHSWWWWWW",
    "THTHTHSWWW",
    "STHSTHTHSSSTWWWW",
    "HTHSHSWWW",
    "HTHSTHSSSTWW",
    "HTHSHSWW",
    "SHTHSTHSSSWW",
    "HTHSTHSSSWWW",
    "SHTHSTHTWWW",
    "HTHSHSW",
    "HTHTHSTWWWWWWW",
    "HTHWWWWW",
    "HTHSHTWWWWWWW",
    "STHTHTHSSTWWWWW",
    "STHSSSTWWWW",
    "SSHTWWWW",
    "STHTHTHSSSTWWWWWWW",
    "HTHTHSSW",
    "HSHSTHSSS",
    "HTHSHTWWWWWW",
    "STHSTHSTHSSST",
    "HSTHTHSSWWWWWW",
    "HTHSSTW",
    "HTHSHTWWWWW",
    "HTHSSTHSSTWW",
    "SSTHSSWWWWWWW",
    "HTHSHTWWWW",
    "HTHSHSSWWWWW",
    "HTHSHSSTW",
    "HSTHSSSTHSSWWWWW",
    "HSHTHTWWWWWWW",
    "HTHSHWWWWWWW",
    "STHSTHSSSTHTWWWW",
    "HTHSSSHSSSTWW",
    "STHTHSTHSSTWWWWWWW",
    "HTHSHWWWWWW",
    "HTHSSTHSTW",
    "HTHSHWWWWW",
    "HTHSHWWWW",
    "HSTHSSTHSSSTWWWWWW",
    "STHTHSTHTWWWWWWW",
    "HTHSSSHTWWW",
    "STHSTHSTHS",
    "SSWWWWWWW",
    "HTHSHW",
    "STHSTHSSSTHSSSW",
    "HTHSH",
    "HTHSWWWWWWW",
    "HTHSTHSSWWWWWW",
    "HSHSSSTHSTWWW",
    "STHSTHTHSSSTWWWWW",
    "THTHSHSSWWWWWWW",
    "THTHSSTHSSWWWWWWW",
    "STHTHSSTHWWWWWW",
    "HSHSSSTHTWWW",
    "HTHS",
    "HTHSSTHWWWWWWW",
    "SHSTHSSSTHTWWWWWW",
    "HTHSSTHTWWWWWWW",
    "THTHSSTHWWWWW",
    "STHSTHSSTHSSWW",
    "HTHTHSSSTWWWWWWW",
    "HSHTHSSSTWWWWWWW",
    "THSHTHSSWWWWWW",
    "HTHSW",
    "THSTHSSSHWWWW",
    "HTHSSSTHWWWWWW",
    "SHSTHSTHSSSTWWWWWWW",
    "HTHTHSSSTWWWWW",
    "THSTHSSSTHWWW",
    "THTHSSSHSST",
    "THSTHTHSSSWWW",
    "HTHSSSTHSWWWWWW",
    "HSHTHWWW",
    "HSTHSSSHSTWW",
    "HTHTHSSSTWW",
    "STHSTHSSTHSSSTWWWWWW",
    "THSTHSSTHSWWWW",
    "HTHSSTHSTWW",
    "HTHTHSSSTW",
    "THSHSTHSWWWWW",
    "HTHSSSHSSTWWWW",
    "HTHTHSSSWWWW",
    "HTHSSWWW",
    "HTHSSS",
    "HTHTHSTWWWWWW",
    "HTHTHSSSWWW",
    "STHSTHSTHSWWWWWWW",
    "HSHTHWWWWWWW",
    "HSTHSSSHSSSTWW",
    "THSHSSTHSSSTWWWW",
    "HTHSSTHSTWWWWWW",
    "SSHTWWWWWWW",
    "HTHTHSSSWW",
    "THSTHSSSTHSWW",
    "HSTHSTHSSTWWWWW",
    "HSHTHSSTWWWWWW",
    "HTHSSSH",
    "HTHTHSSTWWWWWWW",
    "HTHTHSSTWWWWWW",
    "STHSTHSTHSTWWWWWW",
    "HTHSSSTHSSWWWWWW",
    "SSSTHSST",
    "STHTHTHSST",
    "HSHSSTHSWWW",
    "HTHSTHSWW",
    "HTHTWWWWW",
    "HTHTHSSTW",
    "HSHSSSTHSSSWWW",
    "THSTHSTHW",
    "HTHSTHWWWWWW",
    "HTHTHSST",
    "SHSTHSSTHSSWWWWWW",
    "HSHSSSHST",
    "STHSTHSTHTWWWWW",
    "HTHTHSSWWWWWWW",
    "SHSTHSSSTHSSSTWWWWW",
    "HTHSSSHSSSW",
    "HTHTHSSWWWWW",
    "THTHSSSTHSSSTWWWWW",
    "HTHSSWW",
    "HSHTHSSTWWWWW",
    "STHTHSSSTHSSSWWWWWW",
    "HTHTHSSWWWW",
    "HTHTHSS",
    "THSTHSSTHSSSWWW",
    "HTHSSSHSWW",
    "SHSTHSSTHSSSTW",
    "STHTW",
    "SHSTHTHSSTWWWWWW",
    "HTHTHSTWW",
    "THTH",
    "HTHTHSTW",
    "HSHSSSHSSSWW",
    "HTHTHSWWWWWWW",
    "HSHSSTHSST",
    "HSHSSSTHSSSTWWW",
    "HSHSSSTHSSSTWWWW",
    "STHTHTHWWWW",
    "HTHSHST",
    "HSTHSTHTWWWWWWW",
    "STHSTHTHSSWWWWWWW",
    "HTHTHSWWWWWW",
    "THTHSSSWWWWWW",
    "HTHSSSHSTWWWWW",
    "STHTHSSTHTWWWWWWW",
    "HTHTHSWW",
    "THTHSTHSSSWWW",
    "HTHSHTWWW",
    "HTHSSTHTWWWW",
    "SHTHSTHSSTWWW",
    "THTHSSSHSSSWWWW",
    "HSTHTHSSWWWWW",
    "HTHTHTWWWWW",
    "HTHTHTWWWW",
    "TWWW",
    "HTHSSSTHSSSTWWWWW",
    "SHSTHSSTHSSSTWW",
    "IWWWW",
    "STHTHSTHSSTW",
    "HTHTHWWWWWWW",
    "HTHTH",
    "HTHTWWWWWW",
    "THSTHSSSHSSWWWW",
    "HTHSSSHSTWWW",
    "HSTHSSTHST",
    "STHSTHSSTHSSTWWWWWW",
    "THSHSSSTHWW",
    "HTHTWW",
    "HTHSSTHSSSWWWWW",
    "HTHTW",
    "STHTHSSTHSSSTWWW",
    "HTHSHSTWWWWWWW",
    "HSHSSTHSW",
    "STHSTHSSSTHSSTWWWW",
    "SHSTHSSSTHSTWWWWWW",
    "HTHSSTHTWWWWW",
    "THSTWWWWWW",
    "SHSSSWW",
    "SSHSTWWWW",
    "THTHTHSSSWWWWW",
    "HTHWWWW",
    "HTHWWW",
    "SSSHSSTWWWW",
    "HSHSSSHSSWWWWWW",
    "HTHWW",
    "THTHSSSHSSSTWWWWWWW",
    "SHSSTWWW",
    "THTHSSSHW",
    "SSSTHSSSTWWWWWWW",
    "SSSTHSSSTWWWWW",
    "SSSWWWW",
    "THSTHTHSW",
    "SSSHWWWWWWW",
    "THTHSSSTHSTWWW",
    "THTHSTHSTWWWWWWW",
    "STHTHSTHSS",
    "THTHSSSHST",
    "HTHSHSSTWWW",
    "STHSTHTHW",
    "SSSTHSSSTWWW",
    "SSSTHSSST",
    "THSTHSSSTHSSSWWWWWWW",
    "THWWWWWWW",
    "HTHSHSWWWW",
    "STHSTW",
    "THTHTHSSSTWWW",
    "STHSTHSSSTHSWWWW",
    "HSTHSTHSSWWW",
    "THSTHSTHSSWWWWWW",
    "SSSTHSSSWWWWWW",
    "SHSTHSSTHSTWWWWWW",
    "THTHTHSSSTW",
    "SSSTHSSSWWWWW",
    "HSTHSSSHSWWWWW",
    "SHTHSTHSSWW",
    "THTHSSTHSSS",
    "SSSTHSSSWWWW",
    "HSSST",
    "THTHTHSTWWWWWWW",
    "SSSTHSSSWWW",
    "THTHSSTH",
    "SSSTHSSSWW",
    "SSSTHSSTWWWWWW",
    "STHSSSTW",
    "SHTHSSTHTWWWWWWW",
    "SSSTHSSTWWWWW",
    "THTHSSSTHTWWW",
    "SSSTHSSTWWWW",
    "HSTHSSSTHSSSTWWWW",
    "THSHTHSTWWWWWW",
    "STHSSSW",
    "THS",
    "STHSSSTWWWWWWW",
    "THSTHSSSHSSSTWWWWW",
    "SSSTHSSWWW",
    "STHSTHSTHWWWWWWW",
    "SSSTHSSWW",
    "HSTHSSSTHW",
    "HTHSTHST",
    "SSSTHSSW",
    "HTHSTHSSSWWWWW",
    "HSHTHSTWWWWW",
    "SSSTHSTWWWWWW",
    "HTH",
    "THTHTHSSTWWWWWWW",
    "HTHSSTHSSS",
    "HSHSTHSSSWWWWW",
    "STHSTWW",
    "STHSWW",
    "SSHS",
    "HSTHSSSTHSSWW",
    "STHTWWWWWWW",
    "HTHTHSSSTWWWW",
    "HTHSWWW",
    "THSTWW",
    "STHSTHSSTHSST",
    "STHTHSTHSTWWWWWW",
    "SHSTHSTHSSSTWWW",
    "HTHSTHSSSW",
    "HTHTHTW",
    "HWWWWWWW",
    "HSS",
    "SHTHSTHSSSWWWWWWW",
    "HSHSSSHSSSWWWW",
    "THSTHSSTHWWWWWW",
    "HTHTHTWW",
    "SHTHSSTHSSSTWW",
    "THTHSTHSSSTW",
    "HSWWW",
    "THTHWWWWW",
    "STHWW",
    "SHSSSTWWWWW",
    "HSHSTHWWWWW",
    "HSTHTHWWWWWW",
    "HSTHSSSTHSSTWWWWWWW",
    "THTHSSSTHSSSWWWWWW",
    "HTHW",
    "SSHST",
    "SHTHSSSTHSSSWWWWWWW",
    "STHSTWWW",
    "HSHSTHSSSTWWW",
    "HTHSTHSSWW",
    "THTHT",
    "SHSSSTWWWW",
    "THSTHSSSHTWWWWW",
    "HTHSSSTHSSSWWWWWW",
    "HSTHTHSS",
    "STHTHTHST",
    "HTHSHSSSTW",
    "THSTHSTHS",
    "SSSTHSSWWWWW",
    "HTHTHSTWWW",
    "STHTHSSSTHSSST",
    "THSHTHSSSTWWWW",
    "THSHSTHSSSTWWWWWW",
    "SSSHSSTWW",
    "SSTWWWWWWW",
    "THTHSTHTW",
    "HTHSSSTHSSSWWWW",
    "SHSSSWWWWWWW",
    "STHSTHTHTWWWWWW",
    "HSHSTHTWWWW",
    "STHSTWWWWWWW",
    "SHSSS",
    "THSSSTW",
    "HTHSSSTHSSTWWW",
    "SHSSW",
    "SHSSTWWWW",
    "SHTHSSTHSSSTWWWWW",
    "HSSSTWW",
    "HTWWWW",
    "THSHSTHTWW",
    "SHTHSSSTHSTWW",
    "SHSSTWWWWWWW",
    "HTHSSTHWWW",
    "THTHSTHWWWW",
    "STHSSSTWWWWWW",
    "STHSTHSSTHWWWWWW",
    "STWWW",
    "SHSSSWWWWW",
    "STHTHTHSSWWW",
    "SHSSWWWWWWW",
    "SHSSWWWWWW",
    "HSHSSTH",
    "THSHSTHSSSTWWWWWWW",
    "SHSSWW",
    "STHSTHTHWWWW",
    "HTHTHWWWW",
    "THTHSSTHSWWWW",
    "THSHSSTHSSSW",
    "SHSST",
    "HTHSSSTHSST",
    "HTHSSSHTWWWW",
    "HSTHSTHSWWWWW",
    "THTHSHSSSWWWWWW",
    "HTHSSSW",
    "HTHSSSHSSSTWWWWWWW",
    "THTHSSTHSSSTW",
    "HTHWWWWWWW",
    "HTHSSSHSSSWWWW",
    "SHSSSTWWW",
    "HSHSSTHSSTWWWWW",
    "HTHSTHSSS",
    "THSSW",
    "HTHSSTHSTWWW",
    "SSHWWW",
    "STHTHSSSTHSWWW",
    "THTHSSTHSTWWWWWW",
    "SSHTW",
    "SHSWW",
    "HSTHSSSTHSSTWWW",
    "SSW",
    "HSSSWWW",
    "SHSSST",
    "HSTHSSSTHSSWWWW",
    "SHTWWWWWWW",
    "THSTHSSTHSTWWWW",
    "HSHSSTHSSSWWW",
    "THSHSTHTWWWWW",
    "SSHSTWWWWW",
    "THTHSSSTWWW",
    "HTHSSTHSSSWWW",
    "HTHTHW",
    "THTHSSTHWWW",
    "SHTWWWWWW",
    "HTHSTWWWWWWW",
    "HSHSSTHSTWWWWW",
    "HWWWWW",
    "SSSHSSST",
    "SHTWWWWW",
    "SHTWW",
    "SHTW",
    "STHSTHTHSSSTW",
    "SSSTHSSSTWWWWWW",
    "SHSTHSSSTHSSSTWWWW",
    "HSHSSSHSTW",
    "SHWWWWWWW",
    "SSHSSTWWW",
    "HSTHSSSHSWWWWWWW",
    "SHTWWW",
    "THTHSSSTHSWWW",
    "HST",
    "SSHSS",
    "THTHSSTHSSSWWWWWWW",
    "SSTHSSSTWWW",
    "HTHSSSHTWWWWWW",
    "SHWWWWW",
    "SWWW",
    "HTHTHSW",
    "SHSSSTWWWWWW",
    "HSHSTHW",
    "HSHSTHSSST",
    "STHSTHSTHSSSTW",
    "THTHTHSSWWWWWWW",
    "THSHSSSTHSSSW",
    "STHTWWWWWW",
    "SHTHSTHSSSTWWWWWW",
    "HTHSSSWWW",
    "SSHSSSWWW",
    "SSTHTW",
    "SSSHSSTWWW",
    "SHWWWWWW",
    "SWW",
    "HSHSSSHSST",
    "SHSSTWWWWWW",
    "SHSTHSSSTHSSWWWWWWW",
    "HSTHTHSSSTWWWW",
    "THSTW",
    "SW",
    "SHST",
    "S",
    "HSHTHSSWWW",
    "HTHSHSTWW",
    "THSSSTWWWWWWW",
    "HTHTHTWWW",
    "STHTWWWWW",
    "HTHTHSWWWWW",
    "HTHTHSSST",
    "THSSTWW",
    "THSSSWWWWWWW",
    "HTHSTHSWWWW",
    "THSSSWWWWWW",
    "SSTWW",
    "THSSSWWWW",
    "STHSS",
    "SHSTHSSSTH",
    "HSHTHTWWW",
    "THSTHSSSTHSSWWWWW",
    "SSTHSSWWWW",
    "THSSSWWW",
    "THSSS",
    "SSSHSWW",
    "HSSTWWWWWW",
    "THTHSSTHTWWWWWWW",
    "SSSTWWWWW",
    "SSTHSSST",
    "STHTHSTHSSWWWWWWW",
    "SHTHTHSSW",
    "HSSTW",
    "HTHSHSSST",
    "STHTHSSSTHSTWW",
    "SHSSWWWW",
    "THTHSSSTHSSTWWWWWWW",
    "SSSHSSSWW",
    "THSHSTHSSSWWWW",
    "THSTHSSSTH",
    "HSSWW",
    "THSHSSTHSS",
    "SSHTWW",
    "HTHSSSTHSSSTWWWWWWW",
    "HSTHTHSSSWWW",
    "THTHSSSTHTW",
    "THSHTHSSSWWWWW",
    "HTHSSTHSTWWWWW",
    "TWWWWWW",
    "STHTHSSSTHSSWWWWW",
    "SSSHST",
    "HSSTWWWW",
    "SHSTWWWW",
    "HSHSSSTHST",
    "HSTHSSSHTWWWWWWW",
    "HTHSSSHSSTW",
    "HSWWWWWWW",
    "THTHSSSHSSTWWW",
    "HSTHSSSTHSSS",
    "SSSHSSTW",
    "HSTHSSSHSSSWWWWWWW",
    "HTHTHSSSTWWW",
    "THST",
    "SSSHSWWWW",
    "THTHSSSHSSTW",
    "SSTHSST",
    "THTHSSSTHSTWW",
    "THSWWWWWW",
    "HSHSTHSSTWWW",
    "HSSTWW",
    "HSTHSSSHSSWWWWWW",
    "SSTHWWW",
    "THSTHSSTHWWWW",
    "HSTHSSSTHSSSWWW",
    "STHSTHSSSTHTW",
    "SHSSTW",
    "SHSTWWWWW",
    "THTHSSSTHSSSWWWW",
    "THWWW",
    "STHWWWW",
    "HTHSSSTHSTWWW",
    "SSSTWWWW",
    "HTHTHSSTWWWW",
    "SHTHSSTHTWWWWWW",
    "THTHTHSSTWWW",
    "SSTHSWWWWWWW",
    "HTHTHSSSWWWWWWW",
    "STHSWWW",
    "HTHTHSSSW",
    "SSSHSST",
    "STHTHSSTHS",
    "THTHSSSTHTWW",
    "HTHSTW",
    "SHTHSSSTHSSSTWWWW",
    "SSSHTWWWWW",
    "HTHSTHSTWWWWWWW",
    "HTHSTHWWWWWWW",
    "STHWWWWWWW",
    "SHTHSTHSSTWW",
    "SSHT",
    "HTHSSTHSSSWW",
    "HTHTHSSWW",
    "HSHTHSSSWWWWWW",
    "HTHSWWWWWW",
    "HTHSSTHSWWW",
    "HSTHSTHSTWWWW",
    "STHTHSTHSSSWWW",
    "SHSTHSSSTHSSW",
    "HTHSTHSTWWWWW",
    "STHSSWWWWWW",
    "STHSTHSSTHTWW",
    "HTHSSSTHSSWWW",
    "HTHTHSSTWW",
    "HSHSSSTHWWWWW",
    "HTHSTHW",
    "THSHSTHSW",
    "HSSW",
    "THTHSSSHTWWW",
    "STHSTHSSSTHSSS",
    "HSSSTWWWWWWW",
    "THSHSSTHSSS",
    "HSHTHSSSWWWWWWW",
    "HSTHTHSSSTWWWWWWW",
    "HTHSSSTHSTWWWWWW",
    "STHSTHSTHSTWW",
    "STWWWWW",
    "THW",
    "HTHSTHSWWWWWW",
    "HTHSSTHW",
    "HSWWWWW",
    "STHSWWWWWW",
    "STHTWWWW",
    "HTHSSTHSWWWWWW",
    "STHSSSWWW",
    "HTHTHTWWWWWWW",
    "HSHSTHSSSWWWWWWW",
    "SHTHSSTHSTWWWWWWW",
    "HTHSSSTHSTWWWW",
    "HT",
    "STHSTHSSSTHWWWW",
    "HTHSSSHSSWW",
    "SSSTHSSTWW",
    "THSSWWWWW",
    "SHTHTHSSWWWWWW",
    "THSTHSSSTHSSWWW",
    "THTHSSTHSSSWWWWW",
    "HWWWW",
    "STHSTHTHTWWWWW",
    "SHSSSTWWWWWWW",
    "HSTWWWWWWW",
    "HSST",
    "HSTHSSTHWWWW",
    "STHTHSSSTHSTW",
    "IWWWWWWW",
    "SSHSSSWWWW",
    "HSHSSSTHSW",
    "IWWWWWW",
    "SHSTHTHWWW",
    "THSTHSSSTHSTWWWW",
    "STHSSTWWW",
    "HTW",
    "SSSHSTWWWWWWW",
    "HTHSSTHSS",
    "STHSTHSTHSSWWWW",
    "SSSHSWWWWWWW",
    "THSTHSSSTHSSSWWWW",
    "SSTHSWW",
    "SSSTHSSTW",
    "SSTHSSSTWWWWWWW",
    "STHSTHSTHSSSWWW",
    "STHTHSSTHSSTWWWWWWW",
    "HSTHSTHSWWWW",
    "HSTHSSTHSSW",
    "STHTHSSTHSSSTW",
    "THSHSSSTHSSSWWW",
    "STHW",
    "SSHSTWW",
    "SHSTW",
    "STHTHTHSWWWWWW",
    "SHSTHTHW",
    "THSST",
    "THSWWWWWWW",
    "HSTHTHSSSTWWWWWW",
    "SHSTHSSSTHSW",
    "HSTHTHW",
    "HTHTHSSTWWW",
    "HSTHSSTHSSTWWWWWWW",
    "STHSTHTHSTWWW",
    "THSTHTHSSTWWWWWWW",
    "HTHSTHSSTWW",
    "SSTHSSS",
    "HSHSTHSSW",
    "STHTHTHSSTWW",
    "THSHSTHTWWWWWWW",
    "SHSSSTWW",
    "SSHSSST",
    "HTHSSSTHSSWWWW",
    "HTHSSSTHTW",
    "I",
    "SSHSSTWWWWWWW",
    "SHSTHSSTHSSSTWWWWWW",
    "THTHSSTHSTWWWW",
    "HWWW",
    "THTHSSWWWW",
    "STHSWWWWW",
    "STHSTHTHSSWWWW",
    "THTHSSSHSTWW",
    "HSHSTHTWWWWW",
    "HSHSTHTW",
    "HTHWWWWWW",
    "HSHSSSTHWWWW",
    "SSHSSWWWWWW",
    "SSSHT",
    "SSSHSSSWWWWWWW",
    "SSTHSSSW",
    "HSTHTHTWWWWW",
    "HSTHSSSHSSTWW",
    "THTHTHSSSWWWWWW",
    "THSSSWWWWW",
    "HSHSSTHTWWW",
    "THTHSSTHSSSWWWW",
    "SHSTHTHSTWWWWW",
    "HTHSHSTWWWWWW",
    "SSTWWWWWW",
    "HSHTHSTWWWWWWW",
    "HSTHTHSSSWWWWWW",
    "SHSTHSSTHSTW",
    "SHSTHTHTW",
    "THSTHSSSTHTWWWWWWW",
    "SSTWWWWW",
    "SHSWWWW",
    "HTHTWWWW",
    "SHSTHSSSTHSSWWWW",
    "SSSTHSSWWWWWWW",
    "THSHSTHTWWWW",
    "HSHSSTHTWWWWWW",
    "SSSTHT",
    "SHSTHSSTHSSWWWWWWW",
    "HSSWWWWW",
    "SSSHWWWWWW",
    "THSHSSTHSSST",
    "HTHSTHSSSTW",
    "SHSTHSTHWWW",
    "STHSW",
    "HTHSHSSSW",
    "SHSTWWW",
    "THTHSSTHSSWW",
    "THSSWWWWWW",
    "THTHTHSSS",
    "HSTWWWWWW",
    "HTHSSSTHSW",
    "THTHSSWWWWW",
    "HTWW",
    "THTHSHSSTWW",
    "HTHSHTWW",
    "HSHSSSHWWWWWWW",
    "HSTHSSTHSSWWW",
    "THTHSHSS",
    "SHSWWWWWWW",
    "THSTHSSTHSSWWWWW",
    "THTHSSTWWWWWWW",
    "SHSWWW",
    "HSHSTHSSSWWWW",
    "HTHSSSTHSSSTW",
    "SSSTHWWWWW",
    "STHSTHSTHSTW",
    "SSTHSTWWW",
    "THSSTWWWWWWW",
    "THTHSSSTHSSTWW",
    "THSSTWWWW",
    "STW",
    "HSTHSSTHS",
    "STHTHSSTHSSWWW",
    "SSHSSSTWW",
    "SHTHSTH",
    "HTHSSTH",
    "THTHSSSTHSWW",
    "HTHSSSHSTWWWWWW",
    "HTHSSSTHTWWWW",
    "STHSTHTHSWWWW",
    "HTWWWWWW",
    "STHTHSSSTHSSSWWWW",
    "HTHSHSSWWW",
    "STHSTHSTHSSTW",
    "HSTHSSSTHSSTWWWWW",
    "SSWWWW",
    "HTHSSSHSW",
    "TWWWWWWW",
    "STHWWWWWW",
    "SSHWWWWW",
    "STHSTHTHSSSWWWWWWW",
    "HSTHSSSTHSSSWWWWW",
    "IW",
    "SHTHSSSTHS",
    "SSWWW",
    "HSWW",
    "HSTWWWWW",
    "HSSSWWWWWW",
    "THSTHSSSHSSWWWWWWW",
    "HSSTWWW",
    "HS",
    "SHSTHTHSSTWWWWW",
    "HSSSTWWWWW",
    "HTHSSSTHSSSW",
    "SSHSWWWW",
    "SSHTWWWWWW",
    "SHSTHSTHSSSTW",
    "THTWWWWWW",
    "HTHSSSTHTWWWWWWW",
    "THTHTHSSSTWWWW",
    "SHTHSTHTWWWWWW",
    "SSHSSSTWWWWWW",
    "SSTHSSTWWWWWW",
    "HSHTHSWWWWWW",
    "STH",
    "THTHTWWW",
    "HSSSTWWW",
    "SHSSWWW",
    "SHS",
    "HTWWWWW",
    "STHSTHTHSSTWWWWWW",
    "THTHSHSWW",
    "THSTWWWW",
    "STHSTHTHSTWWWWWWW",
    "SHTHSSTHSSSTWWWWWW",
    "STHSSWWWWWWW",
    "HSSTWWWWWWW",
    "SHSSWWWWW",
    "HTHSSSHSSWWWW",
    "SHTHSSSTHSSWWWW",
    "THTHSSSTWWWWW",
    "THWWWW",
    "HSSSTWWWWWW",
    "STHWWWWW",
    "STHTHTHSSS",
    "SSHSSS",
    "HSTWWWW",
    "THTHSTHSTWWWWW",
    "HTHSHT",
    "HSTWW",
    "SHTHTHSWWWWW",
    "STHSSSWWWWWW",
    "HSTHTHWWWWW",
    "TH",
    "HTHSTHSSWWW",
    "THTHSHSSWW",
    "THWWWWWW",
    "SHSTHSTHSSWWWWWW",
    "SHSTHTHSSTW",
    "THSHSSSTHSSWWWW",
    "SHTHTHTWWWWWW",
    "SSSHSTWWW",
    "HSTHSSSHSW",
    "STHSTHSTHST",
    "THSWW",
    "THTWW",
    "HSHSSSHTWW",
    "THSTHSSTHSSSWW",
    "HSHSSSHSSWW",
    "HSHTHSWW",
    "THTHSHSSSW",
    "HTHSTHSSSTWWWWWW",
    "SSTHSSSWWWWW",
    "HTHSSSTHST",
    "HTHSSSTHWWWW",
    "THTWWW",
    "SHSSSWWW",
    "HTHSTHTWWWWW",
    "HTHSTHWWW",
    "THTHSSTHSSSWWWWWW",
    "THSTHSSSHSST",
    "TWWWWW",
    "STHTHSSSTHTWWWW",
    "THSTHSSTHSWWW",
    "THSHSSSTHSSTWWWW",
    "HTHSTHSSSWWWWWWW",
    "SSTHSWWW",
    "HSTHSTHWWWWW",
    "SSSTHSSSTWW",
    "THSW",
    "SHSS",
    "THSHTHSST",
    "HTHTHSSSWWWWWW",
    "SHTHTHSSTWWWWWWW",
    "THSS",
    "SHT",
    "HTHSSTHTWWWWWW",
    "HTHSSTHSSSWWWWWWW",
    "THSSSTWWWWWW",
    "THSTHSSTHTWWWWW",
    "HTHSTHTWWW",
    "SSS",
    "STHTHSSTHSSSTWW",
    "SSTHS",
    "HTHSSSTHSSSTWWWWWW",
    "HSTHTHTWWWWWW",
    "STHTHSTHSSTWWWWWW",
    "THSTHSSTHSSSTW",
    "SSSTHSTWWWWWWW",
    "SSTHTWWWW",
    "HSTHSSSTHSS",
    "SHTHSTHSWWWWWWW",
    "HTHSSSHSSSTWWWWWW",
    "STHTHSSTHSSTWWWWW",
    "HTHSSSTHSS",
    "STHTHSSTHSTWWWW",
    "SSSHSSSTWWWW",
    "THSTWWWWWWW",
    "SSST",
    "SSHSSWWWW",
    "THTWWWWW",
    "THSHSSTHTWWWWW",
    "SHSSSWWWWWW",
    "THSSWWWW",
    "THSSWWW",
    "HTHTHSSS",
    "HSHSSTHSSTW",
    "THSSTW",
    "THSHSSSTHS",
    "HSTHSTHWW",
    "SHSTWW",
    "THTHSTHSSTWWWWW",
    "STHSTHSSTHSSSTWWW",
    "THSSTWWW",
    "THSSSTWWWW",
    "STHTHSSTHWWWWW",
    "HTHTHSSSWWWWW",
    "SHTHSTHSSSTW",
    "THTHTHTWWWWWWW",
    "THSHSSTHWWW",
    "HSSSW",
    "THTHSTH",
    "HSSSTWWWW",
    "HTHSHSST",
    "HTHSSTHSSSTWWWW",
    "SSTHSSSWWWWWWW",
    "STHTHSSSTHSSW",
    "HSHSSTHSTWWWWWW",
    "STHTHSSSTHSWW",
    "THSTHSSTHSSTWWWWW",
    "STHSSWWW",
    "THSTHTHSS",
    "HSTHSSSTHSSSTWW",
    "HTHSSSTHTWWWWWW",
    "HSHSTHSSWWWWWWW",
    "THSHTHSSSTWWWWWWW",
    "HTHSTHS",
    "HSHSSTHWWW",
    "HSTHSSSTHSSST",
    "HSHTHSWWWW",
    "HSTHTHTWWWW",
    "THTHSSSTHSSWWWWW",
    "THSTHSSSTHSSWWWWWW",
    "HTHSSTWWWWWW",
    "THT",
    "SSSTHW",
    "HTHSTHSSSWWWWWW",
    "STHSSTW",
    "HSTHTHSSTWWWWWW",
    "THTHTHSWWWWWWW",
    "SSHSSWWWWW",
    "STHSTHSSTHSSSWWWWW",
    "STHSSTWW",
    "HSTHSSSHSTW",
    "SSTHSSWW",
    "SSSTHTWWWWWWW",
    "SSSHSSSWWWW",
    "THTHSHWWWWWWW",
    "HSSWWWWWWW",
    "HTHSTHTWWWW",
    "THTHSHSSWWW",
    "THTHWWWWWWW",
    "HSHSSSTHTWW",
    "STHSSTWWWWW",
    "SHTHSTHSTWWWWWW",
    "STWWWWWWW",
    "HSTWWW",
    "STHSSSWWWW",
    "THTHSHSTW",
    "STHSSSWWWWW",
    "STHSSSWWWWWWW",
    "STHSSSTWWW",
    "SSSHTWW",
    "SSTHSSSTWW",
    "THSTHTHSWWWWWW",
    "HSTHSSSHSSWWWWW",
    "THTHSHSTWWW",
    "HTHSSTHSSSTWWWWW",
    "THTHSHSSTWWW",
    "THSTHSTHSTWWWW",
    "THSHSSSTHSSSWWWWWWW",
    "THSWWWWW",
    "THSTHSSSTHWWWWW",
    "THSTHTHSSWWWW",
    "SSHSSWWW",
    "HSHSTHSTWWWWWW",
    "THSHTHSW",
    "HTHSTHWW",
    "HTHSHSSSWWWWWWW",
    "THTHTWWWWW",
    "HTHSHSTWWWWW",
    "SSSHTW",
    "THTHSSTHSSSTWWWW",
    "HTHSHWWW",
    "HSTHSTHSSSTWWWWWW",
    "HSTHSSSTHS",
    "SSWW",
    "SSTHWWWWWWW",
    "HSHTHSTWW",
    "SHSTHSSSTHSSTWWW",
    "HTHSSTHSSSWWWWWW",
    "SHTHSTHSWW",
    "SSSTHWWWWWW",
    "HTHSSSTHSSW",
    "SSTHSSWWW",
    "HTHTHSSSTWWWWWW",
    "HSTHSTHSSSW",
    "HTHSTHSSSWW",
    "HTHSTHSW",
    "STHSTHSSSTHSSSTWWWWWWW",
    "STHTHSTHSSWWWW",
    "SHTHSSSTHSSTWWWWWW",
    "HTHSHSSSTWWWWWW",
    "SSH",
    "SHSTHSSSTHSSTWWWW",
    "THSHSTHSWWW",
    "SSHW",
    "HTHTHSSTWWWWW",
    "THTHSHSTWWWW",
    "SHSTHSSSTHSSWW",
    "THSHSTHSWWWWWWW",
    "HSSSWWWW",
    "SSSTHSTWWW",
    "THTHTHSWWWWWW",
    "THSHSTHSWWWWWW",
    "SSHWWWWWW",
    "STHSTHSSTHSSTW",
    "SHTHTHSSWWWW",
    "HSTHSSSTHTWWWWWWW",
    "THTHSHWWW",
    "HTHTHSSWWWWWW",
    "HSSSWW",
    "STHSTHSSSTHSSSTWWW",
    "STHSTHTHTWWWW",
    "SSHTWWW",
    "HTHSHSWWWWW",
    "THTHSSSTHSSSWW",
    "HSTHSSSTHSSSTWWWWWWW",
    "SSSTHSSSTW",
    "THSTHSSTHSSW",
    "SSSTHSSS",
    "SSHSWW",
    "THTHSSTHSS",
    "HSTHTHSSSWWWWW",
    "SSSTHSS",
    "SSHSWWWWW",
    "HTHSTHSTWWWW",
    "STHTWWW",
    "HTHSSSWWWWWW",
    "HSTHTHTWWWWWWW",
    "SSSHSSSTWWWWW",
    "SSSHSSSTWWWWWWW",
    "SHSTHTHSTWWWWWW",
    "HTHSHSSSWWWWWW",
    "STHTHSTHWWWWWW",
    "THSTHSSSTHSSSW",
    "HTHSSSTHSWWWW",
    "HTHSSSHSSTWW",
    "HSTHSSTHSWWWWW",
    "THSHSSSTHTWWWWWW",
    "SSHWW",
    "SSTHSSTWW",
    "HSHSSSHSSTWWWWWW",
    "TWW",
    "SSHSSW",
    "SHTHSSSTHSSWWW",
    "SSSHSTWWWWWW",
    "SHTHTHSSSWWWWWWW",
    "SSHSSWW",
    "STHSTHSTHSSWWWWWW",
    "THTHSSTHSSSTWWW",
    "SSSHSSSW",
    "THTHSSSTHSWWWW",
    "SSHSSTW",
    "HTHSTHSSST",
    "HWW",
    "HSTHTHSSTW",
    "STHSTHTHSTW",
    "SSHSSTWW",
    "HSHSSTHWW",
    "SSHSTWWWWWWW",
    "SSTHTWWWWW",
    "THTHSSSHSSSTW",
    "THSHSSTHSSWWW",
    "SSSTHSWWWWWWW",
    "HSHSTHWWWWWW",
    "STHTHSSSTHSTWWWWWWW",
    "THTHSSSHWWWWW",
    "SH",
    "HTHSSSHWWW",
    "SSHSSTWWWWWW",
    "THSTHSSSTHSSWW",
    "THSHSSSTHSSWWW",
    "SSHSSTWWWWW",
    "HSHSSSTHTWWWW",
    "THSHSSTHSSSTWWWWWW",
    "STHSTHSSSTHSSWWWWWWW",
    "SSHWWWW",
    "STHSSWWWW",
    "SSSTHST",
    "THSTHSTHSTWWWWWWW",
    "SSHSSSTWWW",
    "HSHSTHTWWWWWWW",
    "STHTHSSTHTW",
    "SSHSSSTWWWW",
    "HSTHSTHSS",
    "SSSTHTWWWWWW",
    "SHSSSTW",
    "HTHSSTHSSTWWWWWWW",
    "SSSTHSWWWW",
    "HSHTHWWWWW",
    "HSHTHSSWWWWWWW",
    "STWWWW",
    "HSSWWW",
    "SSSTHWW",
    "SHSTHSSTHSSSWWWWW",
    "THTHSSSTHSSWWWWWW",
    "SSTHWW",
    "THSTHSSTHSSWWWW",
    "STHSSTWWWWWW",
    "STHSTHTHSSSTWW",
    "SSTHSWWWWWW",
    "SSTHWWWWWW",
    "THTHSTHSSST",
    "SSSTHSTWWWWW",
    "SSSHSSWWW",
    "THSHTHSSTWWWW",
    "THSHSSSTHSTWWW",
    "STHSTHSSSTH",
    "THTHSSSTHSSS",
    "HSHTHSSWWWWWW",
    "SHTHTHSSWWWWWWW",
    "THSTHTHSSTWWWWW",
    "SSTHSTWWWWW",
    "SSTHTWW",
    "SSTHTWWWWWWW",
    "HTHSHSSSTWWWW",
    "THSHTHSSW",
    "SSTHSWWWW",
    "THSSSTWWW",
    "HSTHTHSTW",
    "THSTHTHSSTWWWW",
    "THTHTHSSWWWW",
    "THTHSS",
    "SSSTHSTW",
    "THTHSST",
    "THSHSTHWWWW",
    "THTHSSWWWWWWW",
    "SSTHSTWWWWWW",
    "SSTHSTWWWW",
    "SSSHSS",
    "HSHSSTHWWWWWW",
    "SSTHSTWWWWWWW",
    "THSHSSSTHWWWWWWW",
    "SSTHSSW",
    "SSTHSSTW",
    "SSTHSSTWWWW",
    "THSTHSSSTHSSTWWWW",
    "THSHSSTHSSSTWWW",
    "HTHTHSWWW",
    "HTHSSSTHSWW",
    "THSHSSTHSSWWWW",
    "HTHSTHTWW",
    "SSSW",
    "HSHSSSTHSSTWWW",
    "SHTHSSTHSSS",
    "HTHSSSTHSSSWWW",
    "SSTHSSSWW",
    "HSHSSSHSSST",
    "ST",
    "SSTHSSSWWW",
    "HSHSSSTHSSSWWWWWW",
    "HW",
    "HSTHTHWWWW",
    "THTHTHSSST",
    "THSHTHSWW",
    "SSTHST",
    "STHSTHSSSTHSSSWWWWWW",
    "THSHTHSSST",
    "THTHSSSHSSW",
    "SSTHSSSWWWWWW",
    "THSHSSSTHT",
    "SSSTHSSWWWWWW",
    "SSHSSSW",
    "SSTHSSSTW",
    "THTHSSTHWWWWWW",
    "STHTHSSSTHSWWWWWWW",
    "THSHSTHWW",
    "SSTH",
    "HTHSSSHSS",
    "SSTHTWWWWWW",
    "SSSHSTWW",
    "STHTHSSSTHSST",
    "SSTHSSSTWWWWW",
    "HSHSSSTH",
    "SHTHTHSSSTWWWWWW",
    "SSSWWW",
    "HSHSSSHTWWWWWW",
    "THSHSSSTHTWWW",
    "SSSWWWWW",
    "THSTHSSTHTW",
    "THSHSSSTHSSSWW",
    "THTHSHWWWW",
    "HTHSSTHSSSTWWW",
    "SHTHSTHSSWWWWWWW",
    "SSSTHSW",
    "SSSWWWWWW",
    "HTHSSSHWWWW",
    "THSSTWWWWW",
    "HTHSSSHSSTWWWWWWW",
    "THSHSSSTHSTW",
    "SSHSTWWW",
    "SSSHW",
    "HTHSSSHSSTWWW",
    "HSTHSSTHSSSTW",
    "STHTHSTHSSS",
    "SSTHW",
    "SSSHTWWW",
    "THTHSTHSWWWWWW",
    "SHTHSTHSSS",
    "THSHSTHST",
    "STHSTHTHSSTWW",
    "SHTHTHS",
    "HSSWWWW",
    "SSWWWWWW",
    "SSSHS",
    "THTHSHSST",
    "HTHSSTHSSSTW",
    "SSTHSSWWWWWW",
    "SSSHSWWWWW",
    "SSHSSSWW",
    "SSHSTW",
    "HTHSSTHSSWWWWWWW",
    "THTHWWWWWW",
    "SSSHSTWWWWW",
    "HTHSSTHSTWWWWWWW",
    "HSTHSSSHTWWWW",
    "THSTWWW",
    "SHSTHSSSTHSSST",
    "SSSHSSS",
    "SSSTHWWW",
    "SSTHSSSTWWWWWW",
    "SSHSSTWWWW",
    "SSSHSSSTWWWWWW",
    "HSTHSTHSSSWWWWWWW",
    "HSTHSSSTHSSSTW",
    "SHSTHSSSTHSTW",
    "SSSHWWW",
    "SSSHSSWWWWW",
    "SSSTWW",
    "SSSTHSSSW",
    "SSHSWWWWWWW",
    "STHSTHSSSTHTWWW",
    "THTHSTHTWW",
    "HSHSSTHTWW",
    "THTHSSWWWWWW",
    "SSSHSSTWWWWW",
    "THTHSSSTHSW",
    "SSSHSSTWWWWWWW",
    "THSTHSTHST",
    "HSTHTHSSSTWW",
    "THSHTHWWWWW",
    "STHSTWWWWWW",
    "IWWWWW",
    "THSTHSSSHSSSTWWWWWW",
    "SSSHSSSWWWWW",
    "THSHSTHSSTWWW",
    "STHSTWWWW",
    "STHTHSSTHST",
    "SSTW",
    "SHTHSSTHTWWW",
    "HSHSTHSSTW",
    "SSSTHSWWWWWW",
    "THSHSSSTHSST",
    "STHTHSSTHTWWWW",
    "SHSTHSSTHWWWWWWW",
    "SSSHSSSTW",
    "HSHSSSTHSST",
    "HSTHSSSHTWWWWW",
    "THSHSSTHS",
    "SSSHSSSTWWW",
    "THSHTHSTWWWW",
    "SSSHSSWWWW",
    "THSTHSSSTHSTWWW",
    "HSHSSSTHSTWWWWWW",
    "SSHSSWWWWWWW",
    "SSTHWWWW",
    "SSSTH",
    "SSSTW",
    "SSSTWWW",
    "HSHTHSSTWWWW",
    "HTHSTHSST",
    "STHSTHSSTHSSSWWWWWWW",
    "SSSTHWWWWWWW",
    "THSTHSSSTHSSTW",
    "SSSTWWWWWWW",
    "THTHSHSWWWWWW",
    "HTHTHT",
    "SSSTHTW",
    "HTHSSSTHSTWWWWW",
    "HTHSTHT",
    "HSHSSTHSSWWW",
    "SSSTHTWW",
    "SSSTHTWWW",
    "THSTHSTHSSSWWW",
    "THSHSSSTHSSSTW",
    "HSTHSSTHTWWW",
    "HSTHTHSSSWWWW",
    "HSHTHSTWWWW",
    "SSSTHTWWWWW",
    "STHTHSSTHSWWW",
    "HSHSTHSW",
    "HSHSTHWWWW",
    "THTHSSTHSTWWWWW",
    "HTWWWWWWW",
    "HTHSTHSSSTWWWW",
    "SHTHSTHSTWWW",
    "HSHSSSTHT",
    "HSTHTHTW",
    "THSTHTHTWWWWWWW",
    "SSSTHSWWW",
    "HSTHTHSSTWWWWW",
    "HSTHSSSTHSSWWW",
    "HSHSTHSWWWWWW",
    "HSHSTHSWWWWWWW",
    "HSHSTHST",
    "THSSWW",
    "HSHSSTHSWW",
    "STHTHSSSTHST",
    "HSHSTHSTW",
    "HSHSTHSSSWWW",
    "HSHSTHSTWW",
    "THTHSTHWWWWW",
    "HSTHTHSSWWW",
    "HSHSTHSTWWW",
    "HSTHSTHSSSWWWWWW",
    "HSHSTHSTWWWW",
    "SST",
    "HSHSTHSTWWWWW",
    "HSHSTHSS",
    "THTHTHSSWWW",
    "HSHSTHSSWW",
    "SHSTHTHWWWWWWW",
    "HSHSSSHSTWWWWWW",
    "HSHSSSHSWW",
    "THSTHSSTHWWWWWWW",
    "HTHSTHWWWWW",
    "THSHTHSTWWW",
    "HSTHSTHW",
    "STHSTHTHTW",
    "SSHSSSTWWWWWWW",
    "HSHSTHSSWWWW",
    "HSHSSSHWWWWWW",
    "HSHSTHSSWWWWW",
    "HSHSTHSSWWWWWW",
    "HSTHSSTHSSSWWWW",
    "THSTHSSSHSSWWWWW",
    "THTHSHSWWWWWWW",
    "HSHSTHSST",
    "HSTHSSSTHSWWWWWWW",
    "HSHSTHSSTWW",
    "HSHSTHSSTWWWWWW",
    "HSHSTHSSSW",
    "THSTHSSTHSWWWWWW",
    "THTHTHWWWWWW",
    "SWWWW",
    "THTHSSSHSTWWWWWWW",
    "SSSHTWWWWWWW",
    "HSHSTHSSSWWWWWW",
    "HSHTHSS",
    "HSHSTHSSSTW",
    "HSHSTHSSSTWW",
    "HSHSTHSSSTWWWW",
    "HSHSTHSSSTWWWWWW",
    "HSHSTHSSSTWWWWWWW",
    "HSTHTHSST",
    "HSHSSTHW",
    "HSHSSTHWWWW",
    "THSTHTHSTWWW",
    "HSHSSTHWWWWW",
    "STHTHSSSTHTWWW",
    "HSHSSTHWWWWWWW",
    "HSTHSSSHTWWWWWW",
    "THTHTHSSTWWWWWW",
    "THTHSTHSSSWW",
    "HSHSSTHT",
    "THSHSSSTHSSSWWWW",
    "STHTHSTHTWWW",
    "THSTHSTHTWWWWWWW",
    "HSHSSTHTW",
    "HSHSSTHTWWWW",
    "THTHSSSHTWWWW",
    "HSHSSTHTWWWWW",
    "THTHSTHSTWWW",
    "SSSTHSWWWWW",
    "HSHSSTHTWWWWWWW",
    "STHTHSSTHSSSWWW",
    "STHT",
    "THSHSTHSTWWWWWW",
    "HSHSSTHS",
    "HSTHSTHSWWW",
    "HSHSSTHSWWWW",
    "HSHSSTHSWWWWW",
    "HSSWWWWWW",
    "HSHSSTHSWWWWWW",
    "HSHSSTHSWWWWWWW",
    "HSHSSSTHSSSWWWWWWW",
    "THSTHSSSTHSSSTWWW",
    "HTHSTHTWWWWWW",
    "STHWWW",
    "THTHTHSTWWWWWW",
    "HSHSSTHSTWW",
    "HTHSSWWWWWW",
    "THTHSHSSSTWWWW",
    "HSHTHSSW",
    "THTHSSSHSSSWWWWW",
    "HSHSSTHSTWWW",
    "STHTHTHSTWWW",
    "THSHSSSTHSSTWW",
    "HSHSSTHSTWWWWWWW",
    "HSHSSTHSS",
    "HSHSSTHSSSTWWW",
    "HSHSSTHSSW",
    "HSHSSTHSSWW",
    "SHSTHTHSSWWWWWWW",
    "SSSHWWWWW",
    "HSHSSTHSSWWWWW",
    "HSHSSTHSSWWWWWW",
    "HSTHSSSH",
    "THSSSWW",
    "THTHSSST",
    "HSSSWWWWWWW",
    "HSHSSTHSSWWWWWWW",
    "THTHSSTHST",
    "HSHSSTHSSTWW",
    "THSTHSTHSSSTWWWWW",
    "THSHSSTHSTW",
    "SHTHSSSTHTWWWWW",
    "HSHSSTHSSTWWW",
    "HSHSSTHSSTWWWWWW",
    "STHTHSSSTHSSWWWWWWW",
    "SHSTWWWWWWW",
    "HSHSSTHSSTWWWWWWW",
    "STHTHTHSSSTWWWWW",
    "HSTHSSTHWWWWWW",
    "THTHSTWWWWW",
    "THTHSSTHTWWWW",
    "HSHSSTHSSS",
    "HSHSSTHSSSW",
    "STHSTHSSTHSTWWWW",
    "HSHSSSHSTWWW",
    "HSTHSTHWWWWWWW",
    "THSTHSSSTHSSSTWWWWWWW",
    "HSTHSSSHSSWWWWWWW",
    "THSHSTHSSSTWWW",
    "STHSTHSSTHS",
    "HSHSSTHSSSWW",
    "HSHSSTHSSSWWWW",
    "THSHSSSTHTWWWWW",
    "HSHSSTHSSST",
    "STHTHSSSTHSSSW",
    "HTHSSSHSWWWW",
    "HSHSSSTHSWWW",
    "HSHSSTHSSSTWW",
    "HSTHSTHSST",
    "HSHSSTHSSSTWWWWWW",
    "SSSHWWWW",
    "THSHSSTHSST",
    "HSHSSTHSSSTWWWWWWW",
    "HSHSSSH",
    "HSHSSSHWW",
    "SHSTHSSTHSSTWW",
    "HSTHSSSHSTWWW",
    "HSHSSSHWWW",
    "HSHSSSHWWWW",
    "HSHSSSHWWWWW",
    "STHTHTHSSSWWWWWWW",
    "THSHSSSTH",
    "THSTHSSSTHSWWWWWW",
    "HSHSSSHT",
    "HSHSSSHTWWW",
    "THTHSSSHSWW",
    "HSHSSSHTWWWW",
    "HSHSSSHTWWWWW",
    "SHSTHTHSSSTWWWWWW",
    "HSHSSSHTWWWWWWW",
    "HSHSSSHSWWW",
    "HSHSSSHSWWWW",
    "HSHSSSHSWWWWW",
    "THSTHSTHSSWWW",
    "HSTHTHSWWWWW",
    "SHTHSTHSSSWWWW",
    "THSSTWWWWWW",
    "HSHSSSHSWWWWWW",
    "HSHSSSHSWWWWWWW",
    "SHSTHTHSSSWWWW",
    "THTHSSSHSSSWW",
    "HSHSSSHSTWW",
    "THSTHSSTHSSTWWW",
    "THTHSSTHW",
    "THSHSSTHSSSWWW",
    "HTHTHWWWWWW",
    "HSTHSSSTHTWWW",
    "HSHSSSHSTWWWWWWW",
    "THTHSSTHSSTWWWWWW",
    "THTHSTHSSSTWWWW",
    "HSHSSSHSS",
    "HSHSSSHSSW",
    "HSHSSSHSSWWWW",
    "THSHSSTHSSSWWWWWWW",
    "STHTHSTHSSWWWWW",
    "HSHSSSHSSWWWWW",
    "THTHSSSTHWWWWW",
    "HSHSSSHSSWWWWWWW",
    "HTHSSSTHSSTWWWWWW",
    "HSHSSSHSSTWW",
    "THTHSHSSTW",
    "HSTHSTHTWWWWWW",
    "HSHSSSHSSTWWWWWWW",
    "STHSTHSSSTHSSTW",
    "SHSTHTHSSW",
    "HSHSSSHSSS",
    "SHTHSTHSTWW",
    "HSHSTHSWWWWW",
    "THSHTHSS",
    "THSHSSTHTWWW",
    "SWWWWW",
    "HSHSSTHSTWWWW",
    "HSHSSSHSSSW",
    "HSHSSSHSSSWWW",
    "HSHSSSHSSSWWWWW",
    "THTHSHTWWWW",
    "HSHSSSHSSSWWWWWW",
    "IWW",
    "HSHSSSHSSSWWWWWWW",
    "SHTHSSSTHSSSTWWWWWWW",
    "HSHSSSHSSSTW",
    "THSHTHSSSWWW",
    "HSHSSSHSSSTWW",
    "HSHSSSHSSSTWWWW",
    "THSHSTHSSWW",
    "HSHSSSHSSSTWWWWW",
    "THSTHSSSTHSW",
    "HSTHTHSSSTWWWWW",
    "HSHSSSHSSSTWWWWWW",
    "HSHSSSHSSSTWWWWWWW",
    "HSHSSSTHW",
    "THSTHTHSSSW",
    "HSTHSTHSTWWWWWWW",
    "HSHSSSTHWWWWWW",
    "SSTHSSTWWWWWWW",
    "HSHSSSTHTW",
    "HSHSSSTHTWWWWW",
    "THTHSHSSWWWWW",
    "HSHSSSTHTWWWWWW",
    "STHTHSTHSSSWW",
    "HSHSSSTHSSWW",
    "THTHSSSTWWWWWW",
    "HSHSSSTHSWW",
    "HSHSSSTHSWWWW",
    "HSHSSSTHSWWWWW",
    "THTHSSSTHWWWWWWW",
    "HTHSSSTHTWW",
    "HSHSSSTHSWWWWWW",
    "HSTHSSSTHSSSW",
    "THSWWW",
    "HSHSSSTHSTW",
    "THSTHSTHTWWWWW",
    "THTHSSTHWWWWWWW",
    "THTHSSTHSSTWWW",
    "HSHSSSHTW",
    "HSHSSSTHSTWWWW",
    "HSHSSSHSSTWWW",
    "SSSWWWWWWW",
    "THTHSHSSWWWWWW",
    "HSHSSSTHSTWWWWW",
    "HSHSSSTHSTWWWWWWW",
    "THSTHSSTHSSTWWWWWW",
    "SHW",
    "HSHSSSTHSS",
    "HTHSHSSTWW",
    "HSHSSSTHSSW",
    "HSHSSSTHSSWWW",
    "HSHSSSTHSSWWWW",
    "THTHS",
    "THTHSH",
    "THSTHSSSHSSWWW",
    "HSHSSSHSSTWWWWW",
    "HSHSSSTHSSWWWWW",
    "HTHSSSHSSSWW",
    "HSTHSSSTHSWWWWW",
    "HSHSSSTHSSWWWWWW",
    "THTHSSSHSTWWWWW",
    "HSHSSSTHSSTW",
    "HSHSSSTHSSTWW",
    "HSHSSSTHSSTWWWW",
    "HSHSSTHSSSWWWWWWW",
    "HSHSSSTHSSTWWWWW",
    "THSTHSSSHSSW",
    "THSHSSTHT",
    "THTHTHSSWW",
    "HSHSSSTHSSTWWWWWW",
    "THSTHTHSSSWWWWWWW",
    "HSTHSSTHSSSWWWWWW",
    "HSHSSSTHSSTWWWWWWW",
    "HSHSSSTHSSS",
    "SHTHSSSTHSSSTWW",
    "HSHSSSTHSSSW",
    "STHSTHSSSTHSTWW",
    "THSHSSSTHSTWWWWW",
    "HSHSSSTHSSSWWWW",
    "THSHTHWWW",
    "HSHSSSTHSSSWWWWW",
    "THTHTHSSSTWW",
    "SHTHSSSTHSSWWWWW",
    "HSHSSSTHSSST",
    "THTHTHW",
    "HSHSSSTHSSSTW",
    "SSSTHSTWW",
    "THSHSSSTHST",
    "HSHSSSTHSSSTWW",
    "HTHSWW",
    "HSHSSSTHSSSTWWWWW",
    "SHSTHSTHTWWWWW",
    "HSHSSSTHSSSTWWWWWW",
    "HSHSSSTHSSSTWWWWWWW",
    "SHTHSSTHSS",
    "THSHSSTHWWWWW",
    "THSHSSSTHTW",
    "HSTHTHSSWW",
    "HSTHTHWW",
    "HSTHTHWWW",
    "THTHSSSHSSSWWWWWW",
    "STHSTHSTHSSWW",
    "HSTHTHT",
    "HSTHSSTHSST",
    "HSTHTHTWW",
    "HSTHSSSTHWWW",
    "HSTHTHS",
    "HSTHTHSW",
    "HSTHTHSWW",
    "THTHSSTHSSSTWWWWWW",
    "SSSTHSTWWWW",
    "HSTHTHSWWWWWWW",
    "HSTHTHST",
    "HSTHTHSTWW",
    "HSTHTHSTWWW",
    "HSTHTHSTWWWWW",
    "HSTHTHSTWWWWWW",
    "THTHSSSTHSSSTWWW",
    "THSTHSTHSSSWWWWWWW",
    "THSTHTHT",
    "SSSHSSTWWWWWW",
    "HSTHTHSTWWWWWWW",
    "HSTHTHSSW",
    "SHSTHSSSTHSSSWW",
    "HSHTHSW",
    "HSTHTHSSWWWW",
    "HSTHTHSSWWWWWWW",
    "THSHTHTW",
    "HSTHTHSSTWWW",
    "HSTHTHSSTWWWW",
    "STHTHSSSTHSSTWWWWWWW",
    "HSTHTHSSTWWWWWWW",
    "THSTHSSSHTW",
    "HSTHTHSSS",
    "HSTHTHSSSW",
    "HSTHTHSSSWW",
    "SSTHSS",
    "SSTWWWW",
    "HSTHTHSSSWWWWWWW",
    "HSTHTHSSSTW",
    "HSTHTHSSSTWWW",
    "HSTHSTH",
    "HSTHSTHWWW",
    "HSTHSTHWWWW",
    "THSTHSSSHWWW",
    "HTHSSSTHSSSTWW",
    "HTHSHSTW",
    "HSTHSTHT",
    "SHSTHSTHWW",
    "HSTHSTHTW",
    "HSTHSTHTWW",
    "HSTHSTHTWWW",
    "HSTHSSSTHSSSTWWW",
    "THSTHSSSHSSTWW",
    "HSTHSTHTWWWW",
    "THSHSSSTHSSSTWWWWW",
    "HSTHSTHTWWWWW",
    "THTHSSSHSSTWWWW",
    "THTHSSTHSSWWW",
    "HSTHSTHSW",
    "HSTHSTHSWW",
    "HSTHSTHSWWWWWW",
    "THTHSHSSWWWW",
    "HSTHSTHSWWWWWWW",
    "HSTHSTHST",
    "HSHSTHWWWWWWW",
    "HSTHSTHSTW",
    "THSTHSSSHSWWWWW",
    "HSTHSTHSTWW",
    "HSTHSTHSTWWW",
    "SSSTHTWWWW",
    "HSTHSTHSTWWWWW",
    "HSHTHSSSTWWWWW",
    "THSHSSSTHSWWWW",
    "HSTHSTHSTWWWWWW",
    "HSTHSTHSSW",
    "HSTHSTHSSWW",
    "HSTHSTHSSWWWW",
    "THSHSSSTHTWWWW",
    "SHSTHSSTHSSTWWW",
    "SSTHSSSWWWW",
    "HSTHSTHSSWWWWW",
    "THSTHSSSTHSWWWW",
    "HSTHSTHSSWWWWWW",
    "HSTHSSTHT",
    "THTHSSSTHS",
    "HTHSSSHSSWWWWWWW",
    "THSHSTHSSSTWWWW",
    "HSTHSTHSSTW",
    "THSHSTHSTWWWW",
    "THTHSSSTH",
    "HSTHSTHSSTWWW",
    "THSHSTHWWW",
    "SHSTHSTHSSSW",
    "HTHTHSTWWWWW",
    "HSTHSTHSSTWWWW",
    "SHSTHSSTHST",
    "THSTHSTHSTW",
    "THTHSSSTHSSSWWW",
    "HSTHSTHSSSWW",
    "HSTHSTHSSSWWW",
    "HSHTHSWWW",
    "HSTHSSTHSSSTWWWW",
    "THTHSTHSSTWWWWWWW",
    "THTHSSTHSST",
    "THSHSSSTHWWW",
    "HSTHSTHSSSWWWWW",
    "STHSTHSSTHSSS",
    "HSTHSTHSSSTW",
    "THSTHSSTHW",
    "HSTHSTHSSSTWW",
    "HSTHSTHSSSTWWW",
    "HSTHSTHSSSTWWWWWWW",
    "STHTHSSSTHSS",
    "HSHSTHSSTWWWWW",
    "HSTHSSTH",
    "THSTHSSSTHSTWWWWWWW",
    "HSHSTHSSWWW",
    "HSTHSSTHW",
    "SHSSSWWWW",
    "THTHSSTWWWWW",
    "HSTHSSTHWW",
    "THSTHSSSTHTWWWWWW",
    "THTHSSSHSSWWW",
    "HSTHSSTHWWWWW",
    "STHSTHTHSWW",
    "THTHTHSSWWWWWW",
    "HSTHSSTHWWWWWWW",
    "THSHSSSTHW",
    "HSTHSSSHSSTWWW",
    "HSTHSSTHTWWWW",
    "THTHSSTHSSTWW",
    "THTHSTHSTWW",
    "THTHSSTWWW",
    "HSTHSSTHTWWWWW",
    "HSTHSSTHSTWWWWW",
    "HSTHSSTHTWWWWWW",
    "THTHSSSWWWW",
    "HSTHSSTHSWWWW",
    "HSTHSSTHSWWWWWW",
    "SHTHSSSTHSSSWWWWW",
    "SSSHTWWWWWW",
    "HSTHSSTHSWWWWWWW",
    "HSTHSSSHST",
    "STHTHSSTHSSSTWWWWWWW",
    "THSTHTHST",
    "HTHSSSTHSSTWWWWW",
    "HSTHSSTHSTW",
    "HTHSSTHSSSTWWWWWWW",
    "THSHTHSSTWWWWWW",
    "STHTHSTHSW",
    "HSTHSSTHSTWWW",
    "HSTHSSTHSSTWWW",
    "SHSTHTHSSSTWWW",
    "HSTHSSSHSSTWWWWW",
    "STHSSS",
    "HSTHSSTHSTWWWWWW",
    "HSTHSSTHSTWWWWWWW",
    "HSTHSSTHSS",
    "STHSTHSSTHWWWWW",
    "SHSTHSTHWWWWWW",
    "HSTHSSTHSSWW",
    "HSTHSSTHSSWWWWW",
    "HSTHSSTHSSWWWWWW",
    "THTHSHSTWW",
    "HSTHSSTHSSWWWWWWW",
    "HSTHSSTHSSTW",
    "HSTHSSTHSSTWW",
    "THSTHSSSTHSWWWWW",
    "SSWWWWW",
    "HSTHSSTHSSTWWWW",
    "STHSTHSTHSSSW",
    "HSTHSSTHSSTWWWWW",
    "HSTHSSTHSSTWWWWWW",
    "STHTHSTHSWWWWWW",
    "HSTHSSTHSSS",
    "HSTHSSTHSSSWWW",
    "SSSTHSSWWWW",
    "THTHSTHSTWWWWWW",
    "STHSTHSSSTHSTW",
    "THTHSHSW",
    "SHWWW",
    "HSTHSSTHSSSWWWWW",
    "HSTHSSTHSSST",
    "SHTHSSTHSST",
    "THTHTHWWWWWWW",
    "SSSTHSSSWWWWWWW",
    "HSTHSSTHSSSTWWW",
    "HSTHSSTHSSSTWWWWW",
    "THWW",
    "HSTHTH",
    "HSTHSSSHWW",
    "HSTHSSSHWWW",
    "HSTHSSSHWWWW",
    "SSSHTWWWW",
    "HSTHSSSHWWWWW",
    "HSTHSSSHWWWWWW",
    "THSHSSTHWWWWWWW",
    "HSTHSSSHWWWWWWW",
    "SHTHTHTWWW",
    "THSHSTHSSSW",
    "HSTHSSSHT",
    "HSTHSSSHTW",
    "SSHSWWWWWW",
    "THSSSTWWWWW",
    "HSTHSSSHTWW",
    "HSTHSSSHTWWW",
    "SHTHTHSWWWWWW",
    "HTHSSSHSSTWWWWW",
    "HSTHSSSHS",
    "STHSTHSTH",
    "HSWWWW",
    "HSTHSSSTHWWWWW",
    "HSTHSSSHSWW",
    "HTHSSSWWWW",
    "THTHSSTHSSW",
    "HSTHSSSHSWWW",
    "HSHSSSHS",
    "SSSHSW",
    "HSTHSSSHSWWWW",
    "HSTHSSSHSWWWWWW",
    "HTHSHS",
    "HSTHSSSHSTWWWWW",
    "HSTHSSSHSTWWWWWW",
    "HSTHSSSHSTWWWWWWW",
    "STHSTHSSSTHSSSWWWWWWW",
    "HTHTHSWWWW",
    "HSTHSSSHSSWW",
    "HSTHSSSTHSTWWWW",
    "THTHSSSTHSTWWWWWW",
    "STHSTHSSSTHSSWWW",
    "HSTHSSSHSSWWW",
    "SS",
    "HSTHSSSHSSWWWW",
    "HTHSSSTHSWWW",
    "HTHSTH",
    "HSTHSSSHSST",
    "HSTHSSSHSSTW",
    "SHSTHTHWWWWW",
    "HSHSSTHSSSTWWWWW",
    "HSTHSSSHSSTWWWW",
    "STHSTHTHSSST",
    "SHTHSSSTHSTW",
    "HSTHSSSHSSTWWWWWW",
    "STWW",
    "THTHSSSTHSSST",
    "HSTHSSSHSSS",
    "THSHTHTWWWW",
    "THTHSSSHSSSWWW",
    "HTHSHSSWWWW",
    "HSTHSSSHSSSW",
    "HSTHSSSHSSSWW",
    "HSTHSSSTHWW",
    "STHSSW",
    "HSTHSSSHSSSWWWWW",
    "HSTHSSSHSSSWWWWWW",
    "THTHSSSHSSWWWWWW",
    "HSTHSSSHSSST",
    "SWWWWWW",
    "THTHSHSSTWWWW",
    "SSSHSSSWWW",
    "HSTHSSSHSSSTW",
    "HTHSSTHSSTWWWW",
    "THTHSSSTHT",
    "HSTHSSSHSSSTWWW",
    "HSTHSSSHSSSTWWWW",
    "HSTHSSSHSSSTWWWWW",
    "HSTHSSSHSSSTWWWWWW",
    "THSTHTHSTWWWWW",
    "HSTHSSSTH",
    "HSTHSSSTHWWWW",
    "HSW",
    "THTHSSTHTWW",
    "SHSTHSSSTHW",
    "THTHSSSHTWWWWWWW",
    "THTHSSSTHTWWWWWWW",
    "HSTHSSSTHWWWWWW",
    "STHSWWWWWWW",
    "THSHSSTHSWWWWWWW",
    "HSTHSSSTHT",
    "THSTHSSSHSWWWWWWW",
    "THTHSSTHSSWWWWWW",
    "HSTHSSSTHTWW",
    "HSTHSSTHSTWWWW",
    "HSTHSSSTHTWWWW",
    "THTHSSTHTWWW",
    "SSTHSSWWWWW",
    "HSTHSSSTHTWWWWW",
    "THSHSTHSSSWWWWW",
    "THSHSSTHSSTWWWW",
    "STHTHSTHWWWWW",
    "THSHSTHSSTWW",
    "HSTHSSSTHTWWWWWW",
    "HSTHSSSTHSWWW",
    "HSTHSSSTHSWWWW",
    "HTHSSTHSW",
    "THTHSHTWWWWWW",
    "HTHSSSTHSSSWWWWW",
    "THSHSSTHSSSWWWW",
    "SSTWWW",
    "HSTHSSSTHSWWWWWW",
    "SHSTHTHSSS",
    "HSTHSSSTHST",
    "SHSTHSSTHSSWWWW",
    "HSTHSSSTHSTW",
    "THSTHSSTHWWW",
    "SSSTHS",
    "HSTHSSSTHSTWW",
    "THTHSTHSTW",
    "STHSTHSSTHSWWW",
    "HSTHSSSTHSTWWW",
    "THSSST",
    "HSTHSSSTHSTWWWWW",
    "HSTHSSSTHSTWWWWWW",
    "HSTHSSSTHSTWWWWWWW",
    "THTHSTWWWW",
    "THTHSSSTHSSTWWWWWW",
    "HSTHSSSTHSSW",
    "HTHTWWW",
    "HSTHSSSTHSSWWWWWWW",
    "HSTHSSSTHSST",
    "THSHSSTHTWWWWWWW",
    "HSTHSSSTHSSTW",
    "HTHSHSTWWW",
    "HSTHSSSTHSSTWW",
    "THTHSSSHSSSTWWWWW",
    "THTHTHT",
    "SHTHSSSTHWWWWW",
    "HSTHSSSTHSSTWWWWWW",
    "HSTHSSSTHSSSWW",
    "HSTHSSSTHSSSWWWW",
    "HSTHSSTHSSWWWW",
    "HSTHSSSTHSSSWWWWWW",
    "HSTHSSSTHSSSWWWWWWW",
    "HSTHSSSTHSSSTWWWWW",
    "THTHSTHWWWWWW",
    "HSHSSSTHWWWWWWW",
    "THSHSSTHSWWW",
    "HSTHSSSTHSSSTWWWWWW",
    "SHSTHSTHSSWWWWW",
    "THTHW",
    "SSHTWWWWW",
    "HSHSSTHST",
    "THSHSSSTHWWWWWW",
    "THTHWW",
    "SHSTWWWWWW",
    "THTHSSSTHSSSTW",
    "THTHWWW",
    "THTHWWWW",
    "THTHTW",
    "STHSTHTHSTWWWWW",
    "SHSTHSSTHSSSW",
    "THSTHTHSSTWWW",
    "THTHTWW",
    "THTHTWWWW",
    "HSHSTHSTWWWWWWW",
    "THSHSTHSWW",
    "THSHSTHSSTWWWWW",
    "THTHTWWWWWWW",
    "THTHSTHSSSTWWWWWWW",
    "HTHSWWWW",
    "THTHTH",
    "THTHTHWWW",
    "THTHTHWWWWW",
    "THTHTHTW",
    "THTHTHTWW",
    "THTHTHTWWW",
    "THTHTHTWWWWWW",
    "HTHSHSSSTWWW",
    "THTHSHSWWWWW",
    "THTHTHSW",
    "STHTHTHSTWW",
    "THTHSSSHTWWWWW",
    "THTHTHSWW",
    "THTHTHSWWWW",
    "THTHTHSWWWWW",
    "STHSSWW",
    "THTHTHST",
    "SHSTHSSTHW",
    "THTHTHSTW",
    "HSHSSSHSTWWWW",
    "THTHTHSTWW",
    "THTHTHSTWWW",
    "THTHSTHWWW",
    "STHTHTHSSTWWWWWW",
    "THTHTHSTWWWW",
    "THTHTHSTWWWWW",
    "THTHTHSS",
    "THTHSHSSSTWW",
    "STHTHTHSSTW",
    "THTHTHSSW",
    "THSTHTHSSST",
    "THTHTHSSSTWWWWWWW",
    "SHSTHTHSSSTWW",
    "THTHTHSSWWWWW",
    "THTHTHSST",
    "THTHSTHSSWW",
    "THTHTHSSTW",
    "STHTHSSSTHSSSTWWWW",
    "THSHTHSSWW",
    "THSHSSTHSSSTW",
    "THTHTHSSTWW",
    "THTHTHSSTWWWW",
    "THTHTHSSTWWWWW",
    "THTHTHWW",
    "THTHTHSSSW",
    "HSTHSSSTHSW",
    "THTHSTHSSTWWW",
    "THTHTHSSSWWWW",
    "THSHTHSTWW",
    "HSTW",
    "THTHTHSSSWWWWWWW",
    "HSHSSSTHSWWWWWWW",
    "THTHTHSSSTWWWWW",
    "HSHSSSHSSTWWWW",
    "THTHTHSSSTWWWWWW",
    "THTHSW",
    "HWWWWWW",
    "THTHST",
    "SSSTHSSTWWWWWWW",
    "THTHSWWW",
    "THSTHSTHSSTWWWWWWW",
    "THTHSWWWW",
    "THTHSWWWWW",
    "SHSTHSSSTHWWW",
    "THTHSWWWWWW",
    "SHSTHSSSTHSTWW",
    "THSTHTHSSSTWWWWW",
    "THTHSWWWWWWW",
    "THSHTHSSSTWW",
    "STHSTHSTHSWW",
    "HSTHSTHWWWWWW",
    "THTHSHW",
    "THTHSHWW",
    "THTHSHWWWWW",
    "THTHSHWWWWWW",
    "THTHSHT",
    "SHTHSSTHSSTWWWWW",
    "THTHSHTW",
    "STHSSST",
    "THTHSHTWW",
    "HTHSSSTWWWWW",
    "THTHSHTWWW",
    "HTHSHSSTWWWWW",
    "THTHSHTWWWWW",
    "THTHSHTWWWWWWW",
    "THTHSHS",
    "HTHTHS",
    "THTHTHTWWWW",
    "THTHSHSWWW",
    "THTHSHSWWWW",
    "HTHSHSSSWWWW",
    "THTHSHSSSWWWWWWW",
    "HTHSTHSSTWWWWW",
    "THTHSHST",
    "STHSTHSSTHTWWWW",
    "HTHSSSHSSWWWWWW",
    "THTHSHSTWWWWW",
    "THTHSHSTWWWWWW",
    "SHTHSSTHSSSTWWWWWWW",
    "THTHSHSTWWWWWWW",
    "THSTHSSSHSWWW",
    "THTHSTHSSS",
    "THTHSSTHSSSTWW",
    "THTHSHSSW",
    "THTHSHSSSTWWWWW",
    "SSSTHWWWW",
    "THTHSHSSTWWWWW",
    "SHTHSTHSSSTWWW",
    "SSHSW",
    "SSTHSWWWWW",
    "THTHSHSSS",
    "SHTHTHSSTWWWWW",
    "THTHSHSSSWW",
    "THTHSHSSSWWW",
    "THTHSHSSSWWWW",
    "THSTHSSSHSSST",
    "HTHSSWWWW",
    "THTHSHSSSWWWWW",
    "THSTHSSTHSSTWWWWWWW",
    "THTHSHSSSTW",
    "HSHSSTHSSSTW",
    "SHWW",
    "THTHSHSSSTWWW",
    "HSHTHSSSWWW",
    "THTHSHSSSTWWWWWW",
    "STHTHSSTHWWW",
    "THSHSSSTHSTWWWW",
    "THTHSHSSSTWWWWWWW",
    "H",
    "THTHSTW",
    "THTHSTWWW",
    "THTHSTWWWWWW",
    "THTHSTWWWWWWW",
    "THTHSTHW",
    "THTHSTHT",
    "THTHSTHTWWW",
    "STHTHTHSSSWWW",
    "SHSTHSTHT",
    "THTHSTHTWWWW",
    "THTHSSSHWWWWWWW",
    "THTHSTHTWWWWW",
    "STHSTHSSSTHSW",
    "THTHSTHTWWWWWW",
    "STHS",
    "THTHSTHTWWWWWWW",
    "THWWWWW",
    "THTHSTHS",
    "THTHSTHSW",
    "THSHTHSSWWW",
    "THTHSTHSWW",
    "THSTHSSSTHSSSTWWWW",
    "THSTHSSTHSSSWWWWWW",
    "THTHSTHSWWW",
    "STHSTHSTHSSTWWWW",
    "THTHSTHSWWWWW",
    "THTHSTHSWWWWWWW",
    "THSHTHSSTW",
    "THSHSSTHSSSWW",
    "THTHSTHST",
    "SHSSTWW",
    "THTHSSSTHSSTWWWW",
    "SHSTHTHSSWW",
    "THTHSTHSTWWWW",
    "SSSTWWWWWW",
    "THTHSTHSS",
    "THSTHSTHSSTWWWWWW",
    "THTHSTHSSW",
    "THTHSSSTWWWW",
    "THTHSTHSSWWWW",
    "THTHSTHSSWWWWW",
    "STWWWWWW",
    "THTHSTHSSWWWWWW",
    "THTHSTHSST",
    "THSHSSTHWWWWWW",
    "THTHSTHSSTW",
    "SHSTHSSSTHSSTWW",
    "HTHTHWW",
    "THTHSTHSSTWWWWWW",
    "HSHSTHSSTWWWW",
    "THTHSTHSSSW",
    "SHSTHSTHTWWWWWW",
    "HTHSHSSTWWWWWW",
    "THTWWWW",
    "HSTHSTHSSST",
    "HSTHSSSTHSWW",
    "THTHSTHSSSWWWWW",
    "SHTHSSSTHSSWWWWWWW",
    "THTHSTHSSSWWWWWW",
    "THTHSTHSSSWWWWWWW",
    "THTHSSTW",
    "THTHSSTHSTW",
    "THTHSTHSSSTWW",
    "HSTHSSTHWWW",
    "THTHSSSTHSWWWWW",
    "SHSTHSTHSSWWW",
    "THTHSTHSSSTWWW",
    "THTHSTHSSSTWWWWWW",
    "THSTHSSSHSTWW",
    "SSSTHSSTWWW",
    "HSTHSSTHSWW",
    "THTHSSSTHSTWWWW",
    "THTHSSWW",
    "THTHSSWWW",
    "THTHSSTWW",
    "HSHSSTHSSSWWWWWW",
    "THTHSSTWWWWWW",
    "THTHSSTWWWW",
    "THTHSSTHWW",
    "STHSTHSSSTHSSST",
    "SHSTHSSTHSSTWWWWWWW",
    "THTHSSTHT",
    "STHSTHTHSSWW",
    "THTHSSSHWW",
    "STHTHSTHSSW",
    "HTHSSSHSWWWWWWW",
    "SHSSTWWWWW",
    "THTHSSTHTW",
    "THTHSSTHTWWWWW",
    "THTHSSTHS",
    "THTHSSTHSW",
    "THTHSSTHSWW",
    "THTHSSTHSWWW",
    "THTHSSTHSWWWWW",
    "THTHSSTHSWWWWWW",
    "HSTHTHSTWWWW",
    "THTHSSTHSTWW",
    "THSHTHSWWW",
    "HSHSSSTHSSWWWWWWW",
    "THSHSTH",
    "HTHSSSTHSWWWWW",
    "HSSTWWWWW",
    "THTHSSTHSSWWWW",
    "SSSHSSWWWWWW",
    "THTHSSTHSSWWWWW",
    "THTHSSTHSSTW",
    "THTHSSTHSSTWWWW",
    "THTHSSTHSSTWWWWW",
    "STHSTHSSSTHSSWWWWWW",
    "STHTHTHSTWWWWW",
    "SHSTHSTHSSTW",
    "THTHSSTHSSTWWWWWWW",
    "THTHSSTHSSSW",
    "THTHSSTHSSSWW",
    "HSTHSTHSSWWWWWWW",
    "THSHSSSTHSSWWWWWW",
    "THTHSSTHSTWWWWWWW",
    "THTHSSTHSSST",
    "THTHSSTHSSSTWWWWW",
    "THTHSSTHSSSTWWWWWWW",
    "HTHSSSHTWWWWWWW",
    "HTHSHSSW",
    "THTHSSS",
    "HTHSSSHSTWWWW",
    "THSHSSTHSSSWWWWW",
    "THTHSSSW",
    "THTHSSSWW",
    "THTHSSSWWW",
    "THSHTHSSSW",
    "SHTHSTHST",
    "SSSHSSWW",
    "THTHSSSWWWWW",
    "HSHTHTWW",
    "HTHSSSHW",
    "THTHSSSWWWWWWW",
    "THTHSSSH",
    "THTHSSSHWWW",
    "STHTHTHSSW",
    "SHTHTHSSTWWW",
    "HSHSSSHW",
    "THSHTHWWWW",
    "THTHSSSHWWWW",
    "STHTHSSTHTWWWWW",
    "HSTHTHSWWWW",
    "THTHSSSHT",
    "HSTHSSSHSSTWWWWWWW",
    "THTHSSSHTW",
    "STHTHSSTHSSSTWWWW",
    "THSHTHSWWWWW",
    "HTHSSSHSSTWWWWWW",
    "THTHSSSHTWWWWWW",
    "SSHSSSTW",
    "THTHSSSHSW",
    "THTHSSSHSWWW",
    "THTHSSSHSWWWW",
    "THTHSSSHSWWWWW",
    "THTHSSSHSWWWWWW",
    "THTHSSSHSWWWWWWW",
    "THTHSSSHSTW",
    "THSSSW",
    "THTHSSSHSTWWW",
    "HTHTHST",
    "THTHSSSHSTWWWW",
    "THTHSSSTHTWWWWW",
    "STHSTHSSTH",
    "STHSTHTHS",
    "THTHSSSHSS",
    "HSHSSSTHSTWW",
    "HTHSHSSSTWW",
    "THTHSSSHSSWW",
    "STHTHSTHSTWWW",
    "THSHSTHSSS",
    "THTHSSSHSSWWWW",
    "THSTHSSSTHSSWWWWWWW",
    "THTHSSSHSSWWWWW",
    "THTHSSSHSSWWWWWWW",
    "SHSTHSSSTHSWWWWW",
    "HSTHTHTWWW",
    "THSHSTHSSSWWW",
    "THTHSSSHSSTWW",
    "HSSS",
    "THTHSSSHSSTWWWWW",
    "THTHSSSHSSTWWWWWW",
    "SSTHSSTWWWWW",
    "THTHSSSHSSTWWWWWWW",
    "HTHSHSSWWWWWW",
    "THTHSTWW",
    "THTHSSSHSSS",
    "THTHSSSHSSSW",
    "HSHTHSSWWWWW",
    "THTHSSSHSSSWWWWWWW",
    "SHSTHTHSSSWWW",
    "THTHSSSHS",
    "THTHSSSTHSTWWWWWWW",
    "THSHSTHSS",
    "SHSTHTHSSSTW",
    "THTHTHSSSWW",
    "SSHSSSWWWWWWW",
    "HTHSSSTHTWWWWW",
    "HSTHSSSTHTW",
    "THTHSSSHSSST",
    "SHSTHSTHSWWWWWWW",
    "THTHSSSHSSSTWW",
    "THTHSSSHSSSTWWW",
    "THTHSSSTW",
    "HTHSTHSSWWWW",
    "SSSTHSSSTWWWW",
    "THTHSSSTWWWWWWW",
    "STHTHSTHSWWWWW",
    "HSHTHST",
    "THTHSSSHSSSTWWWW",
    "THTHSSSTHW",
    "SSHWWWWWWW",
    "THTHSSSTHWW",
    "STHTHSSSTHSSS",
    "STHSST",
    "THTHSSSTHWWW",
    "SHTHTHSSSWWW",
    "THTHSSSTHWWWW",
    "THTHSSSTHWWWWWW",
    "THTHSSSTHTWWWW",
    "HSTHSTHS",
    "THTHSSSTHSWWWWWW",
    "STHSTHSTHWW",
    "THTHSSSTHSWWWWWWW",
    "STHSTHSTHSSTWWW",
    "HSTHTHWWWWWWW",
    "THTHSSSTHST",
    "SHSTHSTHW",
    "THTHSSSTHSTW",
    "THTHSSSTHSTWWWWW",
    "THTHSSSTHSS",
    "SHSTHSTHTWW",
    "THTWWWWWWW",
    "THTHSSSTHSSWW",
    "THTHSSSTHSSWWW",
    "THTHSSSTHSSWWWW",
    "THTHSSSTHSSWWWWWWW",
    "THTHSSSTHSSSW",
    "STHTHSSTHSSSWWWWWWW",
    "THTHSSSTHSSSWWWWW",
    "HSHSTHTWWW",
    "THTHSSSTHSSSTWW",
    "THTHSSSTHSSSTWWWW",
    "SHTHTHSTWWWWWWW",
    "HTHSSSHSSSWWWWWW",
    "THTHSSSTHSSSTWWWWWW",
    "THTHSSSTHSSSTWWWWWWW",
    "THSHTH",
    "THSHTHW",
    "THSHTHWW",
    "THSHTHWWWWWW",
    "THSSSTWW",
    "THSHTHWWWWWWW",
    "HSHSSSTHSSSWW",
    "THSHTHT",
    "STHSTHSSSTHSSSTWWWW",
    "THSHTHTWWW",
    "SHTHTHTWW",
    "HSTHSTHSSSTWWWW",
    "THSHTHTWWWWW",
    "THSHTHSSSWW",
    "THSHTHTWWWWWWW",
    "STHSSSWW",
    "SSHSWWW",
    "THSHTHS",
    "STHSTHSSSTHSSSTWW",
    "STHTHSSTHSSSTWWWWWW",
    "THSHTHSWWWW",
    "SSSHSSSTWW",
    "THSHTHSWWWWWW",
    "THSHTHSWWWWWWW",
    "STHTHSSTHSSTWWWWWW",
    "THSHTHST",
    "STHSSSTWWWWW",
    "THSHSTHSTWW",
    "SHTHSTHSSW",
    "SSSHSTW",
    "THSHTHSTW",
    "THSTHSTHTWW",
    "THSHTHSTWWWWW",
    "THSHTHSTWWWWWWW",
    "STHSTHTHSWWWWW",
    "THSHTHSSWWWW",
    "THSHTHSSWWWWW",
    "SSTHSSSTWWWW",
    "THSHTHSSWWWWWWW",
    "THSHTHSSTWWW",
    "THSHTHSSTWWWWWWW",
    "THSHTHSSS",
    "THSHTHSSSWWWW",
    "THTHSSSTWW",
    "THSHTHSSSWWWWWW",
    "THSHTHSSSWWWWWWW",
    "THSHTHSSSTW",
    "HSHTHSSST",
    "THSHTHSSSTWWW",
    "STHTHSSSTHSSSTWW",
    "THSHTHSSSTWWWWW",
    "STHTHTHWWWWWW",
    "THSHSTHWWWWW",
    "THSTHSTHTWWWWWW",
    "THSHSTHWWWWWWW",
    "THSHSTHT",
    "THSHSTHTW",
    "THSHSTHTWWW",
    "STHTWW",
    "THSHSTHTWWWWWW",
    "THSHSTHS",
    "STHTHSTHSSSWWWWWW",
    "THSHSTHSTW",
    "HTHSSSTHWW",
    "THSHSTHSTWWW",
    "HSTHSTHSSSWWWW",
    "THSHSTHSSST",
    "HSTHSTHSSTWW",
    "HSTHSSSHSS",
    "THSHSTHSTWWWWW",
    "THTHSSSHSTWWWWWW",
    "THSHSTHSSWWW",
    "HTHSSTHSSWWWWW",
    "THSHSTHSSWWWW",
    "THSHSTHSSWWWWW",
    "THSHSTHSSWWWWWW",
    "THSHSTHSSWWWWWWW",
    "THSHSTHSST",
    "SHSTHSSSTHTWWW",
    "THSHSTHSSTWWWW",
    "THSHSTHSSTWWWWWW",
    "HTHSS",
    "THSHSTHSSTWWWWWWW",
    "THSHSTHSSSWW",
    "THSHSTHSSSWWWWWW",
    "THSTHSTHSWWWWWWW",
    "THSHSSSTHSTWWWWWWW",
    "THSHSTHSSSTW",
    "THSHSTHSSSTWWWWW",
    "THTHSTHSSWWWWWWW",
    "THSHSSTH",
    "THSTHSTHSSTWWW",
    "HSTHTHSWWW",
    "THSHSSTHW",
    "HTHSTHSSWWWWW",
    "THSHSSTHWW",
    "HTHSSSWWWWW",
    "HTHSSW",
    "HSHSSSHSSSTWWW",
    "HSHSSSTHWW",
    "THSHSSTHWWWW",
    "HSHSSSHSTWWWWW",
    "THSHSSTHTWW",
    "STHSSSTWW",
    "THSHSSTHTWWWW",
    "SSTHT",
    "THSHSSTHSW",
    "THSHSSTHSWW",
    "THSHSSTHSWWWW",
    "HSSSTW",
    "THSHSSTHSWWWWW",
    "THSHSSTHSWWWWWW",
    "SHTHSTHSSTWWWWWW",
    "SHTWWWW",
    "THSHSSTHST",
    "HTHSSTHTW",
    "HTHSSSTHTWWW",
    "THSHSSTHSTWWWW",
    "THSHSSTHSTWWWWWW",
    "THSHSSTHSTWWWWWWW",
    "HSTHTHSSST",
    "THSHSSTHSSW",
    "THSHSSTHSSWWWWW",
    "SHSW",
    "THSHSSTHSSTW",
    "THTHSTHWW",
    "TW",
    "THSHSSTHSSTWW",
    "THSHSSTHSSTWWW",
    "STHTHTHWWWWW",
    "THSHSSTHSSTWWWWW",
    "THSHSSTHSSTWWWWWW",
    "THSHSSTHSSTWWWWWWW",
    "SHSTHSSTHWW",
    "THSHSSTHSSSWWWWWW",
    "THSHSSTHSSSTWWWWW",
    "SHSTHSSSTHSWWWWWWW",
    "HSTHSSTHSSSWWWWWWW",
    "THSHTHTWWWWWW",
    "THSHSSTHSSSTWWWWWWW",
    "THSHSTHSSW",
    "THSHSSSTHWWWW",
    "THSHSSSTHTWW",
    "STHTHSSTHSSWWWWWWW",
    "SHSTHSSSTHSSSTWWWWWW",
    "SHSTHSSTHSST",
    "SHTHSSTHSSSWWWW",
    "HSTHSSTHSSSTWWWWWWW",
    "THSHSSSTHTWWWWWWW",
    "THSHSSSTHSW",
    "SHWWWW",
    "THSHSSSTHSWW",
    "HSTHSSTHSSSTWW",
    "THSHSSSTHSWWW",
    "SSHSSSWWWWWW",
    "SSTHSTW",
    "THSHSSSTHSWWWWW",
    "THSHSSSTHSWWWWWW",
    "STHSTHSSTHSSTWWWW",
    "THSHSSSTHSWWWWWWW",
    "THSTHTHSSSWWWW",
    "THSHSSSTHSTWW",
    "THSHSSSTHSTWWWWWW",
    "HSHSTHTWWWWWW",
    "THSHSSSTHSSWW",
    "THSHSSSTHSS",
    "THTHSSSTHSSTWWW",
    "THSHSSSTHSSW",
    "SHSWWWWW",
    "THSHSSSTHSSWWWWW",
    "THSHSSSTHSSWWWWWWW",
    "THSHSSSTHSSTW",
    "THSTHSSTHSSWWW",
    "THSHSSSTHSSTWWW",
    "THTHSSSHTWW",
    "THSHSSSTHSSTWWWWW",
    "THSHSSSTHSSTWWWWWW",
    "THSHSSSTHSSTWWWWWWW"};

// clang-format on

inline domega_matrix_table_t load_s3_table() {
    using namespace std;
    domega_matrix_table_t s3_table;

    for (int i = 0; i < S3_TABLE_SIZE; ++i) {
        auto k = S3_TABLE_KEYS[i];
        auto v = S3_TABLE_VALUES[i];
        DOmegaMatrix mat(ZOmega(k[0], k[1], k[2], k[3]),
                         ZOmega(k[4], k[5], k[6], k[7]), k[8], k[9]);
        s3_table[mat] = v;
    }
    return s3_table;
}

} // namespace grid_synth
} // namespace staq

#endif // GRID_SYNTH_S3_TABLE_HPP_
